#!/bin/sh

# Find out where the script is located

if [ -e /bin/sed -a -x /bin/sed ]
then
    SCRIPT_BASE=$(echo $0 | sed s/autoboot.sh//)
else
    for i in 1 2 3 4 5 6 7 8 9; do
	FIELD=$(echo $0 | cut -d / -f $i)
	if [ "$FIELD" = "autoboot.sh" ]
	then
	    i=$[$i-1]
	    SCRIPT_BASE=$(echo $0 | cut -d / -f -$i)/
	    break
	fi
    done
fi

cd $SCRIPT_BASE

# Make path absolute

SCRIPT_BASE=$(pwd)

. ./scripts/boot_functions

# Read in fancy output functions
. ./scripts/output.sh

export PATH=$PATH:$BINARY_PATH

# Turn on backlight

/usr/bin/fbpwm -b 1 -i 50

##############################################################################

check_exist $MD5SUM
if [ "$?" -eq 0 ] 
then
	begin_operation "Check files agains checksums"

	cd files

	$MD5SUM --check md5sum
	test_result
else
	mit_echo "No md5sum application found!"
        touch /mnt/sandisk/upgrade/apps/softwareupgrade/installFailed
	exit 255
fi

##############################################################################

if test -e $BOOTLDR_FILE; 
then
	begin_operation "Update bootloader"
	sgvt_ldrupd -V $BOOTLDR_FILE
	if [ "$?" -eq 0 ]
	then
		report_uptodate
	else
		sgvt_ldrupd $BOOTLDR_FILE
		test_result
	fi
fi

##############################################################################

if test -e $SCRIPT_FILE; 
then
	begin_operation "Update boot script"
	sgvt_ldrupd -V -s $SCRIPT_FILE
	if [ "$?" -eq 0 ]
	then
		report_uptodate
	else
		sgvt_ldrupd -s $SCRIPT_FILE
		test_result
	fi
fi

##############################################################################

if test -e $FPGA_IMAGE; 
then
	begin_operation "Update FPGA binary"
	sgvt_ldrupd -f -V $FPGA_IMAGE
	if [ "$?" -eq 0 ] 
	then
		report_uptodate
	else
		sgvt_ldrupd -f $FPGA_IMAGE
		test_result
	fi
fi

##############################################################################

if [ -e $FILE_PATH/$AVR_HEX ]; 
then
	begin_operation "Update keyboard controller"

        . ../scripts/dude.sh

	if verify_dude
	then
	    report_uptodate
	else
	    update_dude
	    test_result
	fi
	
	cd ..
fi

##############################################################################

if test -e  $FILE_PATH/$KERNEL;
then
	begin_operation "Update kernel"
	diff_files $DIFF $KERNEL_PART $FILE_PATH/$KERNEL
	if [ "$?" -eq 0 ] 
	then
		report_uptodate
	else
		update_flash $FILE_PATH/$KERNEL $KERNEL_PART
		test_result
	fi
fi

##############################################################################

if test -e $FILE_PATH/$LOGO
then
	begin_operation "Update boot logo"
	diff_files $DIFF $LOGO_PART $FILE_PATH/$LOGO
	if [ "$?" -eq 0 ] 
	then
		report_uptodate
	else
		update_flash $FILE_PATH/$LOGO $LOGO_PART
		test_result
	fi
fi

##############################################################################

begin_operation "Detect flash memory size"
rmmod sgvt_nand
rmmod sgvt_part
modprobe sgvt_nand
test_failure
FLASH_SIZE=$( mtd_size /dev/mtd3 )
rmmod sgvt_nand
test_failure
report_success

##############################################################################

echo $MAKE_PART < $FILE_PATH/$PART_CMD.$FLASH_SIZE

begin_operation "Create partitions"

if test -e $FILE_PATH/$PART_CMD.$FLASH_SIZE
then
	$MAKE_PART < $FILE_PATH/$PART_CMD.$FLASH_SIZE
	test_failure
else
	$MAKE_PART < $FILE_PATH/$PART_CMD
	test_failure
fi

report_success

##############################################################################

        begin_operation "Install root file system"

        for i in $MODULES;
        do
                modprobe $i
                test_failure
        done

        $ERASE_NAND $ROOT_WRITE_DEV
        test_failure


##############################################################################

begin_operation "Mount root file system"
if [ ! -e $MOUNT_POINT ]
then
	mount none / -o remount
	test_failure
	mkdir $MOUNT_POINT
	test_failure
fi
mount $MOUNT_OPTS $ROOT_MOUNT_DEV $MOUNT_POINT
test_failure

cd $MOUNT_POINT
test_result

##############################################################################

begin_operation "Install root file system image"   
tar xzf $FILE_PATH/rootfs.tar.gz
test_result 

##############################################################################

if test -e $FILE_PATH/$DEV_SCRIPT; 
then
	begin_operation "Run device creation script"
	$FILE_PATH/$DEV_SCRIPT
	test_result
fi

if test -e $FILE_PATH/$LINK_SCRIPT; 
then
	begin_operation "Run symbolic link creation script"
	$FILE_PATH/$LINK_SCRIPT
	test_result
fi

##############################################################################

if [ -e $MOUNT_POINT/lib/modules -a -e $MOUNT_POINT/sbin/depmod ]
then
	begin_operation "Run depmod"
	$MOUNT_POINT/sbin/depmod -q -b $MOUNT_POINT
	test_result
fi

##############################################################################

begin_operation "Unmount root file system"

cd ..
test_failure

umount $MOUNT_POINT
test_result

##############################################################################

if test -e $OEM_PATH/$OEM_SCRIPT
then
	$OEM_PATH/$OEM_SCRIPT
fi

##############################################################################

#$BEEP
#$BEEP
touch /mnt/sandisk/upgrade/apps/softwareupgrade/installComplete

mit_echo "Finished."
