##############################################################################
#
# vc/src/tcl/commonobjects.tcl
#
# Defines various common display objects (i.e. Tk frame) objects that can be 
# packed into various screens, e.g. the Navigation frame, the Configuration 
# frame, the Tractor frame, the Calibration frame, etc.
#
# 20020327 MLE - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################


#=============================================================================
# .fwait
#
# A wait screen with message. Has the ability to unpack the current screen, 
# display a wait screen with a message, and repack the original screen
#=============================================================================
namespace eval Wait { 
    global gamma_value

    # Create the wait frame
    frame .fwait
    image create photo gifImage \
            -gamma $gamma_value -format gif \
            -file ${tclPath}gif/AFL_RGB4.7x3.4.gif
    label .fwait.picture -image gifImage
    pack .fwait.picture -side top -fill both -expand yes
    label .fwait.l1 -font $huge_font
    pack .fwait.l1 -side top -fill both -expand yes

    proc actStart {msg__} {
        pack forget .ftop
        .fwait.l1 configure -text $msg__
        pack .fwait -fill both -expand yes
    }

    proc actStop {} {
        pack forget .fwait
        pack .ftop -fill both -expand yes
    }

    proc actConfigLanguage {standby} {
        .fwait.l1 configure -text $standby
    }
}

#=============================================================================
# .images
# color button size(pixels):
# regular  110*110
# btnBlueM 95*50 designed for Change buttons in baseDisplay fieldDisplay
#                jobDisplay implementDisplay tractorDisplay
# btnBlueM2 90*70 for fieldBoundry buttons
# btnBlueM3 120*50 for job headingSelect buttons
# btnBlueM4 160*60 for keyBoard Spacebar
# btnBlueM5 110*60 for autofarm setup and alarm buttons
# btnBlueM6 95*35 for autofarm setup screen reset buttons
# btnKhakiM4 80*60 for keyBoard Cancel button
# btnGreenM4 80*60 for keyboard Enter button
# btnBlueS 50*50 for Slider + and - buttons
# btnBlueS2 60*60 for numpad buttons
# btnBlueS3 50*60 for keyBoard number buttons and letter buttons
# btnBlueS4 65*60 for keyBoard Bksp button
#=============================================================================
namespace eval Images { 
    global gamma_value
    puts stdout "TCL gamma_value $gamma_value"
    image create photo smallNovariantLogo -gamma $gamma_value -format gif \
        -file ${tclPath}gif/smallNovariantLogo.gif
    image create photo smallAutoFarmLogo -gamma $gamma_value -format gif \
        -file ${tclPath}gif/smallAutoFarmLogo.gif
    image create photo smallVirtualRailLogo -gamma $gamma_value -format gif \
        -file ${tclPath}gif/smallVirtualRailLogo.gif
    image create photo smallSTOP -gamma $gamma_value -format gif -file ${tclPath}gif/smallSTOP.gif
    image create photo mediumSTOP \
                 -gamma $gamma_value -format gif -file ${tclPath}gif/mediumSTOP.gif
    image create photo bigSTOP -gamma $gamma_value -format gif -file ${tclPath}gif/bigSTOP.gif
    image create photo smallGO -gamma $gamma_value -format gif -file ${tclPath}gif/smallGO.gif
    image create photo mediumGO -gamma $gamma_value -format gif -file ${tclPath}gif/mediumGO.gif
    image create photo bigGO -gamma $gamma_value -format gif -file ${tclPath}gif/bigGO.gif
    image create photo btnRED -gamma $gamma_value -format gif -file ${tclPath}gif/btn_red.gif
    image create photo btnYELLOW -gamma $gamma_value -format gif \
                 -file ${tclPath}gif/btn_yellow.gif
    image create photo btnGREEN -gamma $gamma_value -format gif -file ${tclPath}gif/btn_green.gif
    image create photo btnGREY -gamma $gamma_value -format gif -file ${tclPath}gif/btn_grey.gif
    image create bitmap bigLNUDGE -foreground $color3 \
            -file ${tclPath}gif/largeleftarrow.xbm
    image create bitmap smallLNUDGE -foreground $color3 \
            -file ${tclPath}gif/leftarrow.xbm
    image create bitmap bigRNUDGE -foreground $color3 \
            -file ${tclPath}gif/largerightarrow.xbm
    image create bitmap smallRNUDGE -foreground $color3 \
            -file ${tclPath}gif/rightarrow.xbm
    image create bitmap circle -foreground $color6 \
            -file ${tclPath}gif/circle.xbm
    image create photo zoomin -format gif -file ${tclPath}gif/zoomin.gif
    image create photo zoomout -format gif -file ${tclPath}gif/zoomout.gif
            
    image create photo checkboxD1 \
            -format gif \
            -file ${tclPath}gif/btnGreenSliderUChecked.gif

    image create photo checkboxU1 \
            -format gif \
            -file ${tclPath}gif/btnGreenSliderDChecked.gif

    # images for "snazzy buttons"
    image create photo btnAcceptU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnAcceptU.gif
    image create photo btnAcceptD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnAcceptD.gif
    image create photo btnRejectU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRejectU.gif
    image create photo btnRejectD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRejectD.gif
    image create photo btnExitKhakiU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiU.gif
    image create photo btnExitKhakiD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiD.gif
    image create photo btnExitRedU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitRedU.gif
    image create photo btnExitRedD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitRedD.gif
        
    image create photo btnBlueMS4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS4U.gif
    image create photo btnBlueMS4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS4D.gif

    image create photo btnGreenU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenU.gif
    image create photo btnGreenD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenD.gif
    image create photo btnGreenM4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM4U.gif
    image create photo btnGreenM4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM4D.gif
    image create photo btnGreenM5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM5U.gif
    image create photo btnGreenM5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM5D.gif
    image create photo btnGrayU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGrayU.gif
    image create photo btnGrayD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGrayD.gif
    image create photo btnYellowU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnYellowU.gif
    image create photo btnYellowD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnYellowD.gif
    image create photo btnRedU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedU.gif
    image create photo btnRedD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedD.gif
    image create photo btnRedM2U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedM2U.gif
    image create photo btnRedM2D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedM2D.gif
    image create photo btnRedForSatVUU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedForSatVUU.gif
    image create photo btnRedForSatVUD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedForSatVUD.gif
    image create photo btnBlueForSatVUU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueForSatVUU.gif
    image create photo btnBlueForSatVUD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueForSatVUD.gif
    image create photo btnBlueU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueU.gif
    image create photo btnBlueD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueD.gif
    image create photo btnKhakiU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiU.gif
    image create photo btnKhakiD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiD.gif
    image create photo btnKhakiM4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM4U.gif
    image create photo btnKhakiM4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM4D.gif
    image create photo btnKhakiForSatVUU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiForSatVUU.gif
    image create photo btnKhakiForSatVUD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiForSatVUD.gif
       
    image create photo btnChangeConfigD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnChangeConfigD.gif
    image create photo btnChangeConfigU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnChangeConfigU.gif
    image create photo btnDriverD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverD.gif
    image create photo btnDriverU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverU.gif
    image create photo btnSupervisorD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorD.gif
    image create photo btnSupervisorU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorU.gif
    image create photo btnMeasureD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnMeasureD.gif
    image create photo btnMeasureU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnMeasureU.gif

    image create photo btnStartSurveyD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnStartSurveyD.gif
    image create photo btnStartSurveyU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnStartSurveyU.gif
    image create photo btnSkipSurveyD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSkipSurveyD.gif       
    image create photo btnSkipSurveyU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSkipSurveyU.gif
        
    image create photo btnBlueMU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMU.gif
    image create photo btnBlueMD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMD.gif
    image create photo btnBlueM2U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM2U.gif
    image create photo btnBlueM2D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM2D.gif
    image create photo btnBlueM3U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM3U.gif
    image create photo btnBlueM3D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM3D.gif
    image create photo btnBlueM4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM4U.gif
    image create photo btnBlueM4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM4D.gif

    image create photo btnBlueM5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM5U.gif
    image create photo btnBlueM5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM5D.gif

    image create photo btnBlueM6U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM6U.gif
    image create photo btnBlueM6D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM6D.gif
        
    image create photo btnBlueM7U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM7U.gif
    image create photo btnBlueM7D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM7D.gif    
    image create photo btnBlueSquareUp -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquareUp.gif
    image create photo btnBlueSquareDown -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquareUp.gif
    image create photo btnBlueSquare2Up -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquare2Up.gif
    image create photo btnBlueSquare2Down -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquare2Up.gif
    image create photo btnExitKhakiSquareUp -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiSquareUp.gif
    image create photo btnExitKhakiSquareDown -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiSquareUp.gif

    image create photo btnBlueSU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSU.gif
    image create photo btnBlueSD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSD.gif
    image create photo btnBlueS2U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS2U.gif
    image create photo btnBlueS2D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS2D.gif
    image create photo btnBlueS3U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS3U.gif
    image create photo btnBlueS3D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS3D.gif
    image create photo btnBlueS4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS4U.gif
    image create photo btnBlueS4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS4D.gif
    image create photo btnDriverU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverU.gif
    image create photo btnDriverD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverD.gif
    image create photo btnSupervisorU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorU.gif
    image create photo btnSupervisorD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorD.gif

    image create photo btnSmallStopU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallStopU.gif
    image create photo btnSmallStopD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallStopD.gif
    image create photo btnBigStopU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigStopU.gif
    image create photo btnBigStopD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigStopD.gif
    image create photo btnBigGoU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigGoU.gif
    image create photo btnBigGoD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigGoD.gif

    image create photo btnBigLeftNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigLeftNudgeU.gif
    image create photo btnBigLeftNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigLeftNudgeD.gif

    image create photo btnBigRightNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigRightNudgeU.gif
    image create photo btnBigRightNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigRightNudgeD.gif

    image create photo btnSmallLeftNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallLeftNudgeU.gif
    image create photo btnSmallLeftNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallLeftNudgeD.gif

    image create photo btnSmallRightNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallRightNudgeU.gif
    image create photo btnSmallRightNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallRightNudgeD.gif

    image create photo btnParallelRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnParallelRowsU.gif
    image create photo btnParallelRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnParallelRowsD.gif

    image create photo btnFanRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnFanRowsU.gif
    image create photo btnFanRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnFanRowsD.gif
    image create photo btnBoxRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBoxRowsU.gif
    image create photo btnBoxRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBoxRowsD.gif

    image create photo btnCircleRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCircleRowsU.gif
    image create photo btnCircleRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCircleRowsD.gif
    image create photo btnCurveRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCurveRowsU.gif
    image create photo btnCurveRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCurveRowsD.gif
    image create photo btnABPathU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnABPathU.gif
    image create photo btnABPathD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnABPathD.gif
    image create photo btnRepeatJobU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRepeatJobU.gif
    image create photo btnRepeatJobD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRepeatJobU.gif

    image create photo gantryLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryLeft.gif
    image create photo gantryRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryRight.gif
    image create photo gantryJoystickLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryJoystickLeft.gif
    image create photo gantryJoystickRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryJoystickRight.gif
    image create photo gantryStraight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryStraight.gif
}

#=============================================================================
#
# The AutoFarm snazzy buttons. 
# A bunch of snzaay buttons that can be placed into any screen.
#=============================================================================
if {1} {          ;# {MLE-031107: Testing a ?faster? version of SnazzyButtons}
namespace eval SnazzyButton {

    proc create { me imgU imgD yFrac command {text " "} } {
        global btn_font color6

        set xMax [expr [image width  $imgU] - 20]
        set yMax [expr [image height $imgU] - 9]

        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }

        button $me -text "$text" -font $btn_font \
            -compound center \
            -command $command \
            -width $xMax -height $yMax \
            -image $imgU \
            -relief flat -borderwidth 0 \
            -background $color6 -activebackground $color6
            ;#-background pink -activebackground pink
    }

    proc setState { me state } {
	$me configure -state $state
    }

    proc setText { me text yFrac } {
        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n$text"
        }
        $me configure -text $text
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [expr [image width  $imgU] - 19]
        set yMax [expr [image height $imgU] - 9]
        $me configure -image $imgU -width $xMax -height $yMax
    }
}

} else {

namespace eval SnazzyButton {

    proc create { me imgU imgD yFrac command {text ""} } {
        global btn_font

        set xMax [image width  $imgU]
        set yMax [image height $imgU]
        set yTxt [expr $yMax*$yFrac]

        canvas $me -width $xMax -height $yMax
        $me create image    0    0 -anchor nw -image $imgU -tags imageU
        $me create image -200 -200 -anchor nw -image $imgD -tags imageD
        $me create text [expr $xMax/2] $yTxt \
                   -text $text -font $btn_font -justify center -tags text
        # create an off-screen text item to store the command
        $me create text 0 -50 -text $command -tags command            
        
        bind $me <ButtonPress-1> {
           %W coords imageD    0    0
           %W coords imageU -200 -200
           set txtCoords [%W coords text]
           %W coords text [expr [lindex $txtCoords 0] + 2] \
                          [expr [lindex $txtCoords 1] + 2]
        }
        
        bind $me <ButtonRelease-1> {
           set xMax [%W cget -width]
           set yMax [%W cget -height]
           %W coords imageD -200 -200
           %W coords imageU    0    0
           set txtCoords [%W coords text]
           %W coords text [expr [lindex $txtCoords 0] - 2] \
                          [expr [lindex $txtCoords 1] - 2]
           if {0 < %x && %x < $xMax && 
               0 < %y && %y < $yMax} {
              uplevel #0 [%W itemcget command -text]
           }
        }
    }

    proc setText { me text yFrac } {
        $me itemconfigure text -text $text

        set xMax [$me cget -width]
        set yMax [$me cget -height]
        $me coords text [expr $xMax/2] [expr $yMax*$yFrac]
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [image width  $imgU]
        set yMax [image height $imgU]
        $me configure -width  $xMax
        $me configure -height $yMax
        $me itemconfigure imageU -image $imgU
        $me itemconfigure imageD -image $imgD
        $me coords text [expr $xMax/2] [expr $yMax*$yFrac]
    }
}
}

#=============================================================================
# .logo
#
# AutoFarm logo frame with separator and secret button. A frame which can be 
# packed into any screen.
#=============================================================================
namespace eval Logo {

    frame .logo

    # INX logo, version, and system button
    frame .logo.inx
    label .logo.inx.1 -image smallNovariantLogo
    pack .logo.inx.1 -side right -padx 10
    label .logo.inx.4 \
            -width 8 -borderwidth 0 -font $small_font -justify right
    pack .logo.inx.4 -side right
    label .logo.inx.5 \
            -width 8 -borderwidth 0 -font $small_font -anchor e
    pack .logo.inx.5 -side left -padx 5
    label .logo.inx.6 \
            -width 6 -borderwidth 0 -font $small_font -anchor w
    pack .logo.inx.6 -side left -padx 2
    button .logo.inx.2 -command {beepOnce; post $DISP_BTN7_SIG 7 } \
            -borderwidth 0 -state disabled \
            -background $color6 -highlightbackground $color6 \
            -activebackground $color6
    pack .logo.inx.2 -side right
    pack .logo -side bottom

    # External disk frame
    frame .logo.inx.extDisk
    # Disk usage bar
    canvas .logo.inx.extDisk.c1 -width 150 -height 15 -relief sunken \
            -borderwidth 1 -background $color6 -state disabled;
    .logo.inx.extDisk.c1 create rectangle 0 0 0 20 \
               -fill green -width 0 -tags flashDiskBar
    .logo.inx.extDisk.c1 create text 75 16 \
        -text "Disk Space: " -anchor s -font $small_font -tags flashDiskText

    # Circular button for datalogger - now letter instead
    #button .logo.inx.extDisk.b1 -bitmap @${tclPath}gif/circle.xbm \
    #        -background $color6 -activebackground $color6 -borderwidth 0 \
    #        -height 11 -width 11 -disabledforeground green -state disabled;
    label .logo.inx.extDisk.t1 -text "D" -font $small_font \
            -foreground gold3

    # Circular button for curve recording - now letter instead
    #button .logo.inx.extDisk.b2 -bitmap @${tclPath}gif/circle.xbm \
    #        -background $color6 -activebackground $color6 -borderwidth 0 \
    #        -height 11 -width 11 -disabledforeground green -state disabled;
    label .logo.inx.extDisk.t2 -text "R" -font $small_font \
            -foreground gold3

    # Letter for other external disk user
    label .logo.inx.extDisk.t3 -text "O" -font $small_font \
            -foreground gold3

    # Letter for geoid data status
    label .logo.inx.extDisk.t4 -text "G" -font $small_font \
            -foreground gold3

    # Letter for control data status
    label .logo.inx.extDisk.t5 -text "C" -font $small_font \
            -foreground gold3

    pack .logo.inx.extDisk -side right -padx 20;


    pack .logo.inx -side bottom -fill x

    frame .logo.sep -height 2 -width $winWidth -borderwidth 0 \
            -background $colorInx
    pack .logo.sep -side bottom

    proc actReconfigLogo {logoImage} {
        .logo.inx.1 configure -image $logoImage
    }

    proc actConfigSwVersion {swVersion } {
        .logo.inx.4 configure -text $swVersion
    }

    proc actConfigLanguage {screen diskSpace} {
        .logo.inx.5 configure -text $screen
        .logo.inx.extDisk.c1 itemconfigure flashDiskText -text $diskSpace
    }

    proc actConfigScreenId {screenId} {
        .logo.inx.6 configure -text $screenId
    }

    proc actSecretSpotOn {} {
        .logo.inx.2 configure -state "normal"
    }

    proc actSecretSpotOff {} {
        .logo.inx.2 configure -state "disabled"
    }

    proc actEnableExtDisk {} {
        pack .logo.inx.extDisk.c1 -side right -padx 10;
        #pack .logo.inx.extDisk.b1 -side right;
        pack .logo.inx.extDisk.t5 -side right;
        pack .logo.inx.extDisk.t1 -side right;
        #pack .logo.inx.extDisk.b2 -side right;
        pack .logo.inx.extDisk.t2 -side right;
        pack .logo.inx.extDisk.t4 -side right;
        pack .logo.inx.extDisk.t3 -side right;
    }

# MLO - Obsolete.  We may some day want a funtion like actConfigExtDisk that
# only changes colors and not the disk space text, but we don't need to need
# this function today.
#    proc actConfigExtDiskToYellow {} {
#        .logo.inx.extDisk.c1 itemconfigure flashDiskBar -fill yellow;
#        #.logo.inx.extDisk.b1 configure -disabledforeground yellow;
#        #.logo.inx.extDisk.b2 configure -disabledforeground yellow;
#        .logo.inx.extDisk.t1 configure -foreground yellow;
#        .logo.inx.extDisk.t2 configure -foreground yellow;
#        .logo.inx.extDisk.t3 configure -foreground yellow;
#    }

    proc actConfigGeoid { geoidStat } {
        if { $geoidStat != 0 } {
            .logo.inx.extDisk.t4 configure -foreground green4;
	} else {
            .logo.inx.extDisk.t4 configure -foreground gold3;
	}
    }

    proc actConfigExtDisk {barText percentUsed isLoggingData \
            isRecordingData isRecordingControlData otherUsingExtDisk } {
        if { $percentUsed >= 100.0 } {
            .logo.inx.extDisk.c1 itemconfigure flashDiskBar -fill red;
            #.logo.inx.extDisk.b1 configure -disabledforeground red;
            #.logo.inx.extDisk.b2 configure -disabledforeground red;
            .logo.inx.extDisk.t1 configure -foreground red;
            .logo.inx.extDisk.t2 configure -foreground red;
            .logo.inx.extDisk.t3 configure -foreground red;
	    .logo.inx.extDisk.t4 configure -foreground red;
        } else {
            if { $isRecordingData == 1 || $isLoggingData == 1 } {
                .logo.inx.extDisk.c1 itemconfigure flashDiskBar -fill green4;
            } else {
                .logo.inx.extDisk.c1 itemconfigure flashDiskBar -fill gold3;
            }
            if { $isLoggingData == 1 } {
                #.logo.inx.extDisk.b1 configure -disabledforeground green4;
                .logo.inx.extDisk.t1 configure -foreground green4;
            } else {
                #.logo.inx.extDisk.b1 configure -disabledforeground gold3;
                .logo.inx.extDisk.t1 configure -foreground gold3;
            }
            if { $isRecordingData == 1 } {
                #.logo.inx.extDisk.b2 configure -disabledforeground green4;
                .logo.inx.extDisk.t2 configure -foreground green4;
            } else {
                #.logo.inx.extDisk.b2 configure -disabledforeground gold3;
                .logo.inx.extDisk.t2 configure -foreground gold3;
            }
            if { $isRecordingControlData == 1 } {
                #.logo.inx.extDisk.b2 configure -disabledforeground green4;
                .logo.inx.extDisk.t5 configure -foreground green4;
            } else {
                #.logo.inx.extDisk.b2 configure -disabledforeground gold3;
                .logo.inx.extDisk.t5 configure -foreground gold3;
            }
            if { $otherUsingExtDisk == 1 } {
                .logo.inx.extDisk.t3 configure -foreground green4;
            } else {
                .logo.inx.extDisk.t3 configure -foreground gold3;
            }
        }
        
        # Set the rectangle indicating the fullness of the disk.
        set rectLen 200.0;
        # The 100.0 in the denominator represents 100.0%.
        set rectPixelPos [expr $percentUsed*$rectLen/100.0];
        if { $rectPixelPos >= $rectLen } {
            set rectPixelPos [expr $rectLen+5];
        } elseif { $rectPixelPos < 0.0 } {
            set rectPixelPos 0.0;
        }
        .logo.inx.extDisk.c1 coords flashDiskBar $rectPixelPos 0 0 20;
        .logo.inx.extDisk.c1 itemconfigure flashDiskText \
                -text $barText;
    }

    proc buzzer {} {
        beepOnce;
    }    

    proc alertBuzzer {} {
        beepOnce;
        beepOnce;
        beepOnce;
    }    
}


#=============================================================================
# .configDisplay
#
# The AutoFarm configuration display. A frame which can be packed into any 
# screen.
#=============================================================================
namespace eval ConfigDisplay { 

    frame .configDisplay -height $configHeight -width $configWidth
    frame .configDisplay.r1
    frame .configDisplay.r2
    frame .configDisplay.r3
    frame .configDisplay.r4
    frame .configDisplay.r5

    label .configDisplay.r1.l -font $medium_font
    label .configDisplay.r2.l -font $medium_font
    label .configDisplay.r3.l -font $medium_font
    label .configDisplay.r4.l -font $medium_font
    pack .configDisplay.r1.l -side left -pady 1
    pack .configDisplay.r2.l -side left -pady 1
    pack .configDisplay.r3.l -side left -pady 1
    pack .configDisplay.r4.l -side left -pady 1

    label .configDisplay.r4.l2 \
            -font $medium_font -justify right
    label .configDisplay.r3.l2 \
            -font $medium_font -justify right
    label .configDisplay.r2.l2 \
            -font $medium_font -justify right
    label .configDisplay.r1.l2 -text "" \
            -font $medium_font -justify right
    pack .configDisplay.r1.l2 -side right
    pack .configDisplay.r2.l2 -side right
    pack .configDisplay.r3.l2 -side right
    pack .configDisplay.r4.l2 -side right

    label .configDisplay.r5.e1 -text "0" -font $medium_font \
            -bg red -width 2 -justify left
    label .configDisplay.r5.e2 -font $medium_font
    label .configDisplay.r5.e3 -text "0" -font $medium_font \
            -bg red -width 2 -justify left
    label .configDisplay.r5.e4 -font $medium_font
    label .configDisplay.r5.e5 -text "0" -font $medium_font \
            -bg red -width 4
    label .configDisplay.r5.e6 -font $medium_font
    pack .configDisplay.r5.e2 -side left
    pack .configDisplay.r5.e1 -side left
    pack .configDisplay.r5.e4 -side left
    pack .configDisplay.r5.e3 -side left
    pack .configDisplay.r5.e5 -side right
    pack .configDisplay.r5.e6 -side right

    pack .configDisplay.r5 -side top -anchor w -fill x
    pack .configDisplay.r2 -side top -anchor w -fill x
 #   pack .configDisplay.r3 -side bottom -anchor w -fill x
 #   pack .configDisplay.r4 -side bottom -anchor w -fill x
    pack .configDisplay.r1 -side top -anchor w -fill x

    proc actChangeTractor { tractorName__ } {
        .configDisplay.r4.l2 configure -text $tractorName__
    }

    proc actChangeImplement { implementName__ } {
        .configDisplay.r3.l2 configure -text $implementName__
    }

    proc actChangeUserLevel { userLevel__ } {
        .configDisplay.r2.l2 configure -text $userLevel__
    }

    proc actGPSHealth { numSvL1Base__ numSvL1BaseColor__ \
            numSvL1Rover0__ numSvL1Rover0Color__ \
            gpsQuality__ gpsQualityColor__ } {

        .configDisplay.r5.e3 configure -text $numSvL1Base__ \
                -bg $numSvL1BaseColor__
        .configDisplay.r5.e1 configure -text $numSvL1Rover0__ \
                -bg $numSvL1Rover0Color__
        .configDisplay.r5.e5 configure -text $gpsQuality__ \
                -bg $gpsQualityColor__
    }

    proc actNumSats { numSvL1Base__ numSvL1BaseColor__ \
            numSvL1Rover0__ numSvL1Rover0Color__ } {
        .configDisplay.r5.e3 configure -text $numSvL1Base__ \
                -bg $numSvL1BaseColor__
        .configDisplay.r5.e1 configure -text $numSvL1Rover0__ \
                -bg $numSvL1Rover0Color__
    }

    proc actPosQual {gpsQuality__ gpsQualityColor__ } {
        .configDisplay.r5.e5 configure -text $gpsQuality__ \
                -bg $gpsQualityColor__
    }

    proc actPosError { posError__ } {
        .configDisplay.r1.l2 configure -text $posError__
    }
    
    proc actConfigLanguage {accuracy userLevel implement vehicle\
                            tractorSats baseSats qual} {
        .configDisplay.r2.l configure -text $userLevel
#        .configDisplay.r3.l configure -text $implement
#        .configDisplay.r4.l configure -text $vehicle
        .configDisplay.r5.e2 configure -text $tractorSats
        .configDisplay.r5.e4 configure -text $baseSats
        .configDisplay.r5.e6 configure -text $qual
    }
}


#=============================================================================
# .navDisplay
#
# The AutoFarm navigation display. A frame which can be packed into any 
# screen.
#=============================================================================
namespace eval NavDisplay { 

    frame .navDisplay -height $navHeight -width $navWidth
    frame .navDisplay.ready
    frame .navDisplay.meas
    frame .navDisplay.meas.pos
    frame .navDisplay.meas.datalink
    frame .navDisplay.meas.att
    frame .navDisplay.meas.steer

    label .navDisplay.ready.b1 -image btnRED    
    pack .navDisplay.ready.b1 -side left
    label .navDisplay.ready.t -text "Ready for AutoLevel" -font $medium_font
    pack .navDisplay.ready.t -side left

    foreach { meas } { pos datalink att} {
        label .navDisplay.meas.$meas.t -font $medium_font
        label .navDisplay.meas.$meas.0 -image btnGREY
        label .navDisplay.meas.$meas.1 -image btnGREY
        label .navDisplay.meas.$meas.2 -image btnGREY
        pack .navDisplay.meas.$meas.t -side right
        pack .navDisplay.meas.$meas.0 -side right
        pack .navDisplay.meas.$meas.1 -side right
        pack .navDisplay.meas.$meas.2 -side right
    }
    pack .navDisplay.meas.steer -side bottom -anchor w
    pack .navDisplay.meas.att -side bottom -anchor w
    pack .navDisplay.meas.datalink -side bottom -anchor w
    pack .navDisplay.meas.pos -side bottom -anchor w
    pack .navDisplay.ready -side bottom -anchor w
    pack .navDisplay.meas -side left -anchor s

    proc actIndicators {status_pos status_datalink status_att 
                        status_steer status_autofarm} {
        global color6 colorMedium colorBad

        foreach { meas } { pos datalink att} {
            set status [set status_$meas]
            if {$status == -1} {
                .navDisplay.meas.$meas.0 configure -image btnGREY
                .navDisplay.meas.$meas.1 configure -image btnGREY
                .navDisplay.meas.$meas.2 configure -image btnGREY
            } elseif {$status == 0} {
                .navDisplay.meas.$meas.0 configure -image btnGREY
                .navDisplay.meas.$meas.1 configure -image btnGREY
                .navDisplay.meas.$meas.2 configure -image btnGREEN
            } elseif {$status == 1} {
                .navDisplay.meas.$meas.0 configure -image btnGREY
                .navDisplay.meas.$meas.1 configure -image btnYELLOW
                .navDisplay.meas.$meas.2 configure -image btnGREY
            } else {
                .navDisplay.meas.$meas.0 configure -image btnRED
                .navDisplay.meas.$meas.1 configure -image btnGREY
                .navDisplay.meas.$meas.2 configure -image btnGREY
            }
        }

        if {$status_autofarm == 0} {
            .navDisplay.ready.b1 configure -image btnGREEN
        } elseif {$status_autofarm == 1} {
            .navDisplay.ready.b1 configure -image btnYELLOW
        } else {
            .navDisplay.ready.b1 configure -image btnRED
        }
    }

    proc actConfigLanguage {ready pos datalink att steer} {
        .navDisplay.ready.t         configure -text $ready
        .navDisplay.meas.pos.t      configure -text $pos
        .navDisplay.meas.datalink.t configure -text $datalink
        .navDisplay.meas.att.t      configure -text $att
#        .navDisplay.meas.steer.t    configure -text $steer
    }

    proc actConfigBaseBatteryOk {} {
        global color6
        .navDisplay.meas.datalink.t configure -background $color6
    }

    proc actConfigBaseBatteryLow {} {
        .navDisplay.meas.datalink.t configure -background yellow
    }
}


#=============================================================================
# .tractorDisplay
#
# The AutoFarm tractor display. A frame which can be packed into any screen.
#=============================================================================
namespace eval TractorDisplay { 
    
    frame .tractorDisplay

    # Speed text and bar
    label .tractorDisplay.speedText -font $medium_font
    
    canvas .tractorDisplay.speedBar -width 200 -height 20 -relief sunken \
            -borderwidth 1 -background $color6
    .tractorDisplay.speedBar \
            create rectangle 100 0 100 20 -fill green -outline black \
            -tags Vxbar820

    # Tractor and implement display canvas 
    # (1 pixel approximately equals 10 cm)
    canvas .tractorDisplay.c -width 200 -height 200 -relief sunken \
            -borderwidth 1 -background blue

    # bind <ButtonPress-1> to DISP_CANVAS0_SIG to enable JobMark
    bind .tractorDisplay.c <ButtonPress-1> \
            { beepOnce; post $DISP_CANVAS1_SIG 0 }

    pack .tractorDisplay.c -side bottom -anchor e -pady 10 -padx 35 
    pack .tractorDisplay.speedBar -side bottom -anchor e -padx 35 
    pack .tractorDisplay.speedText -side bottom -anchor e -padx 35 


    proc drawTractor { me x y type color implWidth implOffs } {
        
        set bodyWidth   20
        set bodyHeight  55
        set implHeight  10

        set x [expr $x - $bodyWidth/2]
        set tractorColor(0) blue
        set tractorColor(1) green
        set tractorColor(2) orange
        set tractorColor(3) red
        set tractorColor(4) yellow
        
        set color $tractorColor($color)
        
        $me create rectangle $x $y \
                             [expr $x + $bodyWidth] [expr $y - $bodyHeight] \
                             -fill $color -outline black
        if {$type == 1} {                     
            $me create line [expr $x - 6] [expr $y - $bodyHeight - 4] \
                            [expr $x - 6] [expr $y - $bodyHeight + 8] \
                            -fill black -width 6 
            $me create line [expr $x + $bodyWidth+7] [expr $y-$bodyHeight -4]\
                            [expr $x + $bodyWidth+7] [expr $y-$bodyHeight +8]\
                            -fill black -width 6 
            set h 24
        } else {
            set h 40
        }
                             
        $me create line [expr $x - 6] [expr $y + 6] \
                        [expr $x - 6] [expr $y + 6 - $h] \
                        -fill black -width 8 
        $me create line [expr $x + $bodyWidth + 7] [expr $y + 6] \
                        [expr $x + $bodyWidth + 7] [expr $y + 6 - $h] \
                        -fill black -width 8
                        
        $me create line [expr $x + $bodyWidth/2] [expr $y] \
                        [expr $x + $bodyWidth/2 + $implOffs] [expr $y + 16] \
                        -fill black -width 1
                        
        $me create rectangle [expr $x +($bodyWidth-$implWidth)/2 + $implOffs]\
                             [expr $y + $implHeight + 16] \
                             [expr $x +($bodyWidth+$implWidth)/2 + $implOffs]\
                             [expr $y + 16] \
                             -fill $color -outline black
        return $me 
    }

    proc actConfigTractorIcon { tractorType__ tractorColor__ \
            implWidth__ implOffset__ } {
        SetTractorIconCoords $implOffset__
        ReconfigTractorIcon $tractorType__ $implWidth__ $tractorColor__
    }

    proc actConfigureRowlineup { manualSteeringText__ } {
        global colorRowlineup autosteerManualSteering
        .tractorDisplay.c configure -background $colorRowlineup
        .tractorDisplay.c itemconfigure $autosteerManualSteering \
                -text $manualSteeringText__
    }

    proc actConfigureAutosteer { } {
        global colorAutosteer autosteerManualSteering
        .tractorDisplay.c configure -background $colorAutosteer
        .tractorDisplay.c itemconfigure $autosteerManualSteering -text " "
    }

    proc actVehicleDispData { velDispText__ Vx__ VxMax__ \
            headingError__ posError__ \
            passWidth__ fanAngle__ rowNumText__ nudgeDistText__ \
            turboMessageText__ ENHANCED_RLM__ \
            ERLM_HE_ENTER__ ERLM_HE_EXIT__ \
            ERLM_YC_ENTER__ ERLM_YC_EXIT__ \
            inTightMode__} {

        global rowHeadingErr rowPosErr
        set rowHeadingErr $headingError__
        set rowPosErr $posError__

        global autosteerRowNum autosteerNudgeDist autosteerTurboMessage
        .tractorDisplay.c itemconfigure $autosteerRowNum \
                -text $rowNumText__
        .tractorDisplay.c itemconfigure $autosteerNudgeDist \
                -text $nudgeDistText__
        .tractorDisplay.c itemconfigure $autosteerTurboMessage \
                -text $turboMessageText__

        ConfigureTractorIcon $passWidth__ $fanAngle__ $ENHANCED_RLM__ \
                $ERLM_HE_ENTER__ $ERLM_HE_EXIT__ \
                $ERLM_YC_ENTER__ $ERLM_YC_EXIT__ \
                $inTightMode__ $Vx__
        ConfigureVelocityDisplay $velDispText__ $Vx__ $VxMax__
    }

    proc actSteerCodeDisp { turningWheelText__ killText__ clutchText__ \
            parkText__ resetReqText__ roadingText__ comsBadText__ \
            notReadyText__ inxUnsText__ } {

        global autosteerTurningWheel autosteerKillSwitchOn \
                autosteerMtClutchIn autosteerMtInPark autosteerMtResetReqd \
                autosteerMtRoading autosteerMtComsBad autosteerMtNotReady \
                autosteerMtInxUns
        .tractorDisplay.c itemconfigure $autosteerTurningWheel \
                -text $turningWheelText__
        .tractorDisplay.c itemconfigure $autosteerKillSwitchOn \
                -text $killText__
        .tractorDisplay.c itemconfigure $autosteerMtClutchIn \
                -text $clutchText__
        .tractorDisplay.c itemconfigure $autosteerMtInPark \
                -text $parkText__
        .tractorDisplay.c itemconfigure $autosteerMtResetReqd \
                -text $resetReqText__
        .tractorDisplay.c itemconfigure $autosteerMtRoading \
                -text $roadingText__
        .tractorDisplay.c itemconfigure $autosteerMtComsBad \
                -text $comsBadText__
        .tractorDisplay.c itemconfigure $autosteerMtNotReady \
                -text $notReadyText__
        .tractorDisplay.c itemconfigure $autosteerMtInxUns \
                -text $inxUnsText__
    }

    proc actConfigLanguage {speed} {
        .tractorDisplay.speedText configure -text $speed
    }
    
    proc actMarkPixels { xPixels yPixels xStart yStart xEnd yEnd } {
        # set the mark coordinates
        global autosteerMark autosteerMarkSide
        .tractorDisplay.c coords $autosteerMark \
                [expr $xPixels-$autosteerMarkSide/2] $yPixels \
                [expr $xPixels+$autosteerMarkSide/2] $yPixels \
                $xPixels [expr $yPixels-$autosteerMarkSide]

        # set the arrow coordinates
        global autosteerMarkArrow
        .tractorDisplay.c coords $autosteerMarkArrow \
                $xStart $yStart $xEnd $yEnd
    }

    proc actSurveyMarkPixels { xPixels yPixels northID eastID } {
        # set the survey mark coordinates
        global autosteerSurveyMark autosteerSurveyText autosteerMarkSide
        .tractorDisplay.c coords $autosteerSurveyMark \
                [expr $xPixels-$autosteerMarkSide/2] $yPixels \
                [expr $xPixels+$autosteerMarkSide/2] $yPixels \
                $xPixels [expr $yPixels-$autosteerMarkSide]
        .tractorDisplay.c coords $autosteerSurveyText \
                $xPixels [expr $yPixels+7]
        .tractorDisplay.c itemconfigure $autosteerSurveyText \
                -text "($northID,$eastID)"
    }

}


#=============================================================================
#
# The AutoFarm sliders. A bunch of sliders that can be packed into any screen.
#=============================================================================
namespace eval Sliders { 
    
    proc slider { me numSteps sig } {
        global medium_font big_font color3

        set readoutWidth  100
        set readoutHeight  45
        set btnWidth       60
        set titleHeight    20

        canvas $me -width [expr $readoutWidth + 2*$btnWidth + 4] \
            -height [expr $readoutHeight + $titleHeight + 4] \
            -relief raised -borderwidth 0 -background "white"
        SnazzyButton::create $me.neg btnBlueSU btnBlueSD 0.5 \
            "beepOnce; post $sig -1" "-"
        SnazzyButton::setText $me.neg "-" 0.5                
        SnazzyButton::create $me.pos btnBlueSU btnBlueSD 0.5 \
            "beepOnce; post $sig 1" "+"
        SnazzyButton::setText $me.pos "+" 0.5

        $me create window [expr 2 + $btnWidth] $titleHeight \
            -anchor ne -window $me.neg
        $me create window [expr 6 + $btnWidth + $readoutWidth] $titleHeight \
            -anchor nw -window $me.pos
        $me create text [expr 2 + $btnWidth + 2 + $readoutWidth/2] 2 \
            -anchor n -font $medium_font -tags title    
        $me create text [expr 2 + $btnWidth + 6] [expr $titleHeight + 0] \
            -anchor nw -font $medium_font -text 10 -tags value
        
        # create an off-screen text to store number of steps        
        $me create text -20 0 -text $numSteps -tags numSteps
                

        set x0 [expr 2 + $btnWidth + 2]

        set notchWidth  [expr 0.6*$readoutWidth/$numSteps]
        for {set i 0} {$i < $numSteps} {incr i} {
            set x  [expr $x0 + $readoutWidth*($i + 0.5)/$numSteps]
            set y1 [expr $titleHeight + \
                    $readoutHeight*(0.9 - 0.8*($i+0.5)/$numSteps)]
            set y2 [expr $titleHeight + $readoutHeight]
            $me create line $x $y1 $x $y2 -width $notchWidth \
                -fill grey -tags notch$i
        }

        set x1 [expr $x0 + $readoutWidth/2 - 1]
        set x2 [expr $x0 + $readoutWidth]
        set y1 [expr $titleHeight + $readoutHeight - 3]
        set y2 [expr $titleHeight + $readoutHeight]
        $me create line $x0 $y2 $x2 $y2 
        $me create line $x0 $y1 $x0 $y2 
        $me create line $x1 $y1 $x1 $y2 
        $me create line $x2 $y1 $x2 $y2
        
        return $me 
    }

    proc sliderConfigureTitle { me title } {
       $me itemconfigure title -text $title
    }

    proc sliderConfigureButtons { me negBtn posBtn } {
        SnazzyButton::setText $me.neg $negBtn 0.5
        SnazzyButton::setText $me.pos $posBtn 0.5 
    }

    proc sliderSetValue { me val notch } {
        $me itemconfigure value -text $val
        set numSteps [$me itemcget numSteps -text]
        for {set i 0} {$i < $notch} {incr i} {
            $me itemconfigure notch$i -fill "green"
        }
        for {set i $notch} {$i < $numSteps} {incr i} {
            $me itemconfigure notch$i -fill "grey"
        }
    }

    slider .s0 20 $DISP_SLDR0_SIG   ;# drives CONT_LEVEL 
    slider .s1 20 $DISP_SLDR1_SIG   ;# drives REV_CONT_LEVEL
    slider .s2 20 $DISP_SLDR2_SIG   ;# drives TRAJ_CONT_LEVEL
    slider .s3 20 $DISP_SLDR3_SIG   ;# drives CAT_DB_LEVEL
    slider .s4 20 $DISP_SLDR4_SIG   ;# drives CAT_DB_POS_LEVEL
    slider .s5 20 $DISP_SLDR5_SIG   ;# drives CAT_DB_NEG_LEVEL
    slider .s6 20 $DISP_SLDR6_SIG   ;# drives K_ROLL

    proc actSliderValue { sliderNum val notch } {
        Sliders::sliderSetValue .s$sliderNum $val $notch
    }
    
    proc actConfigLanguage {s0_title s1_title s2_title
                            s3_title s4_title 
                            s5_title s6_title } {
        
        sliderConfigureTitle .s0 $s0_title
        sliderConfigureTitle .s1 $s1_title    
        sliderConfigureTitle .s2 $s2_title    
        sliderConfigureTitle .s3 $s3_title    
        sliderConfigureTitle .s4 $s4_title    
        sliderConfigureTitle .s5 $s5_title    
        sliderConfigureTitle .s6 $s6_title
    }

    proc actSetCalibValues {s2_value s2_notch 
			    s3_value s3_notch
 			    s4_value s4_notch 
			    s5_value s5_notch 
			    s6_value s6_notch} {
        
        sliderSetValue .s2 $s2_value $s2_notch   
        sliderSetValue .s3 $s3_value $s3_notch   
        sliderSetValue .s4 $s4_value $s4_notch   
        sliderSetValue .s5 $s5_value $s5_notch   
        sliderSetValue .s6 $s6_value $s6_notch   
    }

}


#=============================================================================
# .contDataDisplay
#
# The controller-data display. A general data-plotting display, which can be 
# packed into any screen. 
#=============================================================================
namespace eval ContDataDisplay {

    # define the containing frame
    frame .contDataDisplay

    # define the canvas
    canvas .contDataDisplay.c -width 300 -height 130 -relief sunken \
            -borderwidth 1 -background gray85
    pack .contDataDisplay.c -side bottom -padx 10 -pady 10

    # bind <ButtonPress-1> to DISP_CANVAS0_SIG to enable toggling 
    # continuous GUI data
    bind .contDataDisplay.c <ButtonPress-1> \
            { beepOnce; post $DISP_CANVAS0_SIG 0 }

    # continuous data on/off text
    set contDataMsg [.contDataDisplay.c create text 230 12 -text "" \
            -fill red -font $big_font]

    proc actPlotContData {} {

        beepOnce; 

        global big_font color5
        global timeAxisTag gridLineTag line0Tag line5cm1Tag line5cm2Tag \
            line10cm1Tag line10cm2Tag y1LineTag y2LineTag

        if [catch {open "/tmp/guiDataFile.txt" r} fileId] {
            puts stdout "Cannot open /tmp/guiDataFile.txt: $fileId\n"
        } else {

            # clear the canvas
            if {[info exists timeAxisTag] == 1} {
                .contDataDisplay.c delete $timeAxisTag
                .contDataDisplay.c delete $gridLineTag
                .contDataDisplay.c delete $line0Tag
                .contDataDisplay.c delete $line5cm1Tag
                .contDataDisplay.c delete $line5cm2Tag
                .contDataDisplay.c delete $line10cm1Tag
                .contDataDisplay.c delete $line10cm2Tag
                .contDataDisplay.c delete $y1LineTag
                .contDataDisplay.c delete $y2LineTag
            }

            # first line in file is the time axis; read it and plot it
            gets $fileId timeAxis
            eval {set timeAxisTag [.contDataDisplay.c create line} \
                    $timeAxis {-stipple gray50]}

            # then come a bunch of vertical grid lines (one per second)
            gets $fileId gridLine
            eval {set gridLineTag [.contDataDisplay.c create line} \
                    $gridLine {-stipple gray50]}

            # if coded in the file, plot some 5cm and 10cm lines
            gets $fileId gridLine
            if { [lindex $gridLine 0] == -200 } {
                gets $fileId line0
                eval {set line0Tag [.contDataDisplay.c create line} \
                        $line0 {-fill orange]}
                gets $fileId line5cm
                eval {set line5cm1Tag [.contDataDisplay.c create line} \
                        $line5cm {-fill LimeGreen]}
                gets $fileId line5cm
                eval {set line5cm2Tag [.contDataDisplay.c create line} \
                        $line5cm {-fill LimeGreen]}
                gets $fileId line10cm
                eval {set line10cm1Tag [.contDataDisplay.c create line} \
                        $line10cm {-fill red]}
                gets $fileId line10cm
                eval {set line10cm2Tag [.contDataDisplay.c create line} \
                        $line10cm {-fill red]}
            }

            # Plot the first data line
            gets $fileId y1Line
            eval {set y1LineTag [.contDataDisplay.c create line} \
                    $y1Line {-fill magenta -width 2]}

            # Plot the second data line
            gets $fileId y2Line
            eval {set y2LineTag [.contDataDisplay.c create line} \
                    $y2Line {-fill blue -width 2]}

            close $fileId
        }
    }

    proc actConfigureContDataMsg { msg__ } {
        puts "actConfigureContDataMsg $msg__"
        global contDataMsg
        .contDataDisplay.c itemconfigure \
                $ContDataDisplay::contDataMsg -text $msg__
    }


}


#=============================================================================
# .calDataDisplay
#
# The calibration-data display. Displays input-output graphs during 
# calibration.
#=============================================================================
namespace eval CalDataDisplay {

    # define the containing frame
    frame .calDataDisplay

    # define the canvas
    canvas .calDataDisplay.c -width 400 -height 300 -relief sunken \
            -borderwidth 1 -background gray85
    pack .calDataDisplay.c -side bottom -padx 10 -pady 10

    set calRawAngleMarker \
            [.calDataDisplay.c create line 500 0 500 300 -width 5]
    set calUrawMarker [.calDataDisplay.c create line 0 0 0 300 -width 3]

    proc actPlotCalData { } {

        global small_font 
        global xscale KNX0 yscale KNY0
        global calUrawMarker calRawAngleMarker

        if [catch {open "/tmp/calDataFile.txt" r} fileId] {
            puts stdout "Cannot open /tmp/calDataFile.txt: $fileId\n"
        } else {

            # clear the canvas
            foreach id [.calDataDisplay.c find all] {
                if { $id != $CalDataDisplay::calUrawMarker && \
                        $id != $CalDataDisplay::calRawAngleMarker } {
                    .calDataDisplay.c delete $id
                }
            }

            # plot the new data
            while { [gets $fileId marker] > 0 } {

                if { $marker == "xaxis" } {
                    gets $fileId xaxis
                    eval {.calDataDisplay.c create line} $xaxis \
                            {-stipple gray50}
                } elseif { $marker == "yaxis" } {
                    gets $fileId yaxis
                    eval {.calDataDisplay.c create line} $yaxis \
                            {-stipple gray50}
                } elseif { $marker == "scaling" } {
                    gets $fileId scaling
                    set xscale [lindex $scaling 0]
                    set KNX0 [lindex $scaling 1]
                    set yscale [lindex $scaling 2]
                    set KNY0 [lindex $scaling 3]
                } elseif { $marker == "posPolyOld" } {
                    gets $fileId posPolyOld
                    eval {.calDataDisplay.c create line} \
                            $posPolyOld {-fill magenta -width 2}
                } elseif { $marker == "negPolyOld" } {
                    gets $fileId negPolyOld
                    eval {.calDataDisplay.c create line} \
                            $negPolyOld {-fill magenta -width 2}
                } elseif { $marker == "posPolyNew" } {
                    gets $fileId posPolyNew
                    eval {.calDataDisplay.c create line} \
                            $posPolyNew {-fill blue -width 2}
                } elseif { $marker == "negPolyNew" } {
                    gets $fileId negPolyNew
                    eval {.calDataDisplay.c create line} \
                            $negPolyNew {-fill blue -width 2}
                } elseif { $marker == "oldPosPolyText" } {
                    gets $fileId oldPosPolyText
                    .calDataDisplay.c create text 300 240 \
                            -text $oldPosPolyText -font $small_font
                } elseif { $marker == "newPosPolyText" } {
                    gets $fileId newPosPolyText
                    .calDataDisplay.c create text 300 260 \
                            -text $newPosPolyText -font $small_font
                } elseif { $marker == "oldNegPolyText" } {
                    gets $fileId oldNegPolyText
                    .calDataDisplay.c create text 100 40 \
                            -text $oldNegPolyText -font $small_font
                } elseif { $marker == "newNegPolyText" } {
                    gets $fileId newNegPolyText
                    .calDataDisplay.c create text 100 60 \
                            -text $newNegPolyText -font $small_font
                } elseif { $marker == "newMaxPosRateText" } {
                    gets $fileId newMaxPosRateText
                    .calDataDisplay.c create text 300 280 \
                            -text $newMaxPosRateText -font $small_font
                } elseif { $marker == "newMaxNegRateText" } {
                    gets $fileId newMaxNegRateText
                    .calDataDisplay.c create text 100 80 \
                            -text $newMaxNegRateText -font $small_font
                } elseif { $marker == "datapoints" } {
                    while { [gets $fileId datapoint] > 0 } {
                        eval {.calDataDisplay.c create text} \
                                $datapoint {-text "x" -font $small_font}
                    }   
                }
            }
            
            close $fileId
        }
    }


    proc actResetUrawMarker { } {
        .calDataDisplay.c coords $CalDataDisplay::calUrawMarker \
            0 0 0 300
        .calDataDisplay.c itemconfigure $CalDataDisplay::calUrawMarker \
            -fill red
    }

    proc actUpdateUrawMarker { color__ uraw__ } {

        global xscale KNX0 yscale KNY0
        global calUrawMarker

        set uRawScaled [expr round($uraw__*$xscale+$KNX0)]
        .calDataDisplay.c coords $CalDataDisplay::calUrawMarker \
            $uRawScaled 0 $uRawScaled 300
        if { $color__ != "!"} {
            .calDataDisplay.c itemconfigure $CalDataDisplay::calUrawMarker \
                -fill $color__
        }
    }

    proc actUpdateRawAngleMarker { color__ rawAngle__ } {

        global xscale KNX0 yscale KNY0
        global calRawAngleMarker

        set rawAngleScaled [expr round($rawAngle__*$xscale+$KNX0)]
        .calDataDisplay.c coords $CalDataDisplay::calRawAngleMarker \
                $rawAngleScaled 0 $rawAngleScaled 300
        .calDataDisplay.c itemconfigure $CalDataDisplay::calRawAngleMarker \
                -fill $color__
    }

    proc actHideRawAngleMarker { } {
        .calDataDisplay.c coords $CalDataDisplay::calRawAngleMarker \
                500 0 500 300
    }
}


#=============================================================================
# .curveDisp
#
# The curve-data display. Display canvas for complex curve capabilities. Can
# be packed into any screen. 
#=============================================================================
namespace eval CurveDisp {

    # define the containing frame
    frame .curveDisp

    # define the canvas
    canvas .curveDisp.c -width 400 -height 400 -relief sunken \
            -borderwidth 1 -background gray85
    pack .curveDisp.c -side bottom -anchor s

    # bind <ButtonPress-1> to DISP_CANVAS1_SIG to enable record on/off
    #bind .curveDisp.c <ButtonPress-1> { beepOnce; post $DISP_CANVAS1_SIG 0 }

    # Speed text and bar
    .curveDisp.c create text 210 12 -text "Speed: 0.0 mph Reverse"\
        -font $medium_font -anchor w -tags {cdInfo speedText}
    .curveDisp.c create rectangle 210 20 398 25 -fill $color6 -outline black \
        -tags {cdInfo speedBar1}
    .curveDisp.c create rectangle 210 21 325 24 -fill green \
        -tags {cdInfo speedBar2}

    # Record and stop button
    .curveDisp.c create rectangle 340 340 390 390 \
        -fill $color10 -outline black -width 2 \
        -tags {cdInfo recordOffBtn}
    .curveDisp.c create oval 340 340 390 390 \
        -fill red -outline black -width 2 \
        -tags {cdInfo recordOnBtn}
    .curveDisp.c create polygon 365 340 390 390 340 390 \
        -fill $color7 -outline black -width 2 \
        -tags {cdInfo recordSaveBtn}
    .curveDisp.c create text 365 365 -text "Stop"\
        -font $small_font -justify center -tags {cdInfo recordText}
    .curveDisp.c create text 360 325 -text "Record: On"\
        -font $small_font -justify center -tags {cdInfo recordText2}
    .curveDisp.c create rectangle 300 300 400 400 \
        -fill "" -outline "" -tags {cdInfo area1}
    .curveDisp.c bind area1 <ButtonPress-1> \
        { beepOnce; post $DISP_CANVAS1_SIG 0 }

    # Other text
    .curveDisp.c create text 210 38 -text "Job: STRAIGHT_JOB"\
        -font $medium_font -anchor w -tags {cdInfo jobText}
    .curveDisp.c create text 210 56 -text "Mode: STANDARD"\
        -font $medium_font -anchor w -tags {cdInfo modeText}
    .curveDisp.c create text 210 74 -text "Pass: 12"\
        -font $medium_font -anchor w -tags {cdInfo passText}
    .curveDisp.c create text 210 92 -text "Nudged: 0.000 inches (0)"\
        -font $medium_font -anchor w -tags {cdInfo nudgeText}
    .curveDisp.c create text 210 110 -text " "\
        -font $medium_font -anchor w -tags {cdInfo turboText}

    .curveDisp.c create text 8 92 -text "USER TURNING WHEEL"\
        -font $medium_font -anchor w -tags {cdInfo userText}
    .curveDisp.c create text 8 110 -text "KILL SWITCH ON"\
        -font $medium_font -anchor w -tags {cdInfo killText}
    .curveDisp.c create text 8 128 -text "Clutch In"\
        -font $medium_font -anchor w -tags {cdInfo clutchInText}
    .curveDisp.c create text 8 146 -text "In Park"\
        -font $medium_font -anchor w -tags {cdInfo inParkText}
    .curveDisp.c create text 8 164 -text "Reset Reqd"\
        -font $medium_font -anchor w -tags {cdInfo resetText}
    .curveDisp.c create text 8 182 -text "Roading"\
        -font $medium_font -anchor w -tags {cdInfo roadingText}
    .curveDisp.c create text 8 200 -text "Coms Bad"\
        -font $medium_font -anchor w -tags {cdInfo comsText}
    .curveDisp.c create text 8 218 -text "Not Ready"\
        -font $medium_font -anchor w -tags {cdInfo notReadyText}
    .curveDisp.c create text 8 236 -text "INX Unsafe"\
        -font $medium_font -anchor w -tags {cdInfo unsafeText}

    .curveDisp.c create text 8 300 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText1}
    .curveDisp.c create text 8 313 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText2}
    .curveDisp.c create text 8 326 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText3}
    .curveDisp.c create text 8 339 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText4}
    .curveDisp.c create text 8 352 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText5}
    .curveDisp.c create text 8 365 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText6}
    .curveDisp.c create text 8 378 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText7}
    .curveDisp.c create text 8 391 -text " "\
        -font $tiny_font -anchor w -tags {cdInfo diagText8}

    # Compass arrow
    .curveDisp.c create line 30 50 30 20 -width 2 \
        -arrow last -arrowshape {8 12 4} -tags {cdInfo compassArrow}
    .curveDisp.c create text 30 12 -text "N" \
        -font $medium_font -tags {cdInfo compassText}

    proc actDrawJunkTcl {} {
        global big_font small_font tiny_font
        source /tmp/junk.tcl
    }

    proc actDrawThis {script__} {
        global big_font small_font tiny_font
        eval $script__
    }

    proc actConfigureRowlineup { manualSteeringText__ } {
        global colorRowlineup
        #.curveDisp.c configure -background $colorRowlineup
        #.curveDisp configure -background orange
        #.curveDisp.c itemconfigure turboText -text $manualSteeringText__
    }

    proc actConfigureAutosteer { } {
        global colorAutosteer
        #.curveDisp.c configure -background $colorAutosteer
        #.curveDisp configure -background LimeGreen
        #.curveDisp.c itemconfigure turboText -text " "
    }

    proc actVehicleDispData { speedText__ speedBarEnd__ \
                              jobText__ modeText__ \
                              passText__ nudgeText__ \
                              turboText__ } {

        .curveDisp.c itemconfigure speedText -text $speedText__
        .curveDisp.c coords speedBar2 210 21 $speedBarEnd__ 24      
        .curveDisp.c itemconfigure jobText -text $jobText__
        .curveDisp.c itemconfigure modeText -text $modeText__
        .curveDisp.c itemconfigure passText -text $passText__
        .curveDisp.c itemconfigure nudgeText -text $nudgeText__
        .curveDisp.c itemconfigure turboText -text $turboText__
    }

    proc actSteerCodeDisp { turningWheelText__ killText__ clutchText__ \
            parkText__ resetReqText__ roadingText__ comsBadText__ \
            notReadyText__ inxUnsText__ } {

        .curveDisp.c itemconfigure userText -text $turningWheelText__
        .curveDisp.c itemconfigure killText -text $killText__
        .curveDisp.c itemconfigure clutchInText -text $clutchText__
        .curveDisp.c itemconfigure inParkText -text $parkText__
        .curveDisp.c itemconfigure resetText -text $resetReqText__
        .curveDisp.c itemconfigure roadingText -text $roadingText__
        .curveDisp.c itemconfigure comsText -text $comsBadText__
        .curveDisp.c itemconfigure notReadyText -text $notReadyText__
        .curveDisp.c itemconfigure unsafeText -text $inxUnsText__
    }

    proc actConfigDiagText { line__ text__ } {
        .curveDisp.c itemconfigure diagText$line__ -text $text__
    }

    proc actConfigRecordBtn { text__ } {
        if { $text__ == "Stop" } {
            .curveDisp.c itemconfigure recordOffBtn -state normal
            .curveDisp.c itemconfigure recordSaveBtn -state hidden
            .curveDisp.c itemconfigure recordOnBtn -state hidden
            .curveDisp.c coords recordText 365 365
        } elseif { $text__ == "Save" } {
            .curveDisp.c itemconfigure recordOffBtn -state hidden
            .curveDisp.c itemconfigure recordSaveBtn -state normal
            .curveDisp.c itemconfigure recordOnBtn -state hidden
            .curveDisp.c coords recordText 365 380
        } elseif { $text__ == "Record" } {
            .curveDisp.c itemconfigure recordOffBtn -state hidden
            .curveDisp.c itemconfigure recordSaveBtn -state hidden
            .curveDisp.c itemconfigure recordOnBtn -state normal
            .curveDisp.c coords recordText 365 365
        }
        .curveDisp.c itemconfigure recordText -text $text__
    }

    proc actConfigRecordText { text__ color__ } {
        .curveDisp.c itemconfigure recordText2 -text $text__ \
            -fill $color__
    }
}


