##############################################################################
#
# vc/src/tcl/dataloggingmap.tcl
#
# Data Logging Map screens
#
# 20030710 QC - created
#
# Copyright (c) 2003 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it in Exit
    frame .ftop
    
    frame .ftop.c1
    SnazzyButton::create .ftop.c1.cancel \
        btnKhakiForSatVUU btnKhakiForSatVUD 0.5 \
        "beepOnce; post $DISP_BTN9_SIG 9;" "Return To\nPrevious\nMenu"
    SnazzyButton::create .ftop.c1.b2 btnBlueForSatVUU btnBlueForSatVUD 0.5 \
        "beepOnce; post $DISP_BTN2_SIG 2;" "View\nOverall\nMap"       
    SnazzyButton::create .ftop.c1.b3 btnBlueForSatVUU btnBlueForSatVUD 0.5 \
        "beepOnce; post $DISP_BTN3_SIG 3;" "Full\nScreen\nMap"       
    SnazzyButton::create .ftop.c1.b4 btnBlueForSatVUU btnBlueForSatVUD 0.5 \
        "beepOnce; post $DISP_BTN5_SIG 5;" "Zoom\nOut\nMap"       

    pack .ftop.c1.cancel -side bottom -anchor sw -padx 5 -pady 5
    pack .ftop.c1.b2 -side top -anchor nw -padx 5 -pady 5
    pack .ftop.c1.b3 -side top -anchor nw -padx 5 -pady 5
    pack .ftop.c1.b4 -side top -anchor nw -padx 5 -pady 5

    frame .ftop.c2
    set canvasCHeight 450.0 
    set canvasCWidth 530.0
    set canvasC1Height 460.0
    set canvasC1Width 650.0
    set gapToAxis 50.0

    set minNorth 30000.0
    set maxNorth -30000.0
    set minEast 30000.0
    set maxEast -30000.0

    set cminNorth 30000.0
    set cmaxNorth -30000.0
    set cminEast 30000.0
    set cmaxEast -30000.0
    
    set tracType 1
    set tracColor 1
    set implWidth 1
    set implOffset 0
    set zoomInText "Zoom Map: Press on any part of the map to zoom in"
    set F2RText "Zoom Map: Press on any part of the map to zoom in"
    set currentNorth 5.0
    set currentEast 5.0
    set datalogpath "./datalogs/datalog.txt"

    set pixelsPerFeet 5.0
    
    set canvasCenterX [expr $canvasCWidth / 2]
    set canvasCenterY [expr $canvasCHeight / 2]
    
    canvas .ftop.c2.c -width $canvasCWidth -height $canvasCHeight \
                      -relief sunken -borderwidth 1 -background $color6
    canvas .ftop.c2.c1 -width $canvasC1Width -height $canvasC1Height \
                       -relief sunken -borderwidth 1 -background $color6
    pack .ftop.c2.c -side top
    bind .ftop.c2.c <ButtonPress-1> { set Screen::canvasCenterX %x; \
                                      set Screen::canvasCenterY %y; \
                                      post $DISP_BTN1_SIG 1; }
    bind .ftop.c2.c1 <ButtonPress-1> { beepOnce; post $DISP_BTN4_SIG 4; }
    
    pack .ftop.c1 -side left -anchor sw
    pack .ftop.c2 -side right -fill both -expand yes

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry {cancelBtn viewoverallBtn fullscreenBtn zoomInLbl F2RLbl \
       curNorth curEast tractype traccolor implwidth imploffset datalogpath} {
        SnazzyButton::setText .ftop.c1.cancel $cancelBtn 0.5
        SnazzyButton::setText .ftop.c1.b2 $viewoverallBtn 0.5
        SnazzyButton::setText .ftop.c1.b3 $fullscreenBtn 0.5
        set tcl_precision 17
        set Screen::tracType $tractype
        set Screen::tracColor $traccolor
        set Screen::implWidth $implwidth
        set Screen::implOffset $imploffset
        set Screen::zoomInText $zoomInLbl
        set Screen::F2RText $F2RLbl
        set Screen::currentNorth $curNorth
        set Screen::currentEast $curEast
        set Screen::datalogpath $datalogpath
        Screen::findMinMax
        pack .ftop -fill both -expand yes
    }
    
    # Action called by GuiMgr on exit from this screen
    proc actExit {} {

        destroy .ftop

    }
   
    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    # Update procedures of this template can be called by the GuiMgr
    proc plotOverallMap {tnorth teast} {
        Screen::plotData .ftop.c2.c $Screen::cminNorth $Screen::cmaxNorth \
        $Screen::cminEast $Screen::cmaxEast $Screen::zoomInText $tnorth $teast
    }

    proc goToRegularScreen {} {
        pack forget .ftop.c2.c1
        pack forget .ftop.c2
        
        pack .ftop.c1.cancel -side bottom -anchor sw -padx 5 -pady 5
        pack .ftop.c1.b2 -side top -anchor nw -padx 5 -pady 5
        pack .ftop.c1.b3 -side top -anchor nw -padx 5 -pady 5
        pack .ftop.c1.b4 -side top -anchor nw -padx 5 -pady 5
        pack .ftop.c2.c -side top
        pack .ftop.c1 -side left -anchor sw
        pack .ftop.c2 -side right -fill both -expand yes
    }

    proc plotCurrentMap {tnorth teast} {
      Screen::plotData .ftop.c2.c $Screen::cminNorth $Screen::cmaxNorth \
        $Screen::cminEast $Screen::cmaxEast $Screen::zoomInText $tnorth $teast
    }

    proc plotCurrentPositionRS { prevnorth preveast tnorth teast tyaw } {
        Screen::plotCurrentPosition .ftop.c2.c $prevnorth $preveast \
                                    $tnorth $teast $tyaw
    }

    proc plotCurrentPositionFS { prevnorth preveast tnorth teast tyaw } {
        Screen::plotCurrentPosition .ftop.c2.c1 $prevnorth $preveast \
                                    $tnorth $teast $tyaw
    }

    proc plotFullScreenMap {tnorth teast} {
        Screen::plotData .ftop.c2.c1 $Screen::cminNorth $Screen::cmaxNorth \
           $Screen::cminEast $Screen::cmaxEast $Screen::F2RText $tnorth $teast
    }

    # Display messages to customer on Regular Screen
    proc pleaseStandByRS {} {
        pleaseStandBy .ftop.c2.c $Screen::canvasCWidth $Screen::canvasCHeight
    }

    # Display messages to customer on Full Screen
    proc pleaseStandByFS {} {
        pack forget .ftop.c1
        pack forget .ftop.c1.cancel
        pack forget .ftop.c1.b2
        pack forget .ftop.c1.b3
        pack forget .ftop.c1.b4
        pack forget .ftop.c2
        pack forget .ftop.c2.c
        pack .ftop.c2.c1 -side top
        pack .ftop.c2 -side top
        pleaseStandBy .ftop.c2.c1 $Screen::canvasC1Width \
                      $Screen::canvasC1Height
    }

    proc findMinMax {} {
      global Screen::minNorth Screen::maxNorth
      global Screen::minEast Screen::maxEast
      global Screen::cminNorth Screen::cmaxNorth
      global Screen::cminEast Screen::cmaxEast
      global Screen::canvasCHeight Screen::canvasCWidth
      
      if [catch {open $Screen::datalogpath r} fileId] {
         puts stdout "Cannot open datalog.txt: $fileId\n"
         set totalNorthPixels \
              [expr $Screen::canvasCHeight - $Screen::gapToAxis - 5]
         set totalEastPixels \
              [expr $Screen::canvasCWidth - $Screen::gapToAxis - 5]
         set totalNorthValue 1000.0
         set totalEastValue 1178.0
         
         if {[expr $totalNorthPixels / $totalNorthValue] < \
             [expr $totalEastPixels / $totalEastValue]} {
           set Screen::pixelsPerFeet [expr $totalNorthPixels/$totalNorthValue]
         } else {
           set Screen::pixelsPerFeet [expr $totalEastPixels/$totalEastValue]
         }
         set Screen::cminNorth [expr $Screen::currentNorth - 500.0]
         set Screen::cmaxNorth [expr $Screen::currentNorth + 500.0]
         set Screen::cminEast [expr $Screen::currentEast - 589.0]
         set Screen::cmaxEast [expr $Screen::currentEast + 589.0]
      } else {
         gets $fileId point
         while { [gets $fileId point] > 0 } {
           set north [lindex $point 3]
           set east [lindex $point 4]
           if { $north > $Screen::maxNorth } {
             set Screen::maxNorth $north
           }
           
           if { $north < $Screen::minNorth } {
             set Screen::minNorth $north
           }
           
           if { $east > $Screen::maxEast } {
             set Screen::maxEast $east
           }
           
           if { $east < $Screen::minEast } {
             set Screen::minEast $east
           }
         }
         close $fileId
         
         set totalNorthPixels \
              [expr $Screen::canvasCHeight - $Screen::gapToAxis - 5]
         set totalEastPixels \
              [expr $Screen::canvasCWidth - $Screen::gapToAxis - 5]
         set totalNorthValue [expr $Screen::maxNorth - $Screen::minNorth]
         set totalEastValue [expr $Screen::maxEast - $Screen::minEast]
         
         if {$totalNorthValue == 0} {
           set totalNorthValue 1
         }
         if {$totalEastValue == 0} {
           set totalEastValue 1
         }
         
         if {[expr $totalNorthPixels / $totalNorthValue] < \
             [expr $totalEastPixels / $totalEastValue]} {
           set Screen::pixelsPerFeet [expr $totalNorthPixels/$totalNorthValue]
         } else {
           set Screen::pixelsPerFeet [expr $totalEastPixels/$totalEastValue]
         }
         
         set Screen::cminNorth $Screen::minNorth
         set Screen::cmaxNorth \
             [expr $totalNorthPixels/$Screen::pixelsPerFeet+$Screen::minNorth]
         set Screen::cminEast $Screen::minEast
         set Screen::cmaxEast \
              [expr $totalEastPixels/$Screen::pixelsPerFeet+$Screen::minEast]
      }
      
      puts stdout "datalogpath: $Screen::datalogpath\n"

      puts stdout "minNorth: $Screen::minNorth\n"
      puts stdout "maxNorth: $Screen::maxNorth\n"
      puts stdout "minEast: $Screen::minEast\n"
      puts stdout "maxEast: $Screen::maxEast\n"

      puts stdout "cminNorth: $Screen::cminNorth\n"
      puts stdout "cmaxNorth: $Screen::cmaxNorth\n"
      puts stdout "cminEast: $Screen::cminEast\n"
      puts stdout "cmaxEast: $Screen::cmaxEast\n"
      puts stdout "cmaxEast: $Screen::cmaxEast\n"
      
    }

    proc zoomMap {tnorth teast} {
      global Screen::canvasCHeight Screen::canvasCWidth Screen::gapToAxis
      global Screen::cminNorth Screen::cmaxNorth \
             Screen::cminEast Screen::cmaxEast Screen::pixelsPerFeet

      set north [expr ($Screen::canvasCHeight - $Screen::gapToAxis \
                       - $Screen::canvasCenterY) / $Screen::pixelsPerFeet \
                       + $Screen::cminNorth]
      set east [expr ($Screen::canvasCenterX - $Screen::gapToAxis) \
                       / $Screen::pixelsPerFeet + $Screen::cminEast]

      puts stdout "Zoom Map at $north $east"

      set totalNorthPixels [expr $Screen::canvasCHeight-$Screen::gapToAxis -5]
      set totalEastPixels [expr $Screen::canvasCWidth - $Screen::gapToAxis -5]
      set intervalNorthPixels [expr $totalNorthPixels/10]
      set intervalEastPixels [expr $totalEastPixels/10]
      set intervalNorthValue \
                    [expr $intervalNorthPixels/$Screen::pixelsPerFeet]
      set intervalEastValue [expr $intervalEastPixels/$Screen::pixelsPerFeet]
      
      set rminNorthValue [expr $north - 2 * $intervalNorthValue]
      set rmaxNorthValue [expr $north + 2 * $intervalNorthValue]
      set rminEastValue [expr $east - 2 * $intervalEastValue]
      set rmaxEastValue [expr $east + 2 * $intervalEastValue]
      
      if { $rminNorthValue < $Screen::cminNorth } { 
        set rminNorthValue $Screen::cminNorth
      }
      
      if { $rmaxNorthValue > $Screen::cmaxNorth } { 
        set rmaxNorthValue $Screen::cmaxNorth
      }

      if { $rminEastValue < $Screen::cminEast } { 
        set rminEastValue $Screen::cminEast
      }

      if { $rmaxEastValue > $Screen::cmaxEast } { 
        set rmaxEastValue $Screen::cmaxEast
      }

      set Screen::cminNorth $rminNorthValue
      set Screen::cmaxNorth $rmaxNorthValue
      set Screen::cminEast $rminEastValue
      set Screen::cmaxEast $rmaxEastValue
      
      Screen::plotData .ftop.c2.c $rminNorthValue $rmaxNorthValue \
        $rminEastValue $rmaxEastValue $Screen::zoomInText $tnorth $teast          
    }
    
    proc zoomOutMap {tnorth teast} {
      global Screen::canvasCHeight Screen::canvasCWidth Screen::gapToAxis
      global Screen::cminNorth Screen::cmaxNorth \
             Screen::cminEast Screen::cmaxEast Screen::pixelsPerFeet

      set totalNorthPixels [expr $Screen::canvasCHeight-$Screen::gapToAxis -5]
      set totalEastPixels [expr $Screen::canvasCWidth - $Screen::gapToAxis -5]
      set intervalNorthPixels [expr $totalNorthPixels/10]
      set intervalEastPixels [expr $totalEastPixels/10]
      set intervalNorthValue \
                    [expr $intervalNorthPixels/$Screen::pixelsPerFeet]
      set intervalEastValue [expr $intervalEastPixels/$Screen::pixelsPerFeet]
      
      set rminNorthValue [expr $cminNorth - 2 * $intervalNorthValue]
      set rmaxNorthValue [expr $cmaxNorth + 2 * $intervalNorthValue]
      set rminEastValue [expr $cminEast - 2 * $intervalEastValue]
      set rmaxEastValue [expr $cmaxEast + 2 * $intervalEastValue]
      
      set Screen::cminNorth $rminNorthValue
      set Screen::cmaxNorth $rmaxNorthValue
      set Screen::cminEast $rminEastValue
      set Screen::cmaxEast $rmaxEastValue
      
      Screen::plotData .ftop.c2.c $rminNorthValue $rmaxNorthValue \
        $rminEastValue $rmaxEastValue $Screen::zoomInText $tnorth $teast          
    }

    #-------------------------------------------------------------------------
    # Local procedures
    #-------------------------------------------------------------------------
    proc pleaseStandBy { me cWidth cHeight } {
      global huge_font

      clearCanvas $me
      $me create text [expr $cWidth/2+5] [expr $cHeight/2] -font $huge_font \
           -text "Please standby,loading datalog information..."
    }

    proc clearCanvas { me } {
      foreach id [$me find all] {
         $me delete $id
      }
    }

    proc plotAxis { me } {
      global Screen::gapToAxis Screen::canvasCWidth Screen::canvasCHeight
      global Screen::canvasC1Width Screen::canvasC1Height
      
      if {$me == ".ftop.c2.c"} {
        set cWidth $Screen::canvasCWidth
        set cHeight $Screen::canvasCHeight
      } else {
        set cWidth $Screen::canvasC1Width
        set cHeight $Screen::canvasC1Height
      }
      
      # plot the NorthAxis
      $me create line $Screen::gapToAxis 0 $Screen::gapToAxis \
         [expr $cHeight - $Screen::gapToAxis] -width 2 -arrow first
              
      # plot the EastAxis        
      $me create line $Screen::gapToAxis [expr $cHeight -$Screen::gapToAxis] \
         $cWidth [expr $cHeight - $Screen::gapToAxis] -width 2 -arrow last

      $me create text [expr $Screen::gapToAxis + 10] 8 -text "N" -fill black
      $me create text [expr $cWidth - 5] \
         [expr $cHeight - $Screen::gapToAxis - 10] -text "E" -fill black
    }

    proc plotTicks {me minNorth minEast} {
      global Screen::gapToAxis Screen::canvasCWidth Screen::canvasCHeight
      global Screen::canvasC1Width Screen::canvasC1Height \
             Screen::pixelsPerFeet
      global small_font medium_font

      if {$me == ".ftop.c2.c"} {
        set cWidth $Screen::canvasCWidth
        set cHeight $Screen::canvasCHeight
      } else {
        set cWidth $Screen::canvasC1Width
        set cHeight $Screen::canvasC1Height
      }

      set intervalNorthPixels [expr ($cHeight - $Screen::gapToAxis - 5)/10]
      set intervalEastPixels [expr ($cWidth - $Screen::gapToAxis - 5)/10]
      set intervalNorthValue \
                       [expr $intervalNorthPixels/$Screen::pixelsPerFeet]
      set intervalEastValue \
                       [expr $intervalEastPixels/$Screen::pixelsPerFeet]

      # plot the NorthAxis ticks and texts
      set num 0
      set northValue [expr round($minNorth)]
      while {$num < 10} {
        $me create line [expr $Screen::gapToAxis - 5] \
           [expr $cHeight - $Screen::gapToAxis - $num*$intervalNorthPixels] \
           $Screen::gapToAxis \
           [expr $cHeight - $Screen::gapToAxis - $num*$intervalNorthPixels]
        $me create text [expr $Screen::gapToAxis - 28] \
           [expr $cHeight - $Screen::gapToAxis - $num*$intervalNorthPixels] \
           -text [expr round($northValue)] -fill blue -font $small_font
        set num [expr $num + 1]
        set northValue [expr $northValue + $intervalNorthValue]
      }

      $me create line [expr $Screen::gapToAxis - 5] \
         [expr $cHeight - $Screen::gapToAxis - $num*$intervalNorthPixels] \
         $Screen::gapToAxis \
         [expr $cHeight - $Screen::gapToAxis - $num*$intervalNorthPixels]
      $me create text [expr $Screen::gapToAxis - 28] \
         [expr $cHeight - $Screen::gapToAxis - $num*$intervalNorthPixels+3] \
         -text [expr round($northValue)] -fill blue -font $small_font
      
      # plot the EastAxis ticks and texts
      set num 0
      set eastValue [expr round($minEast)]
      while {$num < 11} {
        $me create line \
           [expr $Screen::gapToAxis + $num*$intervalEastPixels] \
           [expr $cHeight - $Screen::gapToAxis] \
           [expr $Screen::gapToAxis + $num*$intervalEastPixels] \
           [expr $cHeight - $Screen::gapToAxis + 5]
        $me create text \
           [expr $Screen::gapToAxis + $num*$intervalEastPixels - 10] \
           [expr $cHeight - $Screen::gapToAxis + 15] \
           -text [expr round($eastValue)] -fill blue -font $small_font
        set num [expr $num + 1]
        set eastValue [expr $eastValue + $intervalEastValue]
      }
    }

    proc plotCurrentPosition { me prevnorth preveast tnorth teast tyaw } {
      global Screen::gapToAxis Screen::canvasCWidth Screen::canvasCHeight
      global Screen::canvasC1Width Screen::canvasC1Height
      global Screen::cminNorth Screen::cmaxNorth
      global Screen::cminEast Screen::cmaxEast 
      global small_font

      if {$me == ".ftop.c2.c"} {
        set cWidth $Screen::canvasCWidth
        set cHeight $Screen::canvasCHeight
      } else {
        set cWidth $Screen::canvasC1Width
        set cHeight $Screen::canvasC1Height
      }
      
      set totalNorthValue [expr $Screen::cmaxNorth - $Screen::cminNorth]
      set totalEastValue [expr $Screen::cmaxEast - $Screen::cminEast]
      set totalNorthPixels [expr $cHeight - $Screen::gapToAxis - 5]
      set totalEastPixels [expr $cWidth - $Screen::gapToAxis - 5]
 
      if {[expr $totalNorthPixels / $totalNorthValue] < \
          [expr $totalEastPixels / $totalEastValue]} {
         set Screen::pixelsPerFeet [expr $totalNorthPixels / $totalNorthValue]
      } else {
         set Screen::pixelsPerFeet [expr $totalEastPixels / $totalEastValue]
      }

      $me delete tractorRect
      $me delete tractorText
      $me delete tractorArrow
      
      set x1 [expr $Screen::gapToAxis + ($preveast - $Screen::cminEast) * \
                                        $Screen::pixelsPerFeet]
      set y1 [expr $cHeight - $Screen::gapToAxis \
                 - ($prevnorth - $Screen::cminNorth) * $Screen::pixelsPerFeet]     
      set x2 [expr $Screen::gapToAxis + ($teast - $Screen::cminEast) * \
                                        $Screen::pixelsPerFeet]
      set y2 [expr $cHeight - $Screen::gapToAxis \
                  - ($tnorth - $Screen::cminNorth) * $Screen::pixelsPerFeet]

      if { ($Screen::gapToAxis < $x2) && \
           ($y2 < [expr $cHeight - $Screen::gapToAxis]) } {
        $me create line [expr $x2-2] [expr $y2-2] [expr $x2+2] [expr $y2+2] \
            -fill red -width 2
        $me create line [expr $x2-2] [expr $y2+2] [expr $x2+2] [expr $y2-2] \
            -fill red -width 2

        $me create line $x2 $y2 [expr ($x2 + 15 * sin($tyaw))] \
           [expr ($y2 - 15 * cos($tyaw))] \
           -arrow last -width 2 -fill black -tag tractorArrow
          
        $me create rectangle \
           [expr $Screen::gapToAxis - 5 + ($teast - $Screen::cminEast) * \
                                           $Screen::pixelsPerFeet] \
           [expr $cHeight - $Screen::gapToAxis - 5 - 3 \
                 - ($tnorth - $Screen::cminNorth) * $Screen::pixelsPerFeet] \
           [expr $Screen::gapToAxis + 5 \
                 + ($teast - $Screen::cminEast) * $Screen::pixelsPerFeet] \
           [expr $cHeight - $Screen::gapToAxis + 5 - 3 \
                 - ($tnorth - $Screen::cminNorth) * $Screen::pixelsPerFeet] \
           -fill red -outline black -tag tractorRect
        $me create text \
                   [expr $Screen::gapToAxis + ($teast - $Screen::cminEast) * \
                    $Screen::pixelsPerFeet] \
                   [expr $cHeight - $Screen::gapToAxis - 15 \
                  - ($tnorth - $Screen::cminNorth) * $Screen::pixelsPerFeet] \
             -text "You are here" -fill red -font $small_font -tag tractorText
      }
    }
    
    proc plotData {me rminNorth rmaxNorth rminEast rmaxEast cText tnorth \
                   teast} {
      global Screen::gapToAxis Screen::pixelsPerFeet small_font medium_font

      if {$me == ".ftop.c2.c"} {
        set cWidth $Screen::canvasCWidth
        set cHeight $Screen::canvasCHeight
      } else {
        set cWidth $Screen::canvasC1Width
        set cHeight $Screen::canvasC1Height
      }
      
      set totalNorthValue [expr $rmaxNorth - $rminNorth]
      set totalEastValue [expr $rmaxEast - $rminEast]
      set totalNorthPixels [expr $cHeight - $Screen::gapToAxis - 5]
      set totalEastPixels [expr $cWidth - $Screen::gapToAxis - 5]
      
      if {[expr $totalNorthPixels / $totalNorthValue] < \
          [expr $totalEastPixels / $totalEastValue]} {
        set Screen::pixelsPerFeet [expr $totalNorthPixels / $totalNorthValue]
      } else {
        set Screen::pixelsPerFeet [expr $totalEastPixels / $totalEastValue]
      }

      Screen::clearCanvas $me
      
      if [catch {open $Screen::datalogpath r} fileId] {
         puts stdout "Cannot open datalog.txt: $fileId\n"
      } else {
         gets $fileId point
         while { [gets $fileId point] > 0 } {
           set north [lindex $point 3]
           set east [lindex $point 4]
           if { ($rminNorth <= $north) && ($north <= $rmaxNorth) && \
                ($rminEast <= $east) && ($east <= $rmaxEast) } {
             $me create line \
               [expr $Screen::gapToAxis \
                     + ($east - $rminEast) * $Screen::pixelsPerFeet - 2] \
               [expr $cHeight - $Screen::gapToAxis \
                     - ($north - $rminNorth) * $Screen::pixelsPerFeet - 2] \
               [expr $Screen::gapToAxis \
                     + ($east - $rminEast) * $Screen::pixelsPerFeet + 2] \
               [expr $cHeight - $Screen::gapToAxis \
                     - ($north - $rminNorth) * $Screen::pixelsPerFeet + 2] \
               -fill blue -width 2
             $me create line \
               [expr $Screen::gapToAxis \
                     + ($east - $rminEast) * $Screen::pixelsPerFeet - 2] \
               [expr $cHeight - $Screen::gapToAxis \
                     - ($north - $rminNorth) * $Screen::pixelsPerFeet + 2] \
               [expr $Screen::gapToAxis \
                     + ($east - $rminEast) * $Screen::pixelsPerFeet + 2] \
               [expr $cHeight - $Screen::gapToAxis \
                     - ($north - $rminNorth) * $Screen::pixelsPerFeet - 2] \
               -fill blue -width 2
           }
         }
         close $fileId
      }
      
      Screen::plotAxis $me
      Screen::plotTicks $me $rminNorth $rminEast

      # plot the notes
      $me create text \
         [expr $cWidth / 2] [expr $cHeight - $Screen::gapToAxis + 35] \
         -text $cText -font $medium_font

      $me create rectangle \
        [expr $Screen::gapToAxis -5 \
              + ($teast - $rminEast) * $Screen::pixelsPerFeet] \
        [expr $cHeight - $Screen::gapToAxis - 5 \
              - ($tnorth - $rminNorth) * $Screen::pixelsPerFeet] \
        [expr $Screen::gapToAxis + 5 \
              + ($teast - $rminEast) *$Screen::pixelsPerFeet] \
        [expr $cHeight - $Screen::gapToAxis + 5 \
              - ($tnorth - $rminNorth) * $Screen::pixelsPerFeet] \
        -fill red -outline black -tag tractorRect
             
      $me create text \
        [expr $Screen::gapToAxis \
              + ($teast - $rminEast) * $Screen::pixelsPerFeet] \
        [expr $cHeight - $Screen::gapToAxis - 15 \
              - ($tnorth - $rminNorth) * $Screen::pixelsPerFeet] \
        -text "You are here" -fill red -font $small_font -tag tractorText

      set textX \
        [expr $Screen::gapToAxis + (0 - $rminEast) * $Screen::pixelsPerFeet]
      set textY \
        [expr $cHeight - $Screen::gapToAxis - 10 \
              - (0 - $rminNorth) * $Screen::pixelsPerFeet]
                      
      if {($Screen::gapToAxis < $textX) && \
          ($textY < [expr $cHeight - $Screen::gapToAxis])} {
        $me create text $textX $textY \
          -text "BaseStation" -fill green -font $small_font
        $me create oval \
          [expr $Screen::gapToAxis + (0 - $rminEast)*$Screen::pixelsPerFeet-5] \
          [expr $cHeight - $Screen::gapToAxis \
                - (0 - $rminNorth) * $Screen::pixelsPerFeet - 5] \
          [expr $Screen::gapToAxis + (0 - $rminEast)*$Screen::pixelsPerFeet+5] \
          [expr $cHeight - $Screen::gapToAxis \
                - (0 - $rminNorth) * $Screen::pixelsPerFeet + 5] \
          -fill green
      }

      set x [expr $Screen::gapToAxis \
                  + ($teast - $rminEast) * $Screen::pixelsPerFeet]
      set y [expr $cHeight - $Screen::gapToAxis \
                  - ($tnorth - $rminNorth) * $Screen::pixelsPerFeet]     

      $me create line $x $y $x [expr ($y - 15)] \
        -arrow last -width 2 -fill red -tag tractorArrow
    }
}
