##############################################################################
#
# vc/src/tcl/examinePos.tcl
#
# Screen to display details about Position examination.
#
# 20030421 M_S - created
#
# Copyright (c) 2002-2003 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # NOTE: using place for positioning imported frames (-in option)
    #       crashes Tcl interpreter. Therefore pack must be used.
    pack .configDisplay -in .ftop -side left \
            -anchor n -padx 10 -pady 10
    raise .configDisplay
    pack .navDisplay -in .ftop -side right -anchor n -padx 10 -pady 10
    raise .navDisplay


    # title and error indicator
    label .ftop.title -font $huge_font
    place .ftop.title -relx 0.5 -rely 0.25 -anchor n
    
    # Blade position labels
    label .ftop.northLbl -font $big_font
    place .ftop.northLbl -relx 0.25 -x 60 -rely 0.35 -anchor c
    label .ftop.eastLbl -font $big_font
    place .ftop.eastLbl -relx 0.25 -x 180 -rely 0.35 -anchor c
    label .ftop.downLbl -font $big_font
    place .ftop.downLbl -relx 0.25 -x 300 -rely 0.35 -anchor c
    label .ftop.posLbl -font $big_font
    place .ftop.posLbl -relx 0.25 -rely 0.37 -anchor ne


    # Base position labels
    label .ftop.latLbl -font $big_font
    place .ftop.latLbl -relx 0.25 -x 60 -rely 0.45 -anchor c
    label .ftop.lonLbl -font $big_font
    place .ftop.lonLbl -relx 0.25 -x 180 -rely 0.45 -anchor c
    label .ftop.altLbl -font $big_font
    place .ftop.altLbl -relx 0.25 -x 300 -rely 0.45 -anchor c
    label .ftop.rawPosLbl -font $big_font
    place .ftop.rawPosLbl -relx 0.25 -rely 0.47 -anchor ne
       
    label .ftop.posNN -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.posNN -relx 0.25 -x  10 -rely 0.37 -anchor nw
    label .ftop.posEE -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.posEE -relx 0.25 -x 130 -rely 0.37 -anchor nw
    label .ftop.posDD -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.posDD -relx 0.25 -x 250 -rely 0.37 -anchor nw
    label .ftop.rawNN -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.rawNN -relx 0.25 -x  10 -rely 0.47 -anchor nw
    label .ftop.rawEE -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.rawEE -relx 0.25 -x 130 -rely 0.47 -anchor nw
    label .ftop.rawDD -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.rawDD -relx 0.25 -x 250 -rely 0.47 -anchor nw

    #grade related labels
    label .ftop.gradeLbl -font $big_font
    place .ftop.gradeLbl -relx 0.25 -x 240 -rely 0.55 -anchor ne
    label .ftop.grade -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.grade -relx 0.25 -x 250 -rely 0.55 -anchor nw
    label .ftop.geoidLbl -font $big_font
    place .ftop.geoidLbl -relx 0.25 -x 240 -rely 0.625 -anchor ne
    label .ftop.geoid -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.geoid -relx 0.25 -x 250 -rely 0.625 -anchor nw
    label .ftop.laserLbl -font $big_font
    place .ftop.laserLbl -relx 0.25 -x 240 -rely 0.7 -anchor ne
    label .ftop.laser -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.laser -relx 0.25 -x 250 -rely 0.7 -anchor nw
    
    # buttons
    button .ftop.return \
            -height 6 -width 10 \
            -background $color5 -activebackground $color5 \
            -command {beepOnce; post $DISP_BTN9_SIG 9 }
    place .ftop.return -x 10 -rely 1.0 -y -10 -anchor sw            

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    
    proc actEntry { titleLbl returnBtn  \
                    posLbl rawPosLbl \
                    northLbl eastLbl downLbl \
                    latLbl lonLbl altLbl \
		    gradeLbl geoidLbl laserLbl } {
   
        global huge_font big_font
   
        .ftop.title configure -text $titleLbl
        .ftop.return configure -text $returnBtn
        
        .ftop.northLbl configure -text $northLbl
        .ftop.eastLbl configure -text $eastLbl
        .ftop.downLbl configure -text $downLbl

        .ftop.lonLbl configure -text $latLbl
        .ftop.latLbl configure -text $lonLbl
        .ftop.altLbl configure -text $altLbl

        .ftop.posLbl configure -text $posLbl
        .ftop.rawPosLbl configure -text $rawPosLbl

        .ftop.gradeLbl configure -text $gradeLbl
        .ftop.geoidLbl configure -text $geoidLbl
        .ftop.laserLbl configure -text $laserLbl
        pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # other action procedures (called by GuiMgr)
    #-------------------------------------------------------------------------

    proc actDisplayPos { posNN posEE posDD rawNN rawEE rawDD grade geoid laser } {
        .ftop.posNN configure -text $posNN
        .ftop.posEE configure -text $posEE
        .ftop.posDD configure -text $posDD
        .ftop.rawNN configure -text $rawNN
        .ftop.rawEE configure -text $rawEE
        .ftop.rawDD configure -text $rawDD
        .ftop.grade configure -text $grade
        .ftop.geoid configure -text $geoid
        .ftop.laser configure -text $laser
    }
}
