##############################################################################
#
# vc/src/tcl/examineSteeringDisplay.tcl
#
# Screen to display details about Steering examination.
#
# 20021112 QC - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout
    # (Screen has custom display and input frames, and reuses the .logo,
    # .configDisplay, and .navDisplay components)
    frame .ftop.title
    frame .ftop.input
    pack .ftop.input -side bottom -fill both -expand yes \
            -padx 5 -pady 3
    pack .ftop.title -side bottom -fill both -expand yes
    pack .configDisplay -in .ftop -side left \
            -anchor nw -fill y -expand yes -padx 10 -pady 10
    raise .configDisplay
    pack .navDisplay -in .ftop -side right -anchor ne -padx 10 -pady 10
    raise .navDisplay

    # Title
    label .ftop.title.prompt -text "Space for title"
    pack .ftop.title.prompt -side top

    # Row 1
    frame .ftop.input.r1
    label .ftop.input.r1.l0 -text "Wheel angle:" \
          -font $big_font -anchor e
    label .ftop.input.r1.e0 -text "3.20 deg" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.e1 -text "1962" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.l1 -text "Pots:" \
          -font $big_font -anchor e
    label .ftop.input.r1.e2 -text "32521" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.e3 -text "32536" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.e4 -text "32313" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.l2 -text "Kill switch:" \
          -font $big_font
    label .ftop.input.r1.e5 -text "OFF" -font $big_font \
          -width 8 -relief sunken -borderwidth 2        
    label .ftop.input.r1.l -text "" -font $small_font
       
    # Row 1a
    frame .ftop.input.r1a
    label .ftop.input.r1a.l1 -text "Spans:" \
          -font $big_font -anchor e
    label .ftop.input.r1a.e2 -text "32521" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1a.e3 -text "32536" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1a.e4 -text "32313" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
       
    # Row 2
    frame .ftop.input.r2
    label .ftop.input.r2.l1 -text "Left transducer:" -font $big_font -anchor w
    label .ftop.input.r2.l2 -text "Right transducer:" -font $big_font
    label .ftop.input.r2.l3 -text "Joystick:" -font $big_font
    label .ftop.input.r2.e0 -text "11" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r2.e1 -text "12" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r2.e2 -text "13" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    
    # Row 2a
    frame .ftop.input.r2a
    label .ftop.input.r2a.l1 -text "Seat Switch" -font $big_font -anchor w
    label .ftop.input.r2a.l2 -text "Max" -font $big_font
    label .ftop.input.r2a.e0 -text "" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r2a.e1 -text "" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    
    # Row 3
    frame .ftop.input.r3
    button .ftop.input.r3.cancel -text "Return To\nSystem Menu" \
            -width 10 -height 6 \
            -background $color5 -activebackground $color5 \
            -command {beepOnce; post $DISP_BTN9_SIG 9}
    button .ftop.input.r3.hardleft -text "100%\nLeft" \
            -width 6 -height 6 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN1_SIG 1}
    button .ftop.input.r3.left -text "50%\nLeft" \
            -width 6 -height 6 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN2_SIG 2}
    button .ftop.input.r3.zero -text "Zero" \
            -width 6 -height 6 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN3_SIG 3}
    button .ftop.input.r3.right -text "50%Right" \
            -width 6 -height 6 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN4_SIG 4}
    button .ftop.input.r3.hardright -text "100%\nRight" \
            -width 6 -height 6 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN5_SIG 5}
    button .ftop.input.r3.change -text "Change\nPercent" \
            -width 10 -height 6 \
            -background $colorMedium -activebackground $colorMedium \
            -command {beepOnce; post $DISP_BTN6_SIG 6}
    button .ftop.input.r3.stop -image bigSTOP \
            -command { beepOnce; post $DISP_BTN0_SIG 0 "stop autosteer" } \
            -background $color6 -activebackground $color6 \
            -foreground red -activeforeground red  \
            -borderwidth 0

    pack .ftop.input.r3.cancel -side left -padx 5 -pady 10
    pack .ftop.input.r3.hardleft -side left
    pack .ftop.input.r3.left -side left
    pack .ftop.input.r3.zero -side left
    pack .ftop.input.r3.right -side left
    pack .ftop.input.r3.hardright -side left
    pack .ftop.input.r3.change -side right -padx 5 -pady 10

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { packProc__ } {
        
        eval $packProc__
        pack .ftop -fill both -expand yes

    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actDisplayDeereTrack {pot1 pot2 pot3 killswitch} {
        .ftop.input.r1.e2 configure -text $pot1
        .ftop.input.r1.e3 configure -text $pot2
        .ftop.input.r1.e4 configure -text $pot3
        .ftop.input.r1.e5 configure -text $killswitch
    }

    proc actDisplayWheeled {wheelangle rawheelangle \
            killswitch leftrans rightrans} {
        .ftop.input.r1.e0 configure -text $wheelangle
        .ftop.input.r1.e1 configure -text $rawheelangle
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r2.e0 configure -text $leftrans
        .ftop.input.r2.e1 configure -text $rightrans
    }

    proc actDisplayCatTrack {killswitch leftrans rightrans} {
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r2.e0 configure -text $leftrans
        .ftop.input.r2.e1 configure -text $rightrans
    }

    proc actDisplayMT {killswitch msg} {
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r1.l configure -text $msg
    }

    proc actDisplayGantry {wheelangle rawheelangle \
            killswitch leftpot rightpot joystick seatSw seatSwMax} {
        .ftop.input.r1.e0 configure -text $wheelangle
        .ftop.input.r1.e1 configure -text $rawheelangle
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r2.e0 configure -text $leftpot
        .ftop.input.r2.e1 configure -text $rightpot
        .ftop.input.r2.e2 configure -text $joystick
        .ftop.input.r2a.e0 configure -text $seatSw
        .ftop.input.r2a.e1 configure -text $seatSwMax
    }

    proc actDisplayCamecoHarv {sig0 sig1 sig2 span0 span1 span2 killswitch} {
        .ftop.input.r1.e2 configure -text $sig0
        .ftop.input.r1.e3 configure -text $sig1
        .ftop.input.r1.e4 configure -text $sig2
        .ftop.input.r1a.e2 configure -text $span0
        .ftop.input.r1a.e3 configure -text $span1
        .ftop.input.r1a.e4 configure -text $span2
        .ftop.input.r1.e5 configure -text $killswitch
    }

    proc actDisplayPercent { leftPercentBtn rightPercentBtn } {
        .ftop.input.r3.left configure -text $leftPercentBtn
        .ftop.input.r3.right configure -text $rightPercentBtn
    }

    proc actConfigureAutosteer { } {
        pack forget .ftop.input.r3.change
        pack .ftop.input.r3.stop -after .ftop.input.r3.hardright \
                                 -padx 6 -pady 10
    }

    proc actConfigureStopAutosteer { } {
        global color3

        pack forget .ftop.input.r3.stop
        pack .ftop.input.r3.change -after .ftop.input.r3.hardright \
                                   -padx 5 -pady 10
        
        .ftop.input.r3.hardright configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.left configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.zero configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.right configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardleft configure \
                                -background $color3 -activebackground $color3
        
    }

    proc actConfigureHardLeft { } {
        global color3
        .ftop.input.r3.hardleft configure \
                                -background green -activebackground green
        .ftop.input.r3.left configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.zero configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.right configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardright configure \
                                -background $color3 -activebackground $color3
    }
 
    proc actConfigureLeft { } {
        global color3
        .ftop.input.r3.left configure \
                                -background green -activebackground green
        .ftop.input.r3.hardleft configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.zero configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.right configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardright configure \
                                -background $color3 -activebackground $color3
    }

    proc actConfigureZero { } {
        global color3
        .ftop.input.r3.zero configure \
                                -background green -activebackground green
        .ftop.input.r3.left configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardleft configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.right configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardright configure \
                                -background $color3 -activebackground $color3
    }

    proc actConfigureRight { } {
        global color3
        .ftop.input.r3.right configure \
                                -background green -activebackground green
        .ftop.input.r3.left configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.zero configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardleft configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardright configure \
                                -background $color3 -activebackground $color3
    }

    proc actConfigureHardRight { } {
        global color3
        .ftop.input.r3.hardright configure \
                                -background green -activebackground green
        .ftop.input.r3.left configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.zero configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.right configure \
                                -background $color3 -activebackground $color3
        .ftop.input.r3.hardleft configure \
                                -background $color3 -activebackground $color3
    }

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
    proc packDeereTrack { title potsLbl killswitchLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l1 configure -text $potsLbl
        pack .ftop.input.r1.l1 -side left
        pack .ftop.input.r1.e2 -side left -padx 10
        pack .ftop.input.r1.e3 -side left
        pack .ftop.input.r1.e4 -side left -padx 10
        pack .ftop.input.r1.l2 -side left -padx 1
        pack .ftop.input.r1.e5 -side left

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 10
    }

    proc packWheeled { title wheelLbl killswitchLbl leftLbl rightLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left
        pack .ftop.input.r1.e1 -side left
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left

        # Row 2
        .ftop.input.r2.l1 configure -text $leftLbl
        .ftop.input.r2.l2 configure -text $rightLbl
        pack .ftop.input.r2.l1 -side left
        pack .ftop.input.r2.e0 -side left -padx 10
        pack .ftop.input.r2.l2 -side left
        pack .ftop.input.r2.e1 -side left -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 10
    }

    proc packCatTrack { title killswitchLbl leftLbl rightLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left

        # Row 2
        .ftop.input.r2.l1 configure -text $leftLbl
        .ftop.input.r2.l2 configure -text $rightLbl
        pack .ftop.input.r2.l1 -side left
        pack .ftop.input.r2.e0 -side left -padx 10
        pack .ftop.input.r2.l2 -side left
        pack .ftop.input.r2.e1 -side left -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 10
    }

    proc packMT { title killswitchLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left
        pack .ftop.input.r1.l -side left -padx 30
        
        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 10
    }

    proc packGantry { title wheelLbl killswitchLbl leftLbl rightLbl \
            joyLbl seatSwLbl seatSwMaxLbl hardleftBtn leftBtn zeroBtn \
            rightBtn hardrightBtn returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left
        pack .ftop.input.r1.e1 -side left
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left

        # Row 2
        .ftop.input.r2.l1 configure -text $leftLbl
        .ftop.input.r2.l2 configure -text $rightLbl
        .ftop.input.r2.l3 configure -text $joyLbl
        pack .ftop.input.r2.l1 -side left
        pack .ftop.input.r2.e0 -side left -padx 10
        pack .ftop.input.r2.l2 -side left
        pack .ftop.input.r2.e1 -side left -padx 10
        pack .ftop.input.r2.l3 -side left
        pack .ftop.input.r2.e2 -side left -padx 10

        # Row 3
        .ftop.input.r2a.l1 configure -text $seatSwLbl 
        .ftop.input.r2a.l2 configure -text $seatSwMaxLbl 
        pack .ftop.input.r2a.l1 -side left
        pack .ftop.input.r2a.e0 -side left -padx 10
        pack .ftop.input.r2a.l2 -side left
        pack .ftop.input.r2a.e1 -side left -padx 10

        # Row 4
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r2a -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 10
    }

    proc packCamecoHarv { title signalLbl spanLbl killswitchLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l1 configure -text $signalLbl
        pack .ftop.input.r1.l1 -side left
        pack .ftop.input.r1.e2 -side left -padx 10
        pack .ftop.input.r1.e3 -side left
        pack .ftop.input.r1.e4 -side left -padx 10
        pack .ftop.input.r1.l2 -side left -padx 2
        pack .ftop.input.r1.e5 -side left

        # Row 1a
        .ftop.input.r1a.l1 configure -text $spanLbl
        pack .ftop.input.r1a.l1 -side left
        pack .ftop.input.r1a.e2 -side left -padx 10
        pack .ftop.input.r1a.e3 -side left
        pack .ftop.input.r1a.e4 -side left -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r1a -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 10
    }
}

