##############################################################################
#
# vc/src/tcl/implementDisplay.tcl
#
# Screen to display details about and edit an existing implement. Used when
# adding or editing an implement.
#
# 20020830 QC - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout

    # Title
    label .ftop.title -text "??Space for title"

    # Input frame
    frame .ftop.input

    # Row 1
    frame .ftop.input.r1
    label .ftop.input.r1.l -text "Name:" -font $big_font -width 10 -anchor e
    label .ftop.input.r1.e -text "foo" -font $big_font \
                -width 32 -anchor w -relief sunken -borderwidth 2
    pack .ftop.input.r1.l -side left 
    pack .ftop.input.r1.e -side left -padx 10 
    SnazzyButton::create .ftop.input.r1.b btnBlueMU btnBlueMD 0.5 \
                         "beepOnce; post $DISP_BTN2_SIG 2;" "Change"
       
    # Row 2
    frame .ftop.input.r2
    label .ftop.input.r2.l -text "Type:" -font $big_font -width 10 -anchor e
    label .ftop.input.r2.e -text "Hitched" -font $big_font \
                  -width 32 -anchor w -relief sunken -borderwidth 2
    pack .ftop.input.r2.l -side left -anchor w
    pack .ftop.input.r2.e -side left -padx 10 
    SnazzyButton::create .ftop.input.r2.b btnBlueMU btnBlueMD 0.5 \
                         "beepOnce; post $DISP_BTN3_SIG 3;" "Change"
    
    # Row 3
    frame .ftop.input.r3
    label .ftop.input.r3.l -text "Width:" -font $big_font -width 10 -anchor e
    label .ftop.input.r3.e -text "xx feet xx inch" -font $big_font \
                  -width 32 -anchor w -relief sunken -borderwidth 2
    pack .ftop.input.r3.l -side left -anchor w
    pack .ftop.input.r3.e -side left -padx 10 
    SnazzyButton::create .ftop.input.r3.b btnBlueMU btnBlueMD 0.5 \
                         "beepOnce; post $DISP_BTN4_SIG 4;" "Change"

    # Row 4
    frame .ftop.input.r4
    label .ftop.input.r4.l -text "Offset:" -font $big_font -width 10 -anchor e
    label .ftop.input.r4.e -text "xx feet xx inch" -font $big_font \
                  -width 32 -anchor w -relief sunken -borderwidth 2
    pack .ftop.input.r4.l -side left -anchor w
    pack .ftop.input.r4.e -side left -padx 10 
    SnazzyButton::create .ftop.input.r4.b btnBlueMU btnBlueMD 0.5 \
                         "beepOnce; post $DISP_BTN5_SIG 5;" "Change"

    # Row 5
    frame .ftop.input.r5
    label .ftop.input.r5.l -text "Deadband:" -font $big_font -width 10 -anchor e
    label .ftop.input.r5.e -text "xx feet xx inch" -font $big_font \
                  -width 32 -anchor w -relief sunken -borderwidth 2
    pack .ftop.input.r5.l -side left -pady 5 -anchor w
    pack .ftop.input.r5.e -side left -padx 10 -pady 5
    SnazzyButton::create .ftop.input.r5.b btnBlueMU btnBlueMD 0.5 \
                         "beepOnce; post $DISP_BTN6_SIG 5;" "Change"

    # Pack rows
    pack .ftop.input.r1 -side top -anchor e -fill x
    pack .ftop.input.r2 -side top -anchor e -fill x
    pack .ftop.input.r3 -side top -anchor e -fill x
    pack .ftop.input.r4 -side top -anchor e -fill x
#    pack .ftop.input.r5 -side top -anchor e -fill x
    
    # Cancel/Accept frame
    frame .ftop.input2
    SnazzyButton::create .ftop.input2.cancel btnKhakiU btnKhakiD 0.5 \
                         "beepOnce; post $DISP_BTN0_SIG 0;" "Cancel"
    SnazzyButton::create .ftop.input2.accept btnBlueU btnBlueD 0.5 \
                         "beepOnce; post $DISP_BTN1_SIG 4;" "Accept"                    
    SnazzyButton::create .ftop.input2.calibrate btnBlueU btnBlueD 0.5 \
                         "beepOnce; post $DISP_BTN1_SIG 0;" "Calibrate"
        
    pack .ftop.input2.cancel -side left  -pady 10
    pack .ftop.input2.calibrate -side right -padx 10 -pady 10
    pack .ftop.input2.accept -side right  -padx 10 -pady 10

    frame .ftop.input3
    SnazzyButton::create .ftop.input3.up UpNudgeU UpNudgeD 0.5 \
                         "beepOnce; post $DISP_BTN1_SIG 3"
    SnazzyButton::create .ftop.input3.down downNudgeU downNudgeD 0.5 \
                         "beepOnce; post $DISP_BTN1_SIG 2"
    SnazzyButton::create .ftop.input3.stop btnSmallStopU btnSmallStopD 0.5 \
                         "beepOnce; post $DISP_BTN1_SIG 1"

    pack .ftop.input3.down -side bottom  -pady 10
    pack .ftop.input3.stop -side bottom  -padx 20 -pady 10
    pack .ftop.input3.up -side bottom -pady 10

    # Pack .ftop
    pack .ftop.title -side top -pady 10 -fill both -expand yes
    pack .ftop.input2 -side bottom -anchor se -fill both -expand yes
    pack .ftop.input3 -side left -anchor w -fill both -expand yes
    pack .ftop.input -side top -pady 10 -pady 5
                
    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { packProc__ } {

        eval $packProc__
        pack .ftop -fill both -expand yes

    }


    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actDisplayImplement {name__ type__ width__ offset__} {
        .ftop.input.r1.e configure -text $name__
        .ftop.input.r2.e configure -text $type__
        .ftop.input.r3.e configure -text $width__
        .ftop.input.r4.e configure -text $offset__
    }

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
    proc packAdd { addImpl nameLbl typeLbl widthLbl offsetLbl cancelBtn \
                   acceptBtn } {
        .ftop.title configure -text $addImpl
        .ftop.input.r1.l configure -text $nameLbl
        .ftop.input.r2.l configure -text $typeLbl
        .ftop.input.r3.l configure -text $widthLbl
        .ftop.input.r4.l configure -text $offsetLbl
        SnazzyButton::setText .ftop.input2.cancel $cancelBtn 0.5
        SnazzyButton::setText .ftop.input2.accept $acceptBtn 0.5
    }


    proc packRetrieve { retrieveImpl cancelBtn } {
        SnazzyButton::setText .ftop.input2.cancel $cancelBtn 0.5
        SnazzyButton::setText .ftop.input2.accept $acceptBtn 0.5
    }

    proc packDisplay { editImpl nameLbl typeLbl widthLbl offsetLbl\
                    cancelBtn acceptBtn calibrateBtn} {
        pack forget .ftop.input.r1.b 
        pack forget .ftop.input.r2.b 
        pack forget .ftop.input.r3.b 
        pack forget .ftop.input.r4.b 
        pack forget .ftop.input.r5.b 
	pack forget .ftop.input2.accept

        .ftop.title configure -text $editImpl
        .ftop.input.r1.l configure -text $nameLbl

        .ftop.input.r2.l configure -text $typeLbl

        .ftop.input.r3.l configure -text $widthLbl

        .ftop.input.r4.l configure -text $offsetLbl

        SnazzyButton::setText .ftop.input2.cancel $cancelBtn 0.5
        SnazzyButton::setText .ftop.input2.accept $acceptBtn 0.5
        SnazzyButton::setText .ftop.input2.calibrate $calibrateBtn 0.5
    }

    proc setControlState { state } {
	SnazzyButton::setState .ftop.input3.up $state	
	SnazzyButton::setState .ftop.input3.down $state	
	SnazzyButton::setState .ftop.input3.stop $state	
	SnazzyButton::setState .ftop.input2.calibrate $state	
    }
    proc packEdit { editImpl nameLbl typeLbl widthLbl offsetLbl changeBtn\
                    cancelBtn acceptBtn calibrateBtn} {
        
	pack forget .ftop.input2.accept
        pack .ftop.input.r1.b -side right
        pack .ftop.input.r2.b -side right
        pack .ftop.input.r3.b -side right
        pack .ftop.input.r4.b -side right   
        pack .ftop.input.r5.b -side right   

        .ftop.title configure -text $editImpl
        .ftop.input.r1.l configure -text $nameLbl
        SnazzyButton::setText .ftop.input.r1.b $changeBtn 0.5 

        .ftop.input.r2.l configure -text $typeLbl
        SnazzyButton::setText .ftop.input.r2.b $changeBtn 0.5

        .ftop.input.r3.l configure -text $widthLbl
        SnazzyButton::setText .ftop.input.r3.b $changeBtn 0.5

        .ftop.input.r4.l configure -text $offsetLbl
        SnazzyButton::setText .ftop.input.r4.b $changeBtn 0.5

        SnazzyButton::setText .ftop.input2.cancel $cancelBtn 0.5
        SnazzyButton::setText .ftop.input2.accept $acceptBtn 0.5
        SnazzyButton::setText .ftop.input2.calibrate $calibrateBtn 0.5
      }
}
