##############################################################################
#
# vc/src/tcl/curvejobedit.tcl
#
# Screen for editing CurveJobs.
#
# 20030704 MLE - created
#
# Copyright (c) 2002,2003 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout
    label .ftop.title -text "title"
    frame .ftop.display         ;# Display Frame
    frame .ftop.input           ;# Input Frame
    pack .ftop.title -side top -pady 15
    pack .ftop.display -side top -fill both -expand yes
    pack .ftop.input -side top -fill both -expand yes -anchor w

    # Display Frame - displays up to 5 items with labels, values, and a 
    # change button 
    # Labels
    frame .ftop.display.l
    label .ftop.display.l.l0 -text "line 0" -font $big_font -justify right
    label .ftop.display.l.l1 -text "line 1" -font $big_font -justify right
    label .ftop.display.l.l2 -text "line 2" -font $big_font -justify right
    label .ftop.display.l.l3 -text "line 3" -font $big_font -justify right
    label .ftop.display.l.l4 -text "line 4" -font $big_font -justify right
    # Entries (actually sunken labels)
    frame .ftop.display.e
    label .ftop.display.e.e0 -text "entry 0" -font $big_font \
            -width 32 -anchor w -relief sunken -borderwidth 2 -justify left
    label .ftop.display.e.e1 -text "entry 1" -font $big_font \
            -width 32 -anchor w -relief sunken -borderwidth 2 -justify left
    label .ftop.display.e.e2 -text "entry 2" -font $big_font \
            -width 32 -anchor w -relief sunken -borderwidth 2 -justify left
    label .ftop.display.e.e3 -text "entry 3" -font $big_font \
            -width 32 -anchor w -relief sunken -borderwidth 2 -justify left
    label .ftop.display.e.e4 -text "entry 4" -font $big_font \
            -width 32 -anchor w -relief sunken -borderwidth 2 -justify left
    # Change buttons
    frame .ftop.display.b
    button .ftop.display.b.b0 -text "Change" -justify right \
            -height 2 -width 8 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN4_SIG 0; } 
    button .ftop.display.b.b1 -text "Change" -justify right \
            -height 2 -width 8 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN4_SIG 1; } 
    button .ftop.display.b.b2 -text "Change" -justify right \
            -height 2 -width 8 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN4_SIG 2; } 
    button .ftop.display.b.b3 -text "Change" -justify right \
            -height 2 -width 8 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN4_SIG 3; } 
    button .ftop.display.b.b4 -text "Change" -justify right \
            -height 2 -width 8 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN4_SIG 4; } 

    pack .ftop.display.l -side left -fill both -expand yes -anchor n
    pack .ftop.display.e -side left -anchor n
    pack .ftop.display.b -side left -fill both -expand yes -anchor n

    # Input frame 
    #   - up to 1 rows of up to 4 buttons each
    frame .ftop.input.r1
    button .ftop.input.r1.b0 -text "Button 0" \
            -height 6 -width 10 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN0_SIG 0 }
    button .ftop.input.r1.b1 -text "Button 1" \
            -height 6 -width 10 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN1_SIG 1 }
    button .ftop.input.r1.b2 -text "Button 2" \
            -height 6 -width 10 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN2_SIG 2 }
    button .ftop.input.r1.b3 -text "Button 3" \
            -height 6 -width 10 \
            -background $color3 -activebackground $color3 \
            -command {beepOnce; post $DISP_BTN3_SIG 3 }

    # Cancel button
    button .ftop.input.cancel -text "Cancel" \
            -width 10 -height 6 \
            -background $color5 -activebackground $color5 \
            -command {beepOnce; post $DISP_BTN9_SIG 9 }

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { packProc__ } {
        
        eval $packProc__
        pack .ftop -fill both -expand yes

    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }


    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actDisplayStraight {name nudge} {
        .ftop.display.e.e0 configure -text $name
        .ftop.display.e.e1 configure -text $nudge
        # {MLE-030704: tbd...}
        #ftop.display.e.e2 configure -text $nCoordinate
        #ftop.display.e.e3 configure -text $eCoordinate
        #ftop.display.e.e4 configure -text $heading
    }

    proc actDisplayFan {name nudge} {
        .ftop.display.e.e0 configure -text $name
        .ftop.display.e.e1 configure -text $nudge
        # {MLE-030704: tbd...}
        #.ftop.input.rnum.e configure -text $numofrows
        #.ftop.input.rmax.e configure -text $maxspacing
        #.ftop.input.rmin.e configure -text $minspacing
    }

    proc actDisplayBox {name nudge} {
        .ftop.display.e.e0 configure -text $name
        .ftop.display.e.e1 configure -text $nudge
    }

    proc actDisplayCircle {name nudge} {
        .ftop.display.e.e0 configure -text $name
        .ftop.display.e.e1 configure -text $nudge
    }

    proc actDisplayCurve {name nudge} {
        .ftop.display.e.e0 configure -text $name
        .ftop.display.e.e1 configure -text $nudge
    }


    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
    proc packJobEditStraight {title \
            nameLabel nameBtn nudgeLabel nudgeBtn\
            cancelBtn acceptBtn} {

        global huge_font big_font colorGood color3 color0
        
        # Configure title
        .ftop.title configure -text $title -font $huge_font

        # Configure and pack display
        .ftop.display.l.l0 configure -text $nameLabel -font $big_font
        .ftop.display.l.l1 configure -text $nudgeLabel -font $big_font

        .ftop.display.e.e0 configure -text " " -font $big_font
        .ftop.display.e.e1 configure -text " " -font $big_font

        .ftop.display.b.b0 configure -text $nameBtn -font $big_font
        .ftop.display.b.b1 configure -text $nudgeBtn -font $big_font

        pack .ftop.display.l.l0 -side top -pady 18
        pack .ftop.display.l.l1 -side top -pady 18
        pack .ftop.display.e.e0 -side top -pady 18
        pack .ftop.display.e.e1 -side top -pady 18
        pack .ftop.display.b.b0 -side top -pady 5
        pack .ftop.display.b.b1 -side top -pady 5
        
        # Configure and pack input
        .ftop.input.cancel configure -text $cancelBtn
        .ftop.input.r1.b0 configure -text $acceptBtn

        pack .ftop.input.cancel -side left -padx 5 -pady 5 -anchor sw
        pack .ftop.input.r1.b0 -side right -padx 5 -pady 5
        pack .ftop.input.r1 -side bottom -anchor se

    }
    
    proc packJobEditFan {title \
            nameLabel nameBtn nudgeLabel nudgeBtn\
            cancelBtn acceptBtn} {

        global huge_font big_font colorGood color3 color0
        
        # Configure title
        .ftop.title configure -text $title -font $huge_font

        # Configure and pack display
        .ftop.display.l.l0 configure -text $nameLabel -font $big_font
        .ftop.display.l.l1 configure -text $nudgeLabel -font $big_font

        .ftop.display.e.e0 configure -text " " -font $big_font
        .ftop.display.e.e1 configure -text " " -font $big_font

        .ftop.display.b.b0 configure -text $nameBtn -font $big_font
        .ftop.display.b.b1 configure -text $nudgeBtn -font $big_font

        pack .ftop.display.l.l0 -side top -pady 18
        pack .ftop.display.l.l1 -side top -pady 18
        pack .ftop.display.e.e0 -side top -pady 18
        pack .ftop.display.e.e1 -side top -pady 18
        pack .ftop.display.b.b0 -side top -pady 5
        pack .ftop.display.b.b1 -side top -pady 5
        
        # Configure and pack input
        .ftop.input.cancel configure -text $cancelBtn
        .ftop.input.r1.b0 configure -text $acceptBtn

        pack .ftop.input.cancel -side left -padx 5 -pady 5 -anchor sw
        pack .ftop.input.r1.b0 -side right -padx 5 -pady 5
        pack .ftop.input.r1 -side bottom -anchor se

    }
    
    proc packJobEditBox {title \
            nameLabel nameBtn nudgeLabel nudgeBtn\
            cancelBtn acceptBtn} {

        global huge_font big_font colorGood color3 color0
        
        # Configure title
        .ftop.title configure -text $title -font $huge_font

        # Configure and pack display
        .ftop.display.l.l0 configure -text $nameLabel -font $big_font
        .ftop.display.l.l1 configure -text $nudgeLabel -font $big_font

        .ftop.display.e.e0 configure -text " " -font $big_font
        .ftop.display.e.e1 configure -text " " -font $big_font

        .ftop.display.b.b0 configure -text $nameBtn -font $big_font
        .ftop.display.b.b1 configure -text $nudgeBtn -font $big_font

        pack .ftop.display.l.l0 -side top -pady 18
        pack .ftop.display.l.l1 -side top -pady 18
        pack .ftop.display.e.e0 -side top -pady 18
        pack .ftop.display.e.e1 -side top -pady 18
        pack .ftop.display.b.b0 -side top -pady 5
        pack .ftop.display.b.b1 -side top -pady 5
        
        # Configure and pack input
        .ftop.input.cancel configure -text $cancelBtn
        .ftop.input.r1.b0 configure -text $acceptBtn

        pack .ftop.input.cancel -side left -padx 5 -pady 5 -anchor sw
        pack .ftop.input.r1.b0 -side right -padx 5 -pady 5
        pack .ftop.input.r1 -side bottom -anchor se

    }
    
    proc packJobEditCircle {title \
            nameLabel nameBtn nudgeLabel nudgeBtn\
            cancelBtn acceptBtn} {

        global huge_font big_font colorGood color3 color0
        
        # Configure title
        .ftop.title configure -text $title -font $huge_font

        # Configure and pack display
        .ftop.display.l.l0 configure -text $nameLabel -font $big_font
        .ftop.display.l.l1 configure -text $nudgeLabel -font $big_font

        .ftop.display.e.e0 configure -text " " -font $big_font
        .ftop.display.e.e1 configure -text " " -font $big_font

        .ftop.display.b.b0 configure -text $nameBtn -font $big_font
        .ftop.display.b.b1 configure -text $nudgeBtn -font $big_font

        pack .ftop.display.l.l0 -side top -pady 18
        pack .ftop.display.l.l1 -side top -pady 18
        pack .ftop.display.e.e0 -side top -pady 18
        pack .ftop.display.e.e1 -side top -pady 18
        pack .ftop.display.b.b0 -side top -pady 5
        pack .ftop.display.b.b1 -side top -pady 5
        
        # Configure and pack input
        .ftop.input.cancel configure -text $cancelBtn
        .ftop.input.r1.b0 configure -text $acceptBtn

        pack .ftop.input.cancel -side left -padx 5 -pady 5 -anchor sw
        pack .ftop.input.r1.b0 -side right -padx 5 -pady 5
        pack .ftop.input.r1 -side bottom -anchor se

    }
    
    proc packJobEditCurve {title \
            nameLabel nameBtn nudgeLabel nudgeBtn\
            cancelBtn acceptBtn} {

        global huge_font big_font colorGood color3 color0
        
        # Configure title
        .ftop.title configure -text $title -font $huge_font

        # Configure and pack display
        .ftop.display.l.l0 configure -text $nameLabel -font $big_font
        .ftop.display.l.l1 configure -text $nudgeLabel -font $big_font

        .ftop.display.e.e0 configure -text " " -font $big_font
        .ftop.display.e.e1 configure -text " " -font $big_font

        .ftop.display.b.b0 configure -text $nameBtn -font $big_font
        .ftop.display.b.b1 configure -text $nudgeBtn -font $big_font

        pack .ftop.display.l.l0 -side top -pady 18
        pack .ftop.display.l.l1 -side top -pady 18
        pack .ftop.display.e.e0 -side top -pady 18
        pack .ftop.display.e.e1 -side top -pady 18
        pack .ftop.display.b.b0 -side top -pady 5
        pack .ftop.display.b.b1 -side top -pady 5
        
        # Configure and pack input
        .ftop.input.cancel configure -text $cancelBtn
        .ftop.input.r1.b0 configure -text $acceptBtn

        pack .ftop.input.cancel -side left -padx 5 -pady 5 -anchor sw
        pack .ftop.input.r1.b0 -side right -padx 5 -pady 5
        pack .ftop.input.r1 -side bottom -anchor se

    }
    
}

