##############################################################################
#
# vc/src/tcl/msgbox.tcl
#
# A modal-dialog-type msgbox.
#
# 20020625 MLE - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Modal { 

    #-------------------------------------------------------------------------
    # Define and pack the msgbox
    #-------------------------------------------------------------------------
    # Top level frame
    frame .fmodal

    # msgbox frame
    frame .fmodal.msgbox

    # display frame
    frame .fmodal.msgbox.display
    label .fmodal.msgbox.display.msg -text "Space for the message"
    pack .fmodal.msgbox.display.msg -side top -padx 5 -pady 10 \
            -fill both -expand yes

    # Input frame
    frame .fmodal.msgbox.input

    # Cancel button
    SnazzyButton::create .fmodal.msgbox.input.b0 \
                         btnKhakiU btnKhakiD 0.5 \
                         "beepOnce; post $DISP_BTN0_SIG 0"
#    button .fmodal.msgbox.input.b0 -text "Cancel" \
#            -width 10 -height 6 \
#            -background $color5 -activebackground $color5 \
#            -command {beepOnce; post $DISP_BTN0_SIG 0 "cancel";}

    # Confirm button
    SnazzyButton::create .fmodal.msgbox.input.b1 btnAcceptU btnAcceptD 0.3 \
                         "beepOnce; post $DISP_BTN1_SIG 1"
#    button .fmodal.msgbox.input.b1 -text "Confirm" \
#            -width 10 -height 6 \
#            -background $color3 -activebackground $color3 \
#            -command {beepOnce; post $DISP_BTN1_SIG 1 "confirm";}

    # Reject button
    SnazzyButton::create .fmodal.msgbox.input.b2 btnRejectU btnRejectD 0.3 \
                         "beepOnce; post $DISP_BTN2_SIG 2"
#    button .fmodal.msgbox.input.b2 -text "Reject" \
#            -width 10 -height 6 \
#            -background $color3 -activebackground $color3 \
#            -command {beepOnce; post $DISP_BTN2_SIG 2 "reject";}

    # Pack msgbox
    pack .fmodal.msgbox.display -side top -pady 10 -fill both -expand yes
    pack .fmodal.msgbox.input -side bottom -fill x -anchor se

    # Pack various common elements
    pack .fmodal.msgbox -side bottom -fill both -expand yes -anchor s

    #-------------------------------------------------------------------------
    # Entry and exit procedures
    #-------------------------------------------------------------------------
    proc actEntry {} {
        pack forget .ftop
        pack .fmodal -fill both -expand yes
    }

    proc actExit {} {
        destroy .fmodal
        pack .ftop -fill both -expand yes
    }

    #-------------------------------------------------------------------------
    # Action procedures
    #-------------------------------------------------------------------------
    proc actConfigure { msg__ btn0Label__ btn1Label__ btn2Label__ } {
        global color6 target_platform
        .fmodal.msgbox.display.msg configure -text $msg__
        if { $btn0Label__ != "!"} {
           SnazzyButton::setText .fmodal.msgbox.input.b0 $btn0Label__ 0.5
#            .fmodal.msgbox.input.b0 configure -text $btn0Label__
           pack .fmodal.msgbox.input.b0 -side left -padx 5 -pady 5 \
                -anchor sw
        }
        if { $btn2Label__ != "!"} {
           SnazzyButton::setText .fmodal.msgbox.input.b2 $btn2Label__ 0.3
#            .fmodal.msgbox.input.b2 configure -text $btn2Label__
            pack .fmodal.msgbox.input.b2 -side left -padx 5 -pady 5 \
                -anchor sw
        }
        if { $btn1Label__ != "!"} {
           SnazzyButton::setText .fmodal.msgbox.input.b1 $btn1Label__ 0.3
#            .fmodal.msgbox.input.b1 configure -text $btn1Label__
            pack .fmodal.msgbox.input.b1 -side right -padx 5 -pady 5 \
                -anchor se
        }
        #RCM: 060221 Added platform awareness to TCL to fix RGB color 
        #     swapping problem.  TCL on the MPC5200 platform seems to swap
        #     the RGB values causing red to be displayed as blue
        #     instead of the intended color.  This fix is intended to be
        #     temporary and eventually will need to be fixed once the 
        #     undelying TCL error is resolved.  This fix has also been 
        #     implemented in autofarm2.tcl
        set frameColor $bgColor__ 
        if { $target_platform == "A5" && $bgColor__ == "red" } {
          set frameColor #0000FF   
        }

        if { $bgColor__ != "!"} {
          .fmodal.msgbox configure -bg $frameColor
          .fmodal.msgbox.input configure -bg $frameColor
          .fmodal.msgbox.display configure -bg $frameColor
          .fmodal.msgbox.display.msg configure -bg $bgColor__
        }
    }

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    

}
