##############################################################################
#
# vc/src/tcl/responserate.tcl
#
# Response-Rate-Slider-Bar-Tuning screen
#
# 20030205 QC - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it in sExit
    frame .ftop

    # Title
    label .ftop.title -text "Response-Rate-Slider-Bar Tuning" -font $huge_font
    pack .ftop.title -side top

    label .ftop.subtitle -text "Sub Title" -font $medium_font
    pack .ftop.subtitle -side top

    # Adjustable Parameters Subframe
    frame .ftop.params 
    label .ftop.params.title -text "Adjustable Parameters:" -font $big_font
    pack .ftop.params.title -side top -anchor w

    frame .ftop.params.row1
    label .ftop.params.row1.l -text "% Change in Steering Gain:" \
            -font $medium_font
    pack .ftop.params.row1.l -side left
    label .ftop.params.row1.e -anchor w\
            -font $medium_font -width 8 -relief sunken -borderwidth 2
    pack .ftop.params.row1.e -side left
    button .ftop.params.row1.change -text "Change" \
            -command { beepOnce; post $DISP_BTN1_SIG 1 } \
            -width 5 -height 1 \
            -background $color3 -activebackground $color3
    pack .ftop.params.row1.change -side left -pady 10 -padx 10

    frame .ftop.params.row2
    label .ftop.params.row2.l -text "% Change in Heading Gain:" \
            -font $medium_font
    pack .ftop.params.row2.l -side left
    label .ftop.params.row2.e -anchor w\
            -font $medium_font -width 8 -relief sunken -borderwidth 2
    pack .ftop.params.row2.e -side left
    button .ftop.params.row2.change -text "Change" \
            -command { beepOnce; post $DISP_BTN2_SIG 2 } \
            -width 5 -height 1 \
            -background $color3 -activebackground $color3
    pack .ftop.params.row2.change -side left -pady 10 -padx 10

    frame .ftop.params.row3
    label .ftop.params.row3.l -text "% Change in Lateral-Error Gain:" \
            -font $medium_font
    pack .ftop.params.row3.l -side left
    label .ftop.params.row3.e -anchor w\
            -font $medium_font -width 8 -relief sunken -borderwidth 2
    pack .ftop.params.row3.e -side left
    button .ftop.params.row3.change -text "Change" \
            -command { beepOnce; post $DISP_BTN3_SIG 3 } \
            -width 5 -height 1 \
            -background $color3 -activebackground $color3
    pack .ftop.params.row3.change -side left -pady 10 -padx 10

    frame .ftop.params.row4
    label .ftop.params.row4.l -text "% Change in Lateral-Error Integral Rate:" \
            -font $medium_font
    pack .ftop.params.row4.l -side left
    label .ftop.params.row4.e -anchor w\
            -font $medium_font -width 8 -relief sunken -borderwidth 2
    pack .ftop.params.row4.e -side left
    button .ftop.params.row4.change -text "Change" \
            -command { beepOnce; post $DISP_BTN4_SIG 4 } \
            -width 5 -height 1 \
            -background $color3 -activebackground $color3
    pack .ftop.params.row4.change -side left -pady 10 -padx 10

    pack .ftop.params.row1 -side top
    pack .ftop.params.row2 -side top
    pack .ftop.params.row3 -side top
    pack .ftop.params.row4 -side top

    # Accept/Reject Subframe
    frame .ftop.acceptReject
    button .ftop.acceptReject.cancel -text "Cancel" \
            -command { beepOnce; post $DISP_BTN9_SIG 9 } -width 8 -height 3 \
            -background $color5 -activebackground $color5
    pack .ftop.acceptReject.cancel -side left -anchor s -padx 10 -pady 5

    button .ftop.acceptReject.accept -text "Accept\nParameters" \
            -command { beepOnce; post $DISP_BTN6_SIG 6 } -width 8 -height 3 \
            -background $color0 -activebackground $color0
    pack .ftop.acceptReject.accept -side right -anchor s -padx 10 -pady 3

    # Pack the Input Frame
    pack .ftop.params -side top -expand yes 
    pack .ftop.acceptReject -side top -expand yes

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry {responseRateLbl subTitleLbl adjustLbl steeringLbl \
                   headingLbl lateralLbl integralRateLbl changeBtn \
                   cancelBtn acceptBtn} {
        .ftop.title configure -text $responseRateLbl
        .ftop.subtitle configure -text $subTitleLbl       
        .ftop.params.title configure -text $adjustLbl
        .ftop.params.row1.l configure -text $steeringLbl
        .ftop.params.row1.change configure -text $changeBtn
        .ftop.params.row2.l configure -text $headingLbl
        .ftop.params.row2.change configure -text $changeBtn
        .ftop.params.row3.l configure -text $lateralLbl
        .ftop.params.row3.change configure -text $changeBtn
        .ftop.params.row4.l configure -text $integralRateLbl
        .ftop.params.row4.change configure -text $changeBtn

        .ftop.acceptReject.cancel configure -text $cancelBtn
        .ftop.acceptReject.accept configure -text $acceptBtn


        pack .ftop -fill both -expand yes
    }
    
    # Action called by GuiMgr on exit from this screen
    proc actExit {} {

        destroy .ftop

    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    # Update procedures of this template can be called by the GuiMgr
    proc actChangeResponseRateParameters {cpfSteering cpfHeading \
                                          cpfLateral cpfIntegral} {
        .ftop.params.row1.e configure -text $cpfSteering
        .ftop.params.row2.e configure -text $cpfHeading
        .ftop.params.row3.e configure -text $cpfLateral
        .ftop.params.row4.e configure -text $cpfIntegral
    }

    proc actChangeParameterSteering {cpfSteering} {
        .ftop.params.row1.e configure -text $cpfSteering
    }

    proc actChangeParameterHeading {cpfHeading} {
        .ftop.params.row2.e configure -text $cpfHeading
    }

    proc actChangeParameterLateral {cpfLateral} {
        .ftop.params.row3.e configure -text $cpfLateral
    }

    proc actChangeParameterIntegral {cpfIntegral} {
        .ftop.params.row4.e configure -text $cpfIntegral
    }

    proc actFieldBoundary { color text } {
        # {MLE-021112: Empty proc for now...}
    }
    
    #-------------------------------------------------------------------------
    # Local procedures
    #-------------------------------------------------------------------------
    proc ToBeCompleted {} {

    }

}
