##############################################################################
#
# vc/src/tcl/rmcodeupload.tcl
#
# Screen to show RM uploading procedure.
#
# 20041117 QC - created
#
# Copyright (c) 2004 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout

    # Title
    label .ftop.title -text "Uploading Roof Module Code\nIt may take several minutes!"
    label .ftop.subtitle -text "Roof Module Serial Number: Loading ..." -font $big_font

    # Upload progress frame
    frame .ftop.progress

    label .ftop.progress.l1 -text "Upload Progress:"
    pack .ftop.progress.l1 -side top

    set barWidth 400
    set barHeight 20
    canvas .ftop.progress.bar -background $color6 \
            -width $barWidth -height $barHeight -relief sunken -borderwidth 2
    incr barHeight        
    .ftop.progress.bar create rectangle 0 0 0 $barHeight \
            -outline black -fill $colorMedium -tags gauge
    .ftop.progress.bar create text [expr $barWidth/2] 4 \
            -font $small_font -anchor nw -text "0%" -tags value
    pack .ftop.progress.bar -side bottom
    
    # Cancel/Accept frame
    frame .ftop.input
    SnazzyButton::create .ftop.input.cont \
                 btnKhakiU btnKhakiD 0.5 \
                 "beepOnce; post $DISP_BTN9_SIG 9;" "Return To Previous Menu"
    # Exit button
    SnazzyButton::create .ftop.input.exit btnExitKhakiU btnExitKhakiD 0.2 \
                         "beepOnce; guiExit"

    # Pack .ftop
    pack .ftop.title -side top -pady 30
    pack .ftop.subtitle -side top -padx 150 -anchor sw 
    pack .ftop.progress -side top -pady 30
    pack .ftop.input -side bottom -anchor se -padx 5 -pady 5 \
                     -expand yes -fill x
                
    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { titleLbl contBtn } {
        .ftop.title configure -text $titleLbl
        SnazzyButton::setText .ftop.input.cont $contBtn 0.5
 
        pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actProgress { percentComplete color } {
        .ftop.progress.bar itemconfigure gauge -fill $color
        .ftop.progress.bar itemconfigure value -text "$percentComplete%"

        .ftop.progress.bar coords gauge 1 1 \
             [expr 0.01*$percentComplete*$Screen::barWidth] $Screen::barHeight
    }

    proc actConfigureTitle { title } {
        .ftop.title configure -text $title
    }

    proc actHideProgressBar { } {
        pack forget .ftop.progress
    }

    proc actDispContinueBtn { } {
        pack .ftop.input.cont -side left
    }

    proc actConfigureSubtitle { subtitle } {
        .ftop.subtitle configure -text $subtitle
    }

    proc actRmError { errLbl actionLbl exitBtn } {
        global huge_font
        global big_font

#        .ftop.display.l0 configure -text $errLbl -font $huge_font
#        .ftop.display.l1 configure -text $actionLbl -font $big_font
        .ftop.progress.l1 configure -text $errLbl -font $big_font
        .ftop.input.exit configure -state normal
        SnazzyButton::setText .ftop.input.exit $exitBtn 0.2
        pack .ftop.input.exit -side left -padx 5 -pady 5 -anchor sw
    }

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
}
