##############################################################################
#
# vc/src/tcl/samdiagmenu.tcl
#
# Screen to show SAM diagnostics Menu.
#
# 20040916 QC - created
#
# Copyright (c) 2004 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout

    # Title
    label .ftop.title -text "SAM Diagnostics Menu"
    label .ftop.subtitle -text "Version:\nNo Response From SA" -font $big_font
    
    # Cancel/Accept frame
    frame .ftop.input
    SnazzyButton::create .ftop.input.cancel \
                 btnKhakiU btnKhakiD 0.5 \
                 "beepOnce; post $DISP_BTN9_SIG 9;" "Return To Previous Menu"
    pack .ftop.input.cancel -side left
    SnazzyButton::create .ftop.input.nvRamPut \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN0_SIG 0;" "NVRAM Set Test"
    pack .ftop.input.nvRamPut -side right
    SnazzyButton::create .ftop.input.nvRamGet \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN1_SIG 1;" "NVRAM Get Test"
    pack .ftop.input.nvRamGet -side right
    SnazzyButton::create .ftop.input.samdiag \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN2_SIG 2;" "SAM Diagnostics"
    pack .ftop.input.samdiag -side right -padx 5
    SnazzyButton::create .ftop.input.codedownload \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN3_SIG 3;" "Code Download"
#    pack .ftop.input.codedownload -side right

    # Pack .ftop
    pack .ftop.title -side top -pady 40
    pack .ftop.subtitle -side top -pady 10
    pack .ftop.input -side bottom -anchor se -padx 5 -pady 5 \
                     -expand yes -fill x
                
    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { titleLbl subTitleLbl codeDownloadBtn \
                    samDiagBtn nvRamPutBtn nvRamGetBtn returnBtn } {
        .ftop.title configure -text $titleLbl
        .ftop.subtitle configure -text $subTitleLbl
        SnazzyButton::setText .ftop.input.cancel $returnBtn 0.5
#        SnazzyButton::setText .ftop.input.codedownload $codeDownloadBtn 0.5
        SnazzyButton::setText .ftop.input.samdiag $samDiagBtn 0.5
        SnazzyButton::setText .ftop.input.nvRamPut $nvRamPutBtn 0.5
        SnazzyButton::setText .ftop.input.nvRamGet $nvRamGetBtn 0.5
        pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actConfigVersionMsg {msg} {
        .ftop.subtitle configure -text $msg
    }

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
}
