#!/bin/sh
# postmortum script.  Works in conjunction with postmortum application.
#
# The postmortum *script* saves the /tmp/assert file to /data/currdir
# in the event of an assert.  It also activates the postmortum app
# if the user activates the secret button shutdown.
#
# The purpose of the postmortum *application* is to allow the user to
# examine and export files that were created by the mavericks application.
# In particular, the /tmp/assert file contains detailed assert information.
# 
# How it works: the postmortum script runs at the same time as mavericks.
# It is polling for one of two files: mavericks_secretButton or
# mavericks_assert.  If mavericks_assert is found, the script saves
# /tmp/assert in /data/currdir.  If mavericks_secretButton is found,
# then the script launches the postmortum app.
#
# Oct 2005, Randy Antosiak
#

MAVERICKS_RUNNING=true

# remove flag files - useful only for debugging
/bin/rm /tmp/mavericks_assert
/bin/rm /tmp/mavericks_secretButton

# polling loop - look for 'flag' file - mavericks_assert or
#   mavericks_secretButton
while $MAVERICKS_RUNNING; do

   sleep 3;
   if [ -e /tmp/mavericks_assert ]; then
      MAVERICKS_RUNNING=false;
   elif [ -e /tmp/mavericks_secretButton ]; then
      MAVERICKS_RUNNING=false;
   fi

done

# if mavericks asserted, then copy assert to /data/currdir

cd /data/currdir
if [ -e /tmp/mavericks_assert ]; then

#  mount /data/currdir
   /bin/mount -t jffs2 /dev/mtdblock6 /data

   # We store a max of 10 assert files.
   # See if there are already 10 assert files in /data/currdir.
   # If so, then delete assert1, and copy 10 to 9, etc.
   if [ -e assert10 ]; then
      /bin/rm assert1;
      /bin/mv assert2 assert1;
      /bin/mv assert3 assert2;
      /bin/mv assert4 assert3;
      /bin/mv assert5 assert4;
      /bin/mv assert6 assert5;
      /bin/mv assert7 assert6;
      /bin/mv assert8 assert7;
      /bin/mv assert9 assert8;
      /bin/mv assert10 assert9;

      # Save last 400 lines of /tmp/assert
      /usr/bin/tail -n 400 /tmp/output > assert10;

   # don't have 10 assert files, so just add this file as highest numbered
   else
      if [ ! -e assert1 ]; then
         ASSERTFILE=assert1;
      elif [ ! -e assert2 ]; then
         ASSERTFILE=assert2;
      elif [ ! -e assert3 ]; then
         ASSERTFILE=assert3;
      elif [ ! -e assert4 ]; then
         ASSERTFILE=assert4;
      elif [ ! -e assert5 ]; then
         ASSERTFILE=assert5;
      elif [ ! -e assert6 ]; then
         ASSERTFILE=assert6;
      elif [ ! -e assert7 ]; then
         ASSERTFILE=assert7;
      elif [ ! -e assert8 ]; then
         ASSERTFILE=assert8;
      elif [ ! -e assert9 ]; then
         ASSERTFILE=assert9;
      elif [ ! -e assert10 ]; then
         ASSERTFILE=assert10;
      fi
     
      /usr/bin/tail -n 400 /tmp/output > $ASSERTFILE;

# unmount /data
     /bin/umount /data

   fi
   exit;
fi

# if mavericks secret button, then launch postmortum app

if [ -e /tmp/mavericks_secretButton ]; then

    sleep 15
   # make sure data partition is mounted
   mount -t jffs2 /dev/mtdblock6 /data

   # Copy current output file
   /usr/bin/tail -n 400 /tmp/output > /data/currdir/output.current;

   export TCL_LIBRARY=/lib/tcl8.4
   export TK_LIBRARY=/lib/tk8.4
   export DISPLAY=:0.0

   cd /mnt/apps/vc/bin
   echo "starting postmortum"
   ./eldkdebug1_1t/postmortum > /dev/null &

   sleep 5  # have to delay start, otherwise bell change doesn't take
   xset b 100 1505 100

fi

