##############################################################################
#
# vc/src/tcl/wizDispatch.tcl
#
# Calibration Wizard Dispatch Screens
#
# 20031024 QC - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it in sExit
    frame .ftop

    # Define and pack the screen layout
    # (Screen has custom display and input frames
    # and uses .configDisplay, and .navDisplay components)
    frame .ftop.input
    frame .ftop.display
    pack .ftop.input -side bottom -fill both -expand yes -anchor s
    pack .ftop.display -side bottom -fill both -expand yes

    # Display frame - displays up to 5 lines of text
    label .ftop.display.l0 -text "line 0"
    label .ftop.display.l1 -text "line 1" -font $big_font
    label .ftop.display.l2 -text "line 2" -font $big_font
    label .ftop.display.l3 -text "line 3" -font $big_font
    label .ftop.display.l4 -text "line 4" -font $big_font

    # Input frame 
    #   - Cancel button 
    SnazzyButton::create .ftop.input.cancel btnKhakiU btnKhakiD 0.5 \
                         "beepOnce; post $DISP_BTN9_SIG 9" "Cancel"
    SnazzyButton::create .ftop.input.continue btnBlueU btnBlueD 0.5 \
                         "beepOnce; post $DISP_BTN0_SIG 0" "Continue"

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { packProc__ } {
        eval $packProc__
        pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    # Update procedures of this template can be called by the GuiMgr
    proc actConfigureTitle { title__ } {
        .ftop.display.l0 configure -text $title__
    }

    proc actConfigureL1 { line1__ } {
        global big_font
        .ftop.display.l1 configure -text $line1__ -font $big_font
    }

    proc actConfigureL2 { line2__ } {
        global big_font
        .ftop.display.l2 configure -text $line2__ -font $big_font
    }

    proc actConfigureCancelBtn { text__ } {
        SnazzyButton::setText .ftop.input.cancel $text__ 0.5
    }

    proc actConfigureContinueBtn { text__ } {
        SnazzyButton::setText .ftop.input.continue $text__ 0.5
    }

    proc actPackL2 {} {
        pack .ftop.display.l2 -side top
    }

    proc actPackCancelBtn {} {
        pack .ftop.input.cancel -side left -padx 5 -pady 5 -anchor sw
    }

    proc actPackContinueBtn {} {
        pack .ftop.input.continue -side right -padx 5 -pady 5 -anchor se
    }

    proc actForgetL1 {} {
        pack forget .ftop.display.l1
    }

    proc actForgetL2 {} {
        pack forget .ftop.display.l2
    }

    proc actForgetCancelBtn {} {
        pack forget .ftop.input.cancel
    }

    proc actForgetContinueBtn {} {
        pack forget .ftop.input.continue
    }

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    proc packCalWizDispatch {title__ line1__ continue__ cancel__} {

        global huge_font big_font

        SnazzyButton::setText .ftop.input.cancel $cancel__ 0.5
        pack .ftop.input.cancel -side left -padx 5 -pady 5 -anchor sw

        SnazzyButton::setText .ftop.input.continue $continue__ 0.5
        pack .ftop.input.continue -side right -padx 5 -pady 5 -anchor se
        
        .ftop.display.l0 configure -text $title__ -font $huge_font
        .ftop.display.l1 configure -text $line1__ -font $big_font
        pack .ftop.display.l0 -side top -pady 2
        pack .ftop.display.l1 -side top -pady 2
    }
}
