##############################################################################
#
# vc/src/tcl/commonobjects.tcl
#
# Defines various common display objects (i.e. Tk frame) objects that can be 
# packed into various screens, e.g. the Navigation frame, the Configuration 
# frame, the Tractor frame, the Calibration frame, etc.
#
# 20020327 MLE - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################


#=============================================================================
# .fwait
#
# A wait screen with message. Has the ability to unpack the current screen, 
# display a wait screen with a message, and repack the original screen
#=============================================================================
namespace eval Wait { 
    global gamma_value

    # Create the wait frame
    frame .fwait
    image create photo gifImage \
            -gamma $gamma_value -format gif \
            -file ${tclPath}gif/AFL_RGB4.7x3.4.gif
    label .fwait.picture -image gifImage
    pack .fwait.picture -side top -fill both -expand yes
    label .fwait.l1 -font $huge_font
    pack .fwait.l1 -side top -fill both -expand yes

    proc actStart {msg__} {
        pack forget .ftop
        .fwait.l1 configure -text $msg__
        pack .fwait -fill both -expand yes
    }

    proc actStop {} {
        pack forget .fwait
        pack .ftop -fill both -expand yes
    }

    proc actConfigLanguage {standby} {
        .fwait.l1 configure -text $standby
    }
}

#=============================================================================
# .images
# color button size(pixels):
# regular  110*110
# btnBlueM 95*50 designed for Change buttons in baseDisplay fieldDisplay
#                jobDisplay implementDisplay tractorDisplay
# btnBlueM2 90*70 for fieldBoundry buttons
# btnBlueM3 120*50 for job headingSelect buttons
# btnBlueM4 160*60 for keyBoard Spacebar
# btnBlueM5 110*60 for autofarm setup and alarm buttons
# btnBlueM6 95*35 for autofarm setup screen reset buttons
# btnKhakiM4 80*60 for keyBoard Cancel button
# btnGreenM4 80*60 for keyboard Enter button
# btnBlueS 50*50 for Slider + and - buttons
# btnBlueS2 60*60 for numpad buttons
# btnBlueS3 50*60 for keyBoard number buttons and letter buttons
# btnBlueS4 65*60 for keyBoard Bksp button
#=============================================================================
namespace eval Images { 
    global gamma_value
    puts stdout "TCL gamma_value $gamma_value"
    image create photo implAntennaLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/antennaleft.gif
    image create photo implAntennaRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/antennaright.gif
    image create photo implAntennaAft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/antennaaft.gif
    image create photo implAntennaFore -gamma $gamma_value -format gif \
        -file ${tclPath}gif/antennafore.gif
    image create photo implAntennaHeight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/antennaheight.gif
    image create photo outputLevel -gamma $gamma_value -format gif \
        -file ${tclPath}gif/outputlevel.gif
    image create photo angleDisplay -gamma $gamma_value -format gif \
        -file ${tclPath}gif/angledisplay.gif
    image create photo commandButtons -gamma $gamma_value -format gif \
        -file ${tclPath}gif/commandbuttons.gif
    image create photo gainDisplay -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gaindisplay.gif
    image create photo coulterGainLow -gamma $gamma_value -format gif \
        -file ${tclPath}gif/coultergainlow.gif
    image create photo coulterGainHigh -gamma $gamma_value -format gif \
        -file ${tclPath}gif/coultergainhigh.gif
    image create photo coulterGainGood -gamma $gamma_value -format gif \
        -file ${tclPath}gif/coultergaingood.gif

    image create photo implUpperLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implUpperLeft.gif
    image create photo implUpperRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implUpperRight.gif
    image create photo implUpperCenter -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implUpperCenter.gif
    image create photo implCenterLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implCenterLeft.gif
    image create photo implCenterRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implCenterRight.gif
    image create photo implCenterCenter -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implCenterCenter.gif
    image create photo implLowerLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implLowerLeft.gif
    image create photo implLowerRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implLowerRight.gif
    image create photo implLowerCenter -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implLowerCenter.gif
    image create photo implWidth -gamma $gamma_value -format gif \
        -file ${tclPath}gif/implementwidth.gif
    image create photo antenna -gamma $gamma_value -format gif \
        -file ${tclPath}gif/antenna.gif

    image create photo forwardPass -gamma $gamma_value -format gif \
        -file ${tclPath}gif/forwardpass.gif
    image create photo reversePass -gamma $gamma_value -format gif \
        -file ${tclPath}gif/reversepass.gif
    image create photo adjacentPasses -gamma $gamma_value -format gif \
        -file ${tclPath}gif/adjacentpasses.gif
    image create photo adjacentPasses2 -gamma $gamma_value -format gif \
        -file ${tclPath}gif/adjacentpasses2.gif
    image create photo btnNarrowRows -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnnarrowrows.gif
    image create photo btnWideRows -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnwiderows.gif
    image create photo btnEvenRows -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnevenrows.gif

    image create photo aftHydraulicLever -gamma $gamma_value -format gif \
        -file ${tclPath}gif/aftlever.gif
    image create photo aftHydraulicRemotes -gamma $gamma_value -format gif \
        -file ${tclPath}gif/aftremotes.gif

    image create photo guessRow -gamma $gamma_value -format gif \
        -file ${tclPath}gif/guessrow.gif


    image create photo smallNovariantLogo -gamma $gamma_value -format gif \
        -file ${tclPath}gif/smallNovariantLogo.gif
    image create photo smallAutoFarmLogo -gamma $gamma_value -format gif \
        -file ${tclPath}gif/smallAutoFarmLogo.gif
    image create photo smallVirtualRailLogo -gamma $gamma_value -format gif \
        -file ${tclPath}gif/smallVirtualRailLogo.gif
    image create photo smallSTOP -gamma $gamma_value -format gif -file ${tclPath}gif/smallSTOP.gif
    image create photo mediumSTOP \
                 -gamma $gamma_value -format gif -file ${tclPath}gif/mediumSTOP.gif
    image create photo bigSTOP -gamma $gamma_value -format gif -file ${tclPath}gif/bigSTOP.gif
    image create photo smallGO -gamma $gamma_value -format gif -file ${tclPath}gif/smallGO.gif
    image create photo mediumGO -gamma $gamma_value -format gif -file ${tclPath}gif/mediumGO.gif
    image create photo bigGO -gamma $gamma_value -format gif -file ${tclPath}gif/bigGO.gif
    image create photo btnRED -gamma $gamma_value -format gif -file ${tclPath}gif/btn_red.gif
    image create photo btnYELLOW -gamma $gamma_value -format gif \
                 -file ${tclPath}gif/btn_yellow.gif
    image create photo btnGREEN -gamma $gamma_value -format gif -file ${tclPath}gif/btn_green.gif
    image create photo btnGREY -gamma $gamma_value -format gif -file ${tclPath}gif/btn_grey.gif
    image create bitmap bigLNUDGE -foreground $color3 \
            -file ${tclPath}gif/largeleftarrow.xbm
    image create bitmap smallLNUDGE -foreground $color3 \
            -file ${tclPath}gif/leftarrow.xbm
    image create bitmap bigRNUDGE -foreground $color3 \
            -file ${tclPath}gif/largerightarrow.xbm
    image create bitmap smallRNUDGE -foreground $color3 \
            -file ${tclPath}gif/rightarrow.xbm
    image create bitmap circle -foreground $color6 \
            -file ${tclPath}gif/circle.xbm
    image create photo zoomin -gamma $gamma_value -format gif -file ${tclPath}gif/btnZoomIn.gif
    image create photo zoomout -gamma $gamma_value -format gif -file ${tclPath}gif/btnZoomOut.gif
    image create photo panandzoom -gamma $gamma_value -format gif -file ${tclPath}gif/btnPanZoom.gif	
    image create photo normalmode -gamma $gamma_value -format gif -file ${tclPath}gif/btnNormalMode.gif
    image create photo zoominGrn -gamma $gamma_value -format gif -file ${tclPath}gif/btnZoomInGreen.gif
    image create photo zoominRed -gamma $gamma_value -format gif -file ${tclPath}gif/btnZoomInRed.gif
    image create photo zoomoutGrn -gamma $gamma_value -format gif -file ${tclPath}gif/btnZoomOutGreen.gif
    image create photo zoomoutRed -gamma $gamma_value -format gif -file ${tclPath}gif/btnZoomOutRed.gif
    image create photo panandzoomGrn -gamma $gamma_value -format gif -file ${tclPath}gif/btnPanZoomGreen.gif	
    image create photo normalmodeGrn -gamma $gamma_value -format gif -file ${tclPath}gif/btnNormalModeGreen.gif
    image create photo AutoSteer5U -gamma $gamma_value -format gif -file ${tclPath}gif/AutoSteer5U.gif
    image create photo AutoSteer5D -gamma $gamma_value -format gif -file ${tclPath}gif/AutoSteer5D.gif
    image create photo grayup -gamma $gamma_value -format gif -file ${tclPath}gif/grayup.gif
    image create photo graydown -gamma $gamma_value -format gif -file ${tclPath}gif/graydown.gif
    image create photo ExitU -gamma $gamma_value -format gif -file ${tclPath}gif/ExitU.gif
    image create photo ExitD -gamma $gamma_value -format gif -file ${tclPath}gif/ExitD.gif
    image create photo AutoLevelU -gamma $gamma_value -format gif -file ${tclPath}gif/AutoLevel.gif
    image create photo AutoLevelD -gamma $gamma_value -format gif -file ${tclPath}gif/AutoLevelD.gif

    #implement offset
    image create photo implOffsetLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/impl_left_offset.gif
    image create photo implOffsetNone -gamma $gamma_value -format gif \
        -file ${tclPath}gif/impl_no_offset.gif
    image create photo implOffsetRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/impl_right_offset.gif
    image create photo implOffsetLeftDim -gamma $gamma_value -format gif \
        -file ${tclPath}gif/impl_left_offset_dim.gif
    image create photo implOffsetRightDim -gamma $gamma_value -format gif \
        -file ${tclPath}gif/impl_right_offset_dim.gif

    # images for "snazzy buttons"
    image create photo btnAcceptU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnAcceptU.gif
    image create photo btnAcceptD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnAcceptD.gif
    image create photo btnRejectU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRejectU.gif
    image create photo btnRejectD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRejectD.gif
    image create photo btnExitKhakiU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiU.gif
    image create photo btnExitKhakiD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiD.gif
    image create photo btnExitRedU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitRedU.gif
    image create photo btnExitRedD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitRedD.gif
        
    image create photo btnGreenU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenU.gif
    image create photo btnGreenD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenD.gif
    image create photo btnGreenM4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM4U.gif
    image create photo btnGreenM4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM4D.gif
    image create photo btnGreenM5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM5U.gif
    image create photo btnGreenM5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenM5D.gif
    image create photo btnGrayU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGrayU.gif
    image create photo btnGrayD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGrayD.gif
    image create photo btnYellowU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnYellowU.gif
    image create photo btnYellowD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnYellowD.gif
    image create photo btnRedU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedU.gif
    image create photo btnRedD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedD.gif
    image create photo btnRedM2U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedM2U.gif
    image create photo btnRedM2D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedM2D.gif
    image create photo btnRedForSatVUU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedForSatVUU.gif
    image create photo btnRedForSatVUD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedForSatVUD.gif
    image create photo btnBlueForSatVUU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueForSatVUU.gif
    image create photo btnBlueForSatVUD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueForSatVUD.gif
    image create photo btnBlueU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueU.gif
    image create photo btnGreenSliderDChecked -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenSliderDChecked.gif
    image create photo btnGreenSliderUChecked -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenSliderUChecked.gif
    image create photo btnBlueD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueD.gif
    image create photo btnKhakiU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiU.gif
    image create photo btnKhakiD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiD.gif
    image create photo btnKhakiM4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM4U.gif
    image create photo btnKhakiM4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM4D.gif
    image create photo btnKhakiM5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM5U.gif
    image create photo btnKhakiM5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM5D.gif
    image create photo btnKhakiM6U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM6U.gif
    image create photo btnKhakiM6D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiM6D.gif    
    image create photo btnKhakiForSatVUU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiForSatVUU.gif
    image create photo btnKhakiForSatVUD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiForSatVUD.gif
    image create photo btnKhakiIMPLSU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiIMPLSU.gif
    image create photo btnKhakiIMPLSD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnKhakiIMPLSD.gif
    image create photo btnChangeConfigD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnChangeConfigD.gif
    image create photo btnChangeConfigU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnChangeConfigU.gif
    image create photo btnDriverD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverD.gif
    image create photo btnDriverU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverU.gif
    image create photo btnSupervisorD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorD.gif
    image create photo btnSupervisorU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorU.gif
    image create photo btnMeasureD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnMeasureD.gif
    image create photo btnMeasureU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnMeasureU.gif

    image create photo btnStartSurveyD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnStartSurveyD.gif
    image create photo btnStartSurveyU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnStartSurveyU.gif
    image create photo btnSkipSurveyD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSkipSurveyD.gif       
    image create photo btnSkipSurveyU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSkipSurveyU.gif
        
    image create photo btnBlueMU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMU.gif
    image create photo btnBlueMD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMD.gif
    image create photo btnBlueM2U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM2U.gif
    image create photo btnBlueM2D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM2D.gif
    image create photo btnBlueM3U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM3U.gif
    image create photo btnBlueM3D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM3D.gif
    image create photo btnBlueM4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM4U.gif
    image create photo btnBlueM4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM4D.gif
    image create photo btnBlueMS4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS4U.gif
    image create photo btnBlueMS4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS4D.gif
        
    image create photo btnBlueM5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM5U.gif
    image create photo btnBlueM5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM5D.gif
    image create photo btnRedM5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedM5U.gif
    image create photo btnRedM5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRedM5D.gif
    image create photo btnYellowM5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnYellowM5U.gif
    image create photo btnYellowM5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnYellowM5D.gif

    image create photo btnBlueM6U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM6U.gif
    image create photo btnBlueM6D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM6D.gif
        
    image create photo btnBlueMS6U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS6U.gif
    image create photo btnBlueMS6D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS6D.gif
        
    image create photo btnBlueM7U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM7U.gif
    image create photo btnBlueM7D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM7D.gif   
        
    image create photo btnBlueM8U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM8U.gif
    image create photo btnBlueM8D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM8D.gif    
        
    image create photo btnBlueM9U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM9U.gif
    image create photo btnBlueM9D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueM9D.gif    

    image create photo btnBlueSU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSU.gif
    image create photo btnBlueSD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSD.gif
	image create photo btnBlueSlider -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnBlueSlider.gif	
    image create photo btnBlueS2U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS2U.gif
    image create photo btnBlueS2D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS2D.gif
    image create photo btnBlueS3U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS3U.gif
    image create photo btnBlueS3D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS3D.gif
    image create photo btnBlueS4U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS4U.gif
    image create photo btnBlueS4D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueS4D.gif
    image create photo btnBlueMS5U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS5U.gif
    image create photo btnBlueMS5D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS5D.gif
    image create photo btnBlueMS6U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS6U.gif
    image create photo btnBlueMS6D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS6D.gif
    image create photo btnBlueMS7U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS7U.gif  
    image create photo btnBlueMS7D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS7D.gif
    image create photo btnGreenMS7U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenMS7U.gif    
    image create photo btnBlueMS8U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS8U.gif  
    image create photo btnBlueMS8D -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMS8D.gif
    image create photo btnGreenMS8U -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGreenMS8U.gif
    image create photo btnBlueBLRU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueBLRU.gif
    image create photo btnBlueBLRD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueBLRD.gif 
    image create photo btnBlueMaxLRU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMaxLRU.gif
    image create photo btnBlueMaxLRD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueMaxLRD.gif 
         
    image create photo btnDriverU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverU.gif
    image create photo btnDriverD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnDriverD.gif
    image create photo btnSupervisorU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorU.gif
    image create photo btnSupervisorD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSupervisorD.gif

    image create photo btnBigStopU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigStopU.gif
    image create photo btnBigStopD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigStopD.gif
    image create photo btnSmallStopU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallStopU.gif
    image create photo btnSmallStopD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallStopD.gif
    image create photo btnBigGoU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigGoU.gif
    image create photo btnBigGoD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigGoD.gif
    image create photo btnSmallGoU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallGoU.gif
    image create photo btnSmallGoD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallGoD.gif

    image create photo btnBigLeftNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigLeftNudgeU.gif
    image create photo btnBigLeftNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigLeftNudgeD.gif

    image create photo btnBigRightNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigRightNudgeU.gif
    image create photo btnBigRightNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBigRightNudgeD.gif

	image create photo btnMediumLeftNudgeU -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnMediumLeftNudgeU.gif
	image create photo btnMediumRightNudgeU -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnMediumRightNudgeU.gif
	
    image create photo btnSmallLeftNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallLeftNudgeU.gif
    image create photo btnSmallLeftNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallLeftNudgeD.gif

    image create photo btnSmallRightNudgeU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallRightNudgeU.gif
    image create photo btnSmallRightNudgeD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnSmallRightNudgeD.gif

    image create photo btnParallelRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnParallelRowsU.gif
    image create photo btnParallelRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnParallelRowsD.gif

    image create photo btnFanRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnFanRowsU.gif
    image create photo btnFanRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnFanRowsD.gif
    image create photo btnBoxRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBoxRowsU.gif
    image create photo btnBoxRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBoxRowsD.gif

    image create photo btnCircleRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCircleRowsU.gif
    image create photo btnCircleRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCircleRowsD.gif
    image create photo btnCurveRowsU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCurveRowsU.gif
    image create photo btnCurveRowsD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnCurveRowsD.gif
    image create photo btnABPathU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnABPathU.gif
    image create photo btnABPathD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnABPathD.gif
    image create photo btnRepeatJobU -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRepeatJobU.gif
    image create photo btnRepeatJobD -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnRepeatJobU.gif
    image create photo imgCoverageDispPoly -gamma $gamma_value -format gif \
        -file ${tclPath}gif/imgCoverageDispPoly.gif
    image create photo imgCoverageDispBoom -gamma $gamma_value -format gif \
        -file ${tclPath}gif/imgCoverageDispBoom.gif
    image create photo imgCoverageDispWire -gamma $gamma_value -format gif \
        -file ${tclPath}gif/imgCoverageDispWire.gif
    image create photo imgDivider -gamma $gamma_value -format gif \
        -file ${tclPath}gif/divider.gif     

    image create photo gantryLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryLeft.gif
    image create photo gantryRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryRight.gif
    image create photo gantryJoystickLeft -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryJoystickLeft.gif
    image create photo gantryJoystickRight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryJoystickRight.gif
    image create photo gantryStraight -gamma $gamma_value -format gif \
        -file ${tclPath}gif/gantryStraight.gif
        
    # New Square, Stop and Go Buttons
    image create photo btnBlueSquareUp -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquareUp.gif
    image create photo btnBlueSquareDown -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquareUp.gif
    image create photo btnBlueSquare2Up -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquare2Up.gif
    image create photo btnBlueSquare2Down -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueSquare2Up.gif
    image create photo btnBlueRectUp -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueRectUp.gif
    image create photo btnBlueRectDown -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueRectUp.gif
    image create photo btnExitKhakiSquareUp -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiSquareUp.gif
    image create photo btnExitKhakiSquareDown -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnExitKhakiSquareUp.gif
    image create photo checkboxD1 \
            -format gif \
            -file ${tclPath}gif/btnGreenSliderUChecked.gif
    image create photo checkboxU1 \
            -format gif \
            -file ${tclPath}gif/btnGreenSliderDChecked.gif


    #for new implement wizard
    image create photo btnBeginPassSquareUp -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnBeginPassSquareUp.gif	

    image create photo btnBeginPassUp -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnBeginPassUp.gif	
    image create photo btnBeginPassDown -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBeginPassDown.gif
    image create photo btnEndPassUp -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnEndPassUp.gif
    image create photo btnEndPassDown -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnEndPassDown.gif
    image create photo imgDividerDash -gamma $gamma_value -format gif \
        -file ${tclPath}gif/dividerDash.gif

    #wizard progress buttons
    image create photo btnWizardProgressBlue -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueWizard.gif
    image create photo btnWizardProgressBlueChecked -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueWizardChecked.gif
    image create photo btnWizardProgressGray -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnGrayWizard.gif
    image create photo btnBlueUChecked -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueUChecked.gif
    image create photo btnBlueDChecked -gamma $gamma_value -format gif \
        -file ${tclPath}gif/btnBlueDChecked.gif
        
	# Toggle Coverage ON/OFF images
	image create photo btnToggleOn -gamma $gamma_value -format gif \
	   -file ${tclPath}gif/btnRedToggleOn.gif
	image create photo btnToggleOff -gamma $gamma_value -format gif \
	   -file ${tclPath}gif/btnGreenToggleOff.gif	

    # stop buttons for Steered Implement Response Wizard

	image create photo btnMediumStopU -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnMediumStopU.gif
	image create photo btnMediumStopD -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnMediumStopD.gif

	image create photo btnBigStopU -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnBigStopU.gif
	image create photo btnBigStopD -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnBigStopD.gif
	
	image create photo btnBlue8TU -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnBlue8TU.gif
	image create photo btnBlue8TD -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnBlue8TD.gif
	
	image create photo btnGreen8TU -gamma $gamma_value -format gif \
	-file ${tclPath}gif/btnGreen8TU.gif
	
	# Timeout Status images
	image create photo timeout0 -gamma $gamma_value -format gif \
        -file ${tclPath}gif/timeout0.gif
    image create photo timeout25 -gamma $gamma_value -format gif \
        -file ${tclPath}gif/timeout25.gif
    image create photo timeout50 -gamma $gamma_value -format gif \
        -file ${tclPath}gif/timeout50.gif
    image create photo timeout75 -gamma $gamma_value -format gif \
        -file ${tclPath}gif/timeout75.gif
    image create photo timeout100 -gamma $gamma_value -format gif \
        -file ${tclPath}gif/timeout100.gif

    image create photo warningImg -gamma $gamma_value -format gif \
        -file ${tclPath}gif/warningImg.gif
}

#=============================================================================
#
# The AutoFarm snazzy buttons. 
# A bunch of snazzyy buttons that can be placed into any screen.
#=============================================================================
namespace eval SnazzyBlendedButton {

    proc create { me imgU imgD yFrac command {text " "} colorBg} {
        global btn_font color6

        set xMax [expr [image width  $imgU] - 20]
        set yMax [expr [image height $imgU] - 9]

        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }

        button $me -text "$text" -font $btn_font \
            -compound center \
            -command $command \
            -width $xMax -height $yMax \
            -image $imgU \
            -relief flat -borderwidth 0 \
            -background $colorBg -activebackground $colorBg \
			-highlightthickness 0
            ;#-background pink -activebackground pink
    }

    proc setText { me text yFrac } {
        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }
        $me configure -text $text
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [expr [image width  $imgU] - 19]
        set yMax [expr [image height $imgU] - 9]
        $me configure -image $imgU -width $xMax -height $yMax
    }
}

namespace eval SnazzyUnclippedButton {

    proc create { me imgU imgD yFrac command {text " "} } {
        global btn_font color6

        set xMax [expr [image width  $imgU]]
        set yMax [expr [image height $imgU]]

        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }

        button $me -text "$text" -font $btn_font \
            -compound center \
            -command $command \
            -width $xMax -height $yMax \
            -image $imgU \
            -relief flat -borderwidth 0 \
            -background $color6 -activebackground $color6 \
			-highlightthickness 0
            ;#-background pink -activebackground pink
    }

    proc setText { me text yFrac } {
        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }
        $me configure -text $text
    }

    proc setBackground { me backgroundColor } {
        $me configure -background $backgroundColor
        $me configure -activebackground $backgroundColor
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [expr [image width  $imgU]]
        set yMax [expr [image height $imgU]]
        $me configure -image $imgU -width $xMax -height $yMax
    }

    proc setCmd { me command } {

        $me configure -command $command
    }
}


namespace eval SnazzyXclippedButton {

    proc create { me imgU imgD yFrac command {text " "} } {
        global btn_font color6

        set xMax [expr [image width  $imgU] - 18]
        set yMax [expr [image height $imgU]]

        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }

        button $me -text "$text" -font $btn_font \
            -compound center \
            -command $command \
            -width $xMax -height $yMax \
            -image $imgU \
            -relief flat -borderwidth 0 \
            -background $color6 -activebackground $color6 \
			-highlightthickness 0
            ;#-background pink -activebackground pink
    }

    proc setText { me text yFrac } {
        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }
        $me configure -text $text
    }

    proc setBackground { me backgroundColor } {
        $me configure -background $backgroundColor
        $me configure -activebackground $backgroundColor
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [expr [image width  $imgU]]
        set yMax [expr [image height $imgU]]
        $me configure -image $imgU -width $xMax -height $yMax
    }

    proc setCmd { me command } {

        $me configure -command $command
    }
}


if {1} {          ;# {MLE-031107: Testing a ?faster? version of SnazzyButtons}
namespace eval SnazzyButton {

    proc create { me imgU imgD yFrac command {text " "} } {
        global btn_font color6

        set xMax [expr [image width  $imgU] - 20]
        set yMax [expr [image height $imgU] - 9]

        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }

        button $me -text "$text" -font $btn_font \
            -compound center \
            -command $command \
            -width $xMax -height $yMax \
            -image $imgU \
            -relief flat -borderwidth 0 \
            -background $color6 -activebackground $color6 \
			-highlightthickness 0
            ;#-background pink -activebackground pink
    }

    proc setText { me text yFrac } {
        if { $yFrac < 0.35 } {
            set text "$text\n\n\n"
        } elseif { $yFrac > 0.75 } {
            set text "\n\n\n$text"
        }
        $me configure -text $text
    }

    proc setBackground { me bgColor } {
        $me configure -background $bgColor
        $me configure -activebackground $bgColor
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [expr [image width  $imgU] - 20]
        set yMax [expr [image height $imgU] - 9]
        $me configure -image $imgU -width $xMax -height $yMax
    }

    proc setCmd { me command } {

        $me configure -command $command
    }
}

} else {

namespace eval SnazzyButton {

    proc create { me imgU imgD yFrac command {text ""} } {
        global btn_font

        set xMax [image width  $imgU]
        set yMax [image height $imgU]
        set yTxt [expr $yMax*$yFrac]

        canvas $me -width $xMax -height $yMax
        $me create image    0    0 -anchor nw -image $imgU -tags imageU
        $me create image -200 -200 -anchor nw -image $imgD -tags imageD
        $me create text [expr $xMax/2] $yTxt \
                   -text $text -font $btn_font -justify center -tags text
        # create an off-screen text item to store the command
        $me create text 0 -50 -text $command -tags command            
        
        bind $me <ButtonPress-1> {
           %W coords imageD    0    0
           %W coords imageU -200 -200
           set txtCoords [%W coords text]
           %W coords text [expr [lindex $txtCoords 0] + 2] \
                          [expr [lindex $txtCoords 1] + 2]
        }
        
        bind $me <ButtonRelease-1> {
           set xMax [%W cget -width]
           set yMax [%W cget -height]
           %W coords imageD -200 -200
           %W coords imageU    0    0
           set txtCoords [%W coords text]
           %W coords text [expr [lindex $txtCoords 0] - 2] \
                          [expr [lindex $txtCoords 1] - 2]
           if {0 < %x && %x < $xMax && 
               0 < %y && %y < $yMax} {
              uplevel #0 [%W itemcget command -text]
           }
        }
    }

    proc setText { me text yFrac } {
        $me itemconfigure text -text $text

        set xMax [$me cget -width]
        set yMax [$me cget -height]
        $me coords text [expr $xMax/2] [expr $yMax*$yFrac]
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [image width  $imgU]
        set yMax [image height $imgU]
        $me configure -width  $xMax
        $me configure -height $yMax
        $me itemconfigure imageU -image $imgU
        $me itemconfigure imageD -image $imgD
        $me coords text [expr $xMax/2] [expr $yMax*$yFrac]
    }
}
}

#=============================================================================
# .logo
#
# AutoFarm logo frame with separator and secret button. A frame which can be 
# packed into any screen.
#=============================================================================
namespace eval Logo {

    frame .logo

    # INX logo, version, and system button
    frame .logo.inx
    label .logo.inx.1 -image smallNovariantLogo
    pack .logo.inx.1 -side right -padx 0
    label .logo.inx.3 \
            -width 8 -borderwidth 0 -font $small_font -anchor e \
            -text "00C"
    label .logo.inx.4 \
            -width 7 -borderwidth 0 -font $small_font -justify right
    pack .logo.inx.4 -side right

#   Remove the leftmost text "Screen"
#   label .logo.inx.5 \
#            -width 8 -borderwidth 0 -font $small_font -anchor e
#   pack .logo.inx.5 -side left -padx 5
#   label .logo.inx.6 
#           -width 5 -borderwidth 0 -font $small_font -anchor w

    button .logo.inx.6 -width 5 -command {beepOnce; post $DISP_LOGO1_SIG 127 } \
            -borderwidth 0  \
            -background $color6 -highlightbackground $color6 \
            -activebackground $color6 -font $small_font -width 3
            
    label .logo.inx.8 \
           -width 5 -borderwidth 0 -font $small_font -anchor w
           
    pack .logo.inx.8 -side left -padx 2 -anchor w
            
    pack .logo.inx.6 -side left -padx 2 -anchor w

    label .logo.inx.7 \
           -width 12 -borderwidth 0 -font $small_font -text "  "
    #pack .logo.inx.7 -side left -padx 1
    
    # Frame for Timeout Status indicator
    frame .logo.inx.timeout
    label .logo.inx.timeout.status -text "0" -foreground "#ffffff" \
                                   -font $small_font -image timeout25 \
                                   -compound center
    pack  .logo.inx.timeout -side left

    button .logo.inx.2 -command {beepOnce; post $DISP_BTN7_SIG 7 } \
            -borderwidth 0 -state disabled \
            -background $color6 -highlightbackground $color6 \
            -activebackground $color6
    pack .logo.inx.2 -side right
    pack .logo -side bottom

    # External disk frame
    frame .logo.inx.extDisk
    # Disk usage bar
    canvas .logo.inx.extDisk.c1 -width 180 -height 15 -relief sunken \
            -borderwidth 1 -background $color6 -state disabled;
    .logo.inx.extDisk.c1 create rectangle 0 0 0 20 \
               -fill green -width 0 -tags flashDiskBar
    .logo.inx.extDisk.c1 create text 100 16 \
        -text "Disk Space: " -anchor s -font $small_font -tags flashDiskText

    # Label to indicate that the system is in single frequency mode
    label .logo.inx.sf -text "" \
            -width 6 -borderwidth 0 -font $small_font -anchor w
    pack .logo.inx.sf -side right
    
     
    # Circular button for datalogger - now letter instead
    label .logo.inx.extDisk.t1 -text "D" -font $small_font \
            -foreground gold3

    # Circular button for curve recording - now letter instead
    label .logo.inx.extDisk.t2 -text "R" -font $small_font \
            -foreground gold3

    # Letter for other external disk user
    label .logo.inx.extDisk.t3 -text "O" -font $small_font \
            -foreground gold3

    # Letter for recording Control Data to external disk
    label .logo.inx.extDisk.t4 -text "C" -font $small_font \
            -foreground gold3
            
    # Letter for recording Control Data to external disk
    label .logo.inx.extDisk.t5 -text "00C" -font $small_font

    pack .logo.inx.extDisk -side right -padx 10;


    pack .logo.inx -side bottom -fill x -expand yes

    frame .logo.sep -height 2 -width $winWidth -borderwidth 0 \
            -background $colorInx
    pack .logo.sep -side bottom

    proc actReconfigLogo {logoImage} {
        .logo.inx.1 configure -image $logoImage
    }

    proc actConfigSwVersion {swVersion } {
        .logo.inx.4 configure -text $swVersion
    }
    
    proc actDisplyModeIndicator {mode} {
      .logo.inx.sf configure -text $mode 
    }
    
    proc actDisplayUtTemp { utTemp color } {
         regsub -all {[ \r\t\n]+} $utTemp "" newUtTemp
         .logo.inx.7 configure -text $newUtTemp
         .logo.inx.7 configure -fg $color
    }
    
    proc actRemoveUtTempDisplay {} {
         .logo.inx.7 configure -text ""
    }    
    
    proc actConfigLanguage {screen diskSpace} {
 #      .logo.inx.5 configure -text $screen
        .logo.inx.extDisk.c1 itemconfigure flashDiskText -text $diskSpace
    }

    proc actConfigScreenId {screenId} {
        .logo.inx.6 configure -text $screenId
        .logo.inx.8 configure -text $screenId
    }

    proc actConfigWhichStep {part} {
        .logo.inx.7 configure -text $part
         .logo.inx.7 configure -fg "black"
    }

    proc actSecretSpotOn {} {
        .logo.inx.2 configure -state "normal"
    }

    proc actSecretSpotOff {} {
        .logo.inx.2 configure -state "disabled"
    }
    
    proc actEnableLangButton {} {
      pack forget .logo.inx.8  
      pack forget .logo.inx.7
      pack .logo.inx.6 -side left -padx 2 -anchor w
      pack .logo.inx.7 -side left -padx 1 -pady 5 -anchor sw
    }
    
    proc actDisableLangButton {} {
      pack forget .logo.inx.6
      pack forget .logo.inx.7
      pack .logo.inx.8 -side left -padx 2 -anchor w
      pack .logo.inx.7 -side left -padx 1 -pady 5 -anchor sw
    }

    proc actEnableExtDisk {} {
        pack .logo.inx.extDisk.c1 -side right -padx 2;
        #pack .logo.inx.extDisk.b1 -side right;
        pack .logo.inx.extDisk.t4 -side right;
        pack .logo.inx.extDisk.t1 -side right;
        #pack .logo.inx.extDisk.b2 -side right;
        pack .logo.inx.extDisk.t2 -side right;
        pack .logo.inx.extDisk.t3 -side right;
    }

    proc actConfigExtDisk {barText showBar percentUsed isLoggingData \
            isRecordingData otherUsingExtDisk isRecControlData} {
        if { $percentUsed >= 100.0 } {
            .logo.inx.extDisk.c1 itemconfigure flashDiskBar -fill red;
            #.logo.inx.extDisk.b1 configure -disabledforeground red;
            #.logo.inx.extDisk.b2 configure -disabledforeground red;
            .logo.inx.extDisk.t1 configure -foreground red;
            .logo.inx.extDisk.t2 configure -foreground red;
            .logo.inx.extDisk.t3 configure -foreground red;
            .logo.inx.extDisk.t4 configure -foreground red;
        } else {
            if { $isRecordingData == 1 || $isLoggingData == 1 \
                 || $isRecControlData == 1} {
                .logo.inx.extDisk.c1 itemconfigure flashDiskBar -fill green4;
            } else {
                .logo.inx.extDisk.c1 itemconfigure flashDiskBar -fill gold3;
            }
            if { $isLoggingData == 1 } {
                #.logo.inx.extDisk.b1 configure -disabledforeground green4;
                .logo.inx.extDisk.t1 configure -foreground green4;
            } else {
                #.logo.inx.extDisk.b1 configure -disabledforeground gold3;
                .logo.inx.extDisk.t1 configure -foreground gold3;
            }
            if { $isRecordingData == 1 } {
                #.logo.inx.extDisk.b2 configure -disabledforeground green4;
                .logo.inx.extDisk.t2 configure -foreground green4;
            } else {
                #.logo.inx.extDisk.b2 configure -disabledforeground gold3;
                .logo.inx.extDisk.t2 configure -foreground gold3;
            }
            if { $otherUsingExtDisk == 1 } {
                .logo.inx.extDisk.t3 configure -foreground green4;
            } else {
                .logo.inx.extDisk.t3 configure -foreground gold3;
            }
            if { $isRecControlData == 1 } {
                .logo.inx.extDisk.t4 configure -foreground green4;
            } else {
                .logo.inx.extDisk.t4 configure -foreground gold3;
            }
        }
        
	if {$showBar == 1} {
	    # Set the rectangle indicating the fullness of the disk.
	    set rectLen 200.0;
	    # The 100.0 in the denominator represents 100.0%.
	    set rectPixelPos [expr $percentUsed*$rectLen/100.0];
	    if { $rectPixelPos >= $rectLen } {
		set rectPixelPos [expr $rectLen+5];
	    } elseif { $rectPixelPos < 0.0 } {
		set rectPixelPos 0.0;
	    }
	} else {
	    set rectPixelPos 0
	}
	.logo.inx.extDisk.c1 coords flashDiskBar $rectPixelPos 0 0 20;
	.logo.inx.extDisk.c1 itemconfigure flashDiskText \
	    -text $barText;
                
    }

    proc buzzer {} {
        beepOnce;
    }
    
    # Software Timeout Status Indicator
    proc actSetTimeoutStatus { percent days } {
      .logo.inx.timeout.status configure -image [concat timeout$percent]
      .logo.inx.timeout.status configure -text $days
      pack .logo.inx.timeout.status -side left
    }    

}


#=============================================================================
# .configDisplay
#
# The AutoFarm configuration display. A frame which can be packed into any 
# screen.
#=============================================================================
namespace eval ConfigDisplay { 

    frame .configDisplay -height $configHeight -width $configWidth
    frame .configDisplay.r1
    frame .configDisplay.r2
    frame .configDisplay.r3
    frame .configDisplay.r4
    frame .configDisplay.r5

    label .configDisplay.r1.l -font $medium_font
    label .configDisplay.r2.l -font $medium_font
    label .configDisplay.r3.l -font $medium_font
    label .configDisplay.r4.l -font $medium_font
    pack .configDisplay.r1.l -side left -pady 1
    pack .configDisplay.r2.l -side left -pady 1
    pack .configDisplay.r3.l -side left -pady 1
    pack .configDisplay.r4.l -side left -pady 1

    label .configDisplay.r4.l2 \
            -font $medium_font -justify right
    label .configDisplay.r3.l2 \
            -font $medium_font -justify right
    label .configDisplay.r2.l2 \
            -font $medium_font -justify right
    label .configDisplay.r1.l2 -text "" \
            -font $medium_font -justify right
    label .configDisplay.r1.l3 -text "" \
            -font $medium_font -justify right
            
    pack .configDisplay.r1.l3 -side right
    pack .configDisplay.r1.l2 -side right
    pack .configDisplay.r1.l2 -side right
    pack .configDisplay.r2.l2 -side right
    pack .configDisplay.r3.l2 -side right
    pack .configDisplay.r4.l2 -side right

    label .configDisplay.r5.e1 -text "0" -font $medium_font \
            -bg red -width 2 -justify left
    label .configDisplay.r5.e2 -font $medium_font
    label .configDisplay.r5.e3 -text "0" -font $medium_font \
            -bg red -width 2 -justify left
    label .configDisplay.r5.e4 -font $medium_font
    label .configDisplay.r5.e5 -text "0" -font $medium_font \
            -bg red -width 4
    label .configDisplay.r5.e6 -font $medium_font
    pack .configDisplay.r5.e2 -side left
    pack .configDisplay.r5.e1 -side left
    pack .configDisplay.r5.e4 -side left
    pack .configDisplay.r5.e3 -side left
    pack .configDisplay.r5.e5 -side right
    pack .configDisplay.r5.e6 -side right

    pack .configDisplay.r1 -side bottom -anchor w -fill x
    pack .configDisplay.r2 -side bottom -anchor w -fill x
    pack .configDisplay.r3 -side bottom -anchor w -fill x
    pack .configDisplay.r4 -side bottom -anchor w -fill x
    pack .configDisplay.r5 -side bottom -anchor w -fill x

    proc actChangeTractor { tractorName__ } {
        .configDisplay.r4.l2 configure -text $tractorName__
    }

    proc actChangeImplement { implementName__ } {
        .configDisplay.r3.l2 configure -text $implementName__
    }

    proc actChangeUserLevel { userLevel__ } {
        .configDisplay.r2.l2 configure -text $userLevel__
    }

    proc actGPSHealth { numSvL1Base__ numSvL1BaseColor__ \
            numSvL1Rover0__ numSvL1Rover0Color__ \
            gpsQuality__ gpsQualityColor__ } {

        .configDisplay.r5.e3 configure -text $numSvL1Base__ \
                -bg $numSvL1BaseColor__
        .configDisplay.r5.e1 configure -text $numSvL1Rover0__ \
                -bg $numSvL1Rover0Color__
        .configDisplay.r5.e5 configure -text $gpsQuality__ \
                -bg $gpsQualityColor__
    }

    proc actNumSats { numSvL1Base__ numSvL1BaseColor__ \
            numSvL1Rover0__ numSvL1Rover0Color__ } {
        .configDisplay.r5.e3 configure -text $numSvL1Base__ \
                -bg $numSvL1BaseColor__
        .configDisplay.r5.e1 configure -text $numSvL1Rover0__ \
                -bg $numSvL1Rover0Color__
    }

    proc actPosQual {gpsQuality__ gpsQualityColor__ } {
        .configDisplay.r5.e5 configure -text $gpsQuality__ \
                -bg $gpsQualityColor__
    }

    proc actPosError { posError__ } {
        .configDisplay.r1.l2 configure -text $posError__
    }
    
    proc actPosUnits { posUnits__ } {
        .configDisplay.r1.l3 configure -text $posUnits__
    }
    
    proc actUnpackPosUnits { } {
        pack forget .configDisplay.r1.l3
    }
    
    proc actPackPosUnits { } {
        pack .configDisplay.r1.l3 -side right
    }
    
    proc actRepackPosUnits { } {
        pack forget .configDisplay.r1.l2
        pack forget .configDisplay.r1.l3
        pack .configDisplay.r1.l3 -side right
        pack .configDisplay.r1.l2 -side right
    }
    
    proc actConfigLanguage {accuracy userLevel implement vehicle\
                            tractorSats baseSats qual} {
        .configDisplay.r1.l configure -text $accuracy
        .configDisplay.r2.l configure -text $userLevel
        .configDisplay.r3.l configure -text $implement
        .configDisplay.r4.l configure -text $vehicle
        .configDisplay.r5.e2 configure -text $tractorSats
        .configDisplay.r5.e4 configure -text $baseSats
        .configDisplay.r5.e6 configure -text $qual
    }
}


#=============================================================================
# .navDisplay
#
# The AutoFarm navigation display. A frame which can be packed into any 
# screen.
#=============================================================================
namespace eval NavDisplay { 

    frame .navDisplay -height $navHeight -width $navWidth
    frame .navDisplay.ready
    frame .navDisplay.meas
    frame .navDisplay.meas.pos
    frame .navDisplay.meas.datalink
    frame .navDisplay.meas.att
    frame .navDisplay.meas.steer

    label .navDisplay.ready.b1 -image btnRED
    
    pack .navDisplay.ready.b1 -side left
    label .navDisplay.ready.t -text "Ready for AutoSteer" -font $medium_font
    pack .navDisplay.ready.t -side left

    foreach { meas } { pos datalink att steer } {
        label .navDisplay.meas.$meas.t -font $medium_font
        label .navDisplay.meas.$meas.0 -image btnGREY
        label .navDisplay.meas.$meas.1 -image btnGREY
        label .navDisplay.meas.$meas.2 -image btnGREY
        pack .navDisplay.meas.$meas.t -side right
        pack .navDisplay.meas.$meas.0 -side right
        pack .navDisplay.meas.$meas.1 -side right
        pack .navDisplay.meas.$meas.2 -side right
    }
    pack .navDisplay.meas.steer -side bottom -anchor w
    pack .navDisplay.meas.att -side bottom -anchor w
    pack .navDisplay.meas.datalink -side bottom -anchor w
    pack .navDisplay.meas.pos -side bottom -anchor w
    pack .navDisplay.ready -side bottom -anchor w
    pack .navDisplay.meas -side left -anchor s

    proc actIndicators {status_pos status_datalink status_att 
                        status_steer status_autofarm} {
        global color6 colorMedium colorBad

        foreach { meas } { pos datalink att steer} {
            set status [set status_$meas]
            if {$status == -1} {
                .navDisplay.meas.$meas.0 configure -image btnGREY
                .navDisplay.meas.$meas.1 configure -image btnGREY
                .navDisplay.meas.$meas.2 configure -image btnGREY
            } elseif {$status == 0} {
                .navDisplay.meas.$meas.0 configure -image btnGREY
                .navDisplay.meas.$meas.1 configure -image btnGREY
                .navDisplay.meas.$meas.2 configure -image btnGREEN
            } elseif {$status == 1} {
                .navDisplay.meas.$meas.0 configure -image btnGREY
                .navDisplay.meas.$meas.1 configure -image btnYELLOW
                .navDisplay.meas.$meas.2 configure -image btnGREY
            } elseif {$status == -2} {
                .navDisplay.meas.$meas.0 configure -image btnGREY
                .navDisplay.meas.$meas.1 configure -image btnGREY
                .navDisplay.meas.$meas.2 configure -image btnGREY
            } else {
                .navDisplay.meas.$meas.0 configure -image btnRED
                .navDisplay.meas.$meas.1 configure -image btnGREY
                .navDisplay.meas.$meas.2 configure -image btnGREY
            }
        }

        if {$status_autofarm == 0} {
            .navDisplay.ready.b1 configure -image btnGREEN
        } elseif {$status_autofarm == 1} {
            .navDisplay.ready.b1 configure -image btnYELLOW
        } else {
            .navDisplay.ready.b1 configure -image btnRED
        }
    }

    proc actConfigLanguage {ready pos datalink att steer} {
        .navDisplay.ready.t         configure -text $ready
        .navDisplay.meas.pos.t      configure -text $pos
        .navDisplay.meas.datalink.t configure -text $datalink
        .navDisplay.meas.att.t      configure -text $att
        .navDisplay.meas.steer.t    configure -text $steer
    }

    proc actConfigBaseBatteryOk {} {
        global color6
        .navDisplay.meas.datalink.t configure -background $color6
    }

    proc actConfigBaseBatteryLow {} {
        .navDisplay.meas.datalink.t configure -background yellow
    }
}

#=============================================================================
# .wizardTopDisplay
#
# The Implement Steer Wizard top of screen.
#=============================================================================
namespace eval wizardTopDisplay { 

#    frame .wizardTopDisplay -height $wizNavHeight -width $winWidth
     frame .wizardTopDisplay -height $wizNavHeight -width 300

    frame .wizardTopDisplay.wReady
    frame .wizardTopDisplay.wReady.top
    frame .wizardTopDisplay.wReady.bottom
    frame .wizardTopDisplay.wProgress
    frame .wizardTopDisplay.wHelp

    label .wizardTopDisplay.wReady.top.header -text "Implement Steering Wizard" -font $medium_font
    pack .wizardTopDisplay.wReady.top.header -side left
    pack .wizardTopDisplay.wReady.top -side top -anchor nw

    label .wizardTopDisplay.wReady.bottom.b1 -image btnRED
    pack .wizardTopDisplay.wReady.bottom.b1 -side left
    label .wizardTopDisplay.wReady.bottom.t -text "Ready for AutoSteer" -font $medium_font
    pack .wizardTopDisplay.wReady.bottom.t -side left
    pack .wizardTopDisplay.wReady.bottom.t -side left
    pack .wizardTopDisplay.wReady.bottom -side bottom -anchor sw

    label .wizardTopDisplay.wProgress.t -text " " -font $medium_font
    pack .wizardTopDisplay.wProgress.t -side top

    label .wizardTopDisplay.wProgress.b1 -image btnWizardProgressGray
    pack .wizardTopDisplay.wProgress.b1 -side left
    label .wizardTopDisplay.wProgress.b2 -image btnWizardProgressGray
    pack .wizardTopDisplay.wProgress.b2 -side left
    label .wizardTopDisplay.wProgress.b3 -image btnWizardProgressGray
    pack .wizardTopDisplay.wProgress.b3 -side left
    label .wizardTopDisplay.wProgress.b4 -image btnWizardProgressGray
    pack .wizardTopDisplay.wProgress.b4 -side left
    label .wizardTopDisplay.wProgress.b5 -image btnWizardProgressGray
    pack .wizardTopDisplay.wProgress.b5 -side left
    label .wizardTopDisplay.wProgress.b6 -image btnWizardProgressGray
    pack .wizardTopDisplay.wProgress.b6 -side left

#   Help button
    image create photo btnBlueQuestionU -format gif -file \
	${tclPath}gif/btnBlueQuestionU.gif
    image create photo btnBlueQuestionD -format gif -file \
	${tclPath}gif/btnBlueQuestionD.gif
    SnazzyButton::create .wizardTopDisplay.wHelp.b0 btnBlueQuestionU btnBlueQuestionD 0.5 \
                         "beepOnce; post $DISP_WIZARD_HELP_SIG 48" 

#   spacer
    label .wizardTopDisplay.wProgress.tH -text "     " -font $medium_font
    pack .wizardTopDisplay.wProgress.tH -side right

    pack .wizardTopDisplay.wHelp -side right

#   bottom line
   frame .wizardTopDisplay.wSep -height 2 -width $winWidth -borderwidth 0 \
            -background $colorInx
    pack .wizardTopDisplay.wSep -side bottom -anchor w

    pack .wizardTopDisplay.wReady -side left -fill x -expand yes -anchor w
    pack .wizardTopDisplay.wProgress -side right
     
     proc actHeaderLabel { header ready } {
	 .wizardTopDisplay.wReady.top.header configure -text $header
	 .wizardTopDisplay.wReady.bottom.t configure -text $ready
     }

     proc actWizardLabel { wizardName } {
           .wizardTopDisplay.wProgress.t configure -text $wizardName
     }

     proc actAddWizardHelp { } {
	 pack .wizardTopDisplay.wHelp.b0 -side right
     }

     proc actDeleteWizardHelp { } {
	 pack forget .wizardTopDisplay.wHelp.b0
     }
  
    proc actIndicators { status_autofarm} {

        global color6 colorMedium colorBad

        if {$status_autofarm == 0} {
            .wizardTopDisplay.wReady.bottom.b1 configure -image btnGREEN
        } elseif {$status_autofarm == 1} {
            .wizardTopDisplay.wReady.bottom.b1 configure -image btnYELLOW
        } else {
            .wizardTopDisplay.wReady.bottom.b1 configure -image btnRED
        }
    }
    
    proc actConfigIsKillSwitchOn { killSwitchText } {
      .wizardTopDisplay.wReady.bottom.t configure -text $killSwitchText
      .wizardTopDisplay.wReady.bottom.b1 configure -image warningImg    
    }
    
    proc actConfigIsKillSwitchOff { readyText } {
      .wizardTopDisplay.wReady.bottom.t configure -text $readyText    
    }

    proc actConfigProgress { num_wizards w1 w2 w3 w4 w5 w6 } {
	if { $num_wizards > 0 } {
	    if { $w1 == 0 || $w1 == 1} {
		.wizardTopDisplay.wProgress.b1 configure -image btnWizardProgressBlue
	    } else {
		.wizardTopDisplay.wProgress.b1 configure -image btnWizardProgressBlueChecked
	    }
	    pack .wizardTopDisplay.wProgress.b1 -side left
        }
	if {$num_wizards >  1} {
	    if {$w2 == 0} {
		.wizardTopDisplay.wProgress.b2 configure -image btnWizardProgressGray
	    } elseif {$w2 == 1} {
		.wizardTopDisplay.wProgress.b2 configure -image btnWizardProgressBlue
	    } else {
		.wizardTopDisplay.wProgress.b2 configure -image btnWizardProgressBlueChecked
	    }
	    pack .wizardTopDisplay.wProgress.b2 -side left
        }
	if {$num_wizards >  2} {
	    if {$w3 == 0} {
		.wizardTopDisplay.wProgress.b3 configure -image btnWizardProgressGray
	    } elseif {$w3 == 1} {
		.wizardTopDisplay.wProgress.b3 configure -image btnWizardProgressBlue
	    } else {
		.wizardTopDisplay.wProgress.b3 configure -image btnWizardProgressBlueChecked
	    }
	    pack .wizardTopDisplay.wProgress.b3 -side left
        }
	if {$num_wizards >  3} {
	    if {$w4 == 0} {
		.wizardTopDisplay.wProgress.b4 configure -image btnWizardProgressGray
	    } elseif {$w4 == 1} {
		.wizardTopDisplay.wProgress.b4 configure -image btnWizardProgressBlue
	    } else {
		.wizardTopDisplay.wProgress.b4 configure -image btnWizardProgressBlueChecked
	    }
	    pack .wizardTopDisplay.wProgress.b4 -side left
        }
	if {$num_wizards >  4} {
	    if {$w5 == 0} {
		.wizardTopDisplay.wProgress.b5 configure -image btnWizardProgressGray
	    } elseif {$w5 == 1} {
		.wizardTopDisplay.wProgress.b5 configure -image btnWizardProgressBlue
	    } else {
		.wizardTopDisplay.wProgress.b5 configure -image btnWizardProgressBlueChecked
	    }
	    pack .wizardTopDisplay.wProgress.b5 -side left
        }
	if {$num_wizards >  5} {
	    if {$w6 == 0} {
		.wizardTopDisplay.wProgress.b6 configure -image btnWizardProgressGray
	    } elseif {$w6 == 1} {
		.wizardTopDisplay.wProgress.b6 configure -image btnWizardProgressBlue
	    } else {
		.wizardTopDisplay.wProgress.b6 configure -image btnWizardProgressBlueChecked
	    }
	    pack .wizardTopDisplay.wProgress.b6 -side left
        }

    }
 
}

#=============================================================================
# .tractorDisplay
#
# The AutoFarm tractor display. A frame which can be packed into any screen.
#=============================================================================
namespace eval TractorDisplay { 
    
    frame .tractorDisplay

    # Speed text and bar
    label .tractorDisplay.speedText -font $medium_font
    
    canvas .tractorDisplay.speedBar -width 200 -height 20 -relief sunken \
            -borderwidth 1 -background $color6
    .tractorDisplay.speedBar \
            create rectangle 100 0 100 20 -fill green -outline black \
            -tags Vxbar820

    # Tractor and implement display canvas 
    # (1 pixel approximately equals 10 cm)
    canvas .tractorDisplay.c -width 200 -height 200 -relief sunken \
            -borderwidth 1 -background blue

    # bind <ButtonPress-1> to DISP_CANVAS0_SIG to enable JobMark
    bind .tractorDisplay.c <ButtonPress-1> \
            { beepOnce; post $DISP_CANVAS1_SIG 0 }

    pack .tractorDisplay.c -side bottom -anchor e -pady 10 -padx 35 
    pack .tractorDisplay.speedBar -side bottom -anchor e -padx 35 
    pack .tractorDisplay.speedText -side bottom -anchor e -padx 35 


    proc drawTractor { me x y type color implWidth implOffs } {
        
        set bodyWidth   20
        set bodyHeight  55
        set implHeight  10

        set x [expr $x - $bodyWidth/2]
        set tractorColor(0) blue
        set tractorColor(1) green
        set tractorColor(2) orange
        set tractorColor(3) red
        set tractorColor(4) yellow
        
        set color $tractorColor($color)
        
        $me create rectangle $x $y \
                             [expr $x + $bodyWidth] [expr $y - $bodyHeight] \
                             -fill $color -outline black
        if {$type == 1} {                     
            $me create line [expr $x - 6] [expr $y - $bodyHeight - 4] \
                            [expr $x - 6] [expr $y - $bodyHeight + 8] \
                            -fill black -width 6 
            $me create line [expr $x + $bodyWidth+7] [expr $y-$bodyHeight -4]\
                            [expr $x + $bodyWidth+7] [expr $y-$bodyHeight +8]\
                            -fill black -width 6 
            set h 24
        } else {
            set h 40
        }
                             
        $me create line [expr $x - 6] [expr $y + 6] \
                        [expr $x - 6] [expr $y + 6 - $h] \
                        -fill black -width 8 
        $me create line [expr $x + $bodyWidth + 7] [expr $y + 6] \
                        [expr $x + $bodyWidth + 7] [expr $y + 6 - $h] \
                        -fill black -width 8
                        
        $me create line [expr $x + $bodyWidth/2] [expr $y] \
                        [expr $x + $bodyWidth/2 + $implOffs] [expr $y + 16] \
                        -fill black -width 1
                        
        $me create rectangle [expr $x +($bodyWidth-$implWidth)/2 + $implOffs]\
                             [expr $y + $implHeight + 16] \
                             [expr $x +($bodyWidth+$implWidth)/2 + $implOffs]\
                             [expr $y + 16] \
                             -fill $color -outline black
        return $me 
    }

    proc actConfigTractorIcon { tractorType__ tractorColor__ \
            implWidth__ implOffset__ } {
        SetTractorIconCoords $implOffset__
        ReconfigTractorIcon $tractorType__ $implWidth__ $tractorColor__
    }

    proc actConfigureRowlineup { manualSteeringText__ } {
        global colorRowlineup autosteerManualSteering
        .tractorDisplay.c configure -background $colorRowlineup
        .tractorDisplay.c itemconfigure $autosteerManualSteering \
                -text $manualSteeringText__
    }

    proc actConfigureAutosteer { } {
        global colorAutosteer autosteerManualSteering
        .tractorDisplay.c configure -background $colorAutosteer
        .tractorDisplay.c itemconfigure $autosteerManualSteering -text " "
    }

    proc actVehicleDispData { velDispText__ Vx__ VxMax__ \
            headingError__ posError__ \
            passWidth__ fanAngle__ rowNumText__ nudgeDistText__ \
            turboMessageText__ ENHANCED_RLM__ \
            ERLM_HE_ENTER__ ERLM_HE_EXIT__ \
            ERLM_YC_ENTER__ ERLM_YC_EXIT__ \
            inTightMode__} {

        global rowHeadingErr rowPosErr
        set rowHeadingErr $headingError__
        set rowPosErr $posError__

        global autosteerRowNum autosteerNudgeDist autosteerTurboMessage
        .tractorDisplay.c itemconfigure $autosteerRowNum \
                -text $rowNumText__
        .tractorDisplay.c itemconfigure $autosteerNudgeDist \
                -text $nudgeDistText__
        .tractorDisplay.c itemconfigure $autosteerTurboMessage \
                -text $turboMessageText__

        ConfigureTractorIcon $passWidth__ $fanAngle__ $ENHANCED_RLM__ \
                $ERLM_HE_ENTER__ $ERLM_HE_EXIT__ \
                $ERLM_YC_ENTER__ $ERLM_YC_EXIT__ \
                $inTightMode__ $Vx__
        ConfigureVelocityDisplay $velDispText__ $Vx__ $VxMax__
    }

    proc actSteerCodeDisp { turningWheelText__ killText__ clutchText__ \
            parkText__ resetReqText__ roadingText__ comsBadText__ \
            notReadyText__ inxUnsText__ } {

        global autosteerTurningWheel autosteerKillSwitchOn \
                autosteerMtClutchIn autosteerMtInPark autosteerMtResetReqd \
                autosteerMtRoading autosteerMtComsBad autosteerMtNotReady \
                autosteerMtInxUns
        .tractorDisplay.c itemconfigure $autosteerTurningWheel \
                -text $turningWheelText__
        .tractorDisplay.c itemconfigure $autosteerKillSwitchOn \
                -text $killText__
        .tractorDisplay.c itemconfigure $autosteerMtClutchIn \
                -text $clutchText__
        .tractorDisplay.c itemconfigure $autosteerMtInPark \
                -text $parkText__
        .tractorDisplay.c itemconfigure $autosteerMtResetReqd \
                -text $resetReqText__
        .tractorDisplay.c itemconfigure $autosteerMtRoading \
                -text $roadingText__
        .tractorDisplay.c itemconfigure $autosteerMtComsBad \
                -text $comsBadText__
        .tractorDisplay.c itemconfigure $autosteerMtNotReady \
                -text $notReadyText__
        .tractorDisplay.c itemconfigure $autosteerMtInxUns \
                -text $inxUnsText__
    }

    proc actConfigLanguage {speed} {
        .tractorDisplay.speedText configure -text $speed
    }
    
    proc actMarkPixels { xPixels yPixels xStart yStart xEnd yEnd } {
        # set the mark coordinates
        global autosteerMark autosteerMarkSide
        .tractorDisplay.c coords $autosteerMark \
                [expr $xPixels-$autosteerMarkSide/2] $yPixels \
                [expr $xPixels+$autosteerMarkSide/2] $yPixels \
                $xPixels [expr $yPixels-$autosteerMarkSide]

        # set the arrow coordinates
        global autosteerMarkArrow
        .tractorDisplay.c coords $autosteerMarkArrow \
                $xStart $yStart $xEnd $yEnd
    }

    proc actSurveyMarkPixels { xPixels yPixels northID eastID } {
        # set the survey mark coordinates
        global autosteerSurveyMark autosteerSurveyText autosteerMarkSide
        .tractorDisplay.c coords $autosteerSurveyMark \
                [expr $xPixels-$autosteerMarkSide/2] $yPixels \
                [expr $xPixels+$autosteerMarkSide/2] $yPixels \
                $xPixels [expr $yPixels-$autosteerMarkSide]
        .tractorDisplay.c coords $autosteerSurveyText \
                $xPixels [expr $yPixels+7]
        .tractorDisplay.c itemconfigure $autosteerSurveyText \
                -text "($northID,$eastID)"
    }

}


#=============================================================================
#
# The AutoFarm sliders. A bunch of sliders that can be packed into any screen.
#=============================================================================
namespace eval Sliders { 
    
    proc slider { me numSteps sig } {
        global medium_font big_font color3 color6

#        set readoutWidth  125
#        set readoutHeight  45
#        set btnWidth       60
#        set titleHeight    20

        set readoutWidth  115
        set readoutHeight  45
        set btnWidth       55
        set titleHeight    18
        
        canvas $me -width [expr $readoutWidth + 2*$btnWidth + 4] \
            -height 80 -background $color6
        
		# left button
	SnazzyButton::create $me.neg btnBlueSlider btnBlueSlider 0.5 \
            "beepOnce; post $sig -1" "-"
        SnazzyButton::setText $me.neg "-" 0.5                
        
		# right button
	SnazzyButton::create $me.pos btnBlueSlider btnBlueSlider 0.5 \
            "beepOnce; post $sig 1" "+"
        SnazzyButton::setText $me.pos "+" 0.5

        $me create window [expr 2 + $btnWidth] $titleHeight \
            -anchor ne -window $me.neg
        $me create window [expr 6 + $btnWidth + $readoutWidth] $titleHeight \
            -anchor nw -window $me.pos
        $me create text [expr 2 + $btnWidth + 2 + $readoutWidth/2] 2 \
            -anchor n -font $medium_font -tags title    
        $me create text [expr 2 + $btnWidth + 6] [expr $titleHeight + 0] \
            -anchor nw -font $medium_font -text 10 -tags value
        $me create text [expr 25 + $btnWidth + $readoutWidth/2] 65 \
            -anchor n -justify left -font $medium_font -tags center    
        
        # create an off-screen text to store number of steps        
        $me create text -20 0 -text $numSteps -tags numSteps
                

        set x0 [expr 2 + $btnWidth + 2]

        set notchWidth  [expr 0.6*$readoutWidth/$numSteps]
        for {set i 0} {$i < $numSteps} {incr i} {
            set x  [expr $x0 + $readoutWidth*($i + 0.5)/$numSteps]
            set y1 [expr $titleHeight + \
                    $readoutHeight*(0.9 - 0.8*($i+0.5)/$numSteps)]
            set y2 [expr $titleHeight + $readoutHeight]
            $me create line $x $y1 $x $y2 -width $notchWidth \
                -fill gray -tags notch$i
        }

        set x1 [expr $x0 + $readoutWidth/2 - 1]
        set x2 [expr $x0 + $readoutWidth]
        set y1 [expr $titleHeight + $readoutHeight - 3]
        set y2 [expr $titleHeight + $readoutHeight]
        $me create line $x0 $y2 $x2 $y2 
        $me create line $x0 $y1 $x0 $y2 
        $me create line $x1 $y1 $x1 $y2 
        $me create line $x2 $y1 $x2 $y2
        
        return $me 
    }

    proc sliderConfigureTitle { me title } {
       $me itemconfigure title -text $title
    }

    proc sliderConfigureCenter { me center } {
       $me itemconfigure center -text $center
    }

    proc sliderConfigureButtons { me negBtn posBtn } {
        SnazzyButton::setText $me.neg $negBtn 0.5
        SnazzyButton::setText $me.pos $posBtn 0.5 
    }

    proc sliderConfigureBackground { me backgroundColor } {
        SnazzyButton::setBackground $me.neg $backgroundColor
        SnazzyButton::setBackground $me.pos $backgroundColor
	$me configure -background $backgroundColor
    }

    proc sliderSetValue { me val notch } {
        $me itemconfigure value -text $val
        set numSteps [$me itemcget numSteps -text]
        for {set i 0} {$i < $notch} {incr i} {
            $me itemconfigure notch$i -fill "green"
        }
        for {set i $notch} {$i < $numSteps} {incr i} {
            $me itemconfigure notch$i -fill "gray"
        }
    }

    slider .s0 20 $DISP_SLDR0_SIG   ;# drives CONT_LEVEL 
    slider .s1 20 $DISP_SLDR1_SIG   ;# drives REV_CONT_LEVEL
    slider .s2 20 $DISP_SLDR2_SIG   ;# drives TRAJ_CONT_LEVEL
    slider .s3 20 $DISP_SLDR3_SIG   ;# drives CAT_DB_LEVEL
    slider .s4 20 $DISP_SLDR4_SIG   ;# drives CAT_DB_POS_LEVEL
    slider .s5 20 $DISP_SLDR5_SIG   ;# drives CAT_DB_NEG_LEVEL
    slider .s6 20 $DISP_SLDR6_SIG   ;# drives K_ROLL
    slider .s7 20 $DISP_SLDR7_SIG   ;# drives Implement Steer Resp Rate
    slider .s8 20 $DISP_SLDR8_SIG   ;# drives NUM_PRESS_SAMPLES

    proc actSliderValue { sliderNum val notch } {
        Sliders::sliderSetValue .s$sliderNum $val $notch
    }
    
    proc actConfigLanguage {s0_title s1_title s2_title
                            s3_title s3_neg s3_pos
                            s4_title s5_title } {
        
        sliderConfigureTitle .s0 $s0_title
        sliderConfigureTitle .s1 $s1_title    
        sliderConfigureTitle .s2 $s2_title    
        sliderConfigureTitle .s3 $s3_title    
        sliderConfigureButtons .s3 $s3_neg $s3_pos
        sliderConfigureTitle .s4 $s4_title    
        sliderConfigureTitle .s5 $s5_title    
        sliderConfigureTitle .s6 "K_ROLL"
        sliderConfigureTitle .s7 "Implement Response Rate"
        sliderConfigureTitle .s8 "% of Default  "
        sliderConfigureCenter .s8 "Default"
    }

}


#=============================================================================
# .stateHistoryDisplay
#
# displays a list of string is sequential order.  May be packed into any
# screen.  
#=============================================================================
namespace eval StateHistoryDisplay {

    # define the containing frame
    frame .stateHistoryDisplay

    # define the canvas
    canvas .stateHistoryDisplay.c -width 400 -height 300 -relief sunken \
            -borderwidth 1 -background gray85
    pack .stateHistoryDisplay.c -side bottom -padx 10 -pady 1

    .stateHistoryDisplay.c create text 10 12 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state1Tag}
    .stateHistoryDisplay.c create text 10 24 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state2Tag}
    .stateHistoryDisplay.c create text 10 36 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state3Tag}
    .stateHistoryDisplay.c create text 10 48 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state4Tag}
    .stateHistoryDisplay.c create text 10 60 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state5Tag}
    .stateHistoryDisplay.c create text 10 84 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state6Tag}
    .stateHistoryDisplay.c create text 10 96 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state7Tag}
    .stateHistoryDisplay.c create text 10 108 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state8Tag}
    .stateHistoryDisplay.c create text 10 120 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state9Tag}
    .stateHistoryDisplay.c create text 10 132 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state10Tag}
    .stateHistoryDisplay.c create text 10 144 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state11Tag}
    .stateHistoryDisplay.c create text 10 156 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state12Tag}
    .stateHistoryDisplay.c create text 10 168 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state13Tag}
    .stateHistoryDisplay.c create text 10 180 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state14Tag}
    .stateHistoryDisplay.c create text 10 204 -text ""\
    -font $tiny_font -anchor w -tags {cdInfo state15Tag}

    proc setMessage { msgId msgString } {
	if { $msgId == "1" } {
	    .stateHistoryDisplay.c itemconfigure state1Tag -text $msgString
	}
	if { $msgId == "2" } {
	    .stateHistoryDisplay.c itemconfigure state2Tag -text $msgString
	}
	if { $msgId == "3" } {
	    .stateHistoryDisplay.c itemconfigure state3Tag -text $msgString
	}
	if { $msgId == "4" } {
	    .stateHistoryDisplay.c itemconfigure state4Tag -text $msgString
	}
	if { $msgId == "5" } {
	    .stateHistoryDisplay.c itemconfigure state5Tag -text $msgString
	}
	if { $msgId == "6" } {
	    .stateHistoryDisplay.c itemconfigure state6Tag -text $msgString
	}
	if { $msgId == "7" } {
	    .stateHistoryDisplay.c itemconfigure state7Tag -text $msgString
	}
	if { $msgId == "8" } {
	    .stateHistoryDisplay.c itemconfigure state8Tag -text $msgString
	}
	if { $msgId == "9" } {
	    .stateHistoryDisplay.c itemconfigure state9Tag -text $msgString
	}
	if { $msgId == "10" } {
	    .stateHistoryDisplay.c itemconfigure state10Tag -text $msgString
	}
	if { $msgId == "11" } {
	    .stateHistoryDisplay.c itemconfigure state11Tag -text $msgString
	}
	if { $msgId == "12" } {
	    .stateHistoryDisplay.c itemconfigure state12Tag -text $msgString
	}
	if { $msgId == "13" } {
	    .stateHistoryDisplay.c itemconfigure state13Tag -text $msgString
	}
	if { $msgId == "14" } {
	    .stateHistoryDisplay.c itemconfigure state14Tag -text $msgString
	}
	if { $msgId == "15" } {
	    .stateHistoryDisplay.c itemconfigure state15Tag -text $msgString
	}
    }

    proc clearAllMessages {} {
	.stateHistoryDisplay.c itemconfigure state1Tag -text ""
	.stateHistoryDisplay.c itemconfigure state2Tag -text ""
	.stateHistoryDisplay.c itemconfigure state3Tag -text ""
	.stateHistoryDisplay.c itemconfigure state4Tag -text ""
	.stateHistoryDisplay.c itemconfigure state5Tag -text ""
	.stateHistoryDisplay.c itemconfigure state6Tag -text ""
	.stateHistoryDisplay.c itemconfigure state7Tag -text ""
	.stateHistoryDisplay.c itemconfigure state8Tag -text ""
	.stateHistoryDisplay.c itemconfigure state9Tag -text ""
	.stateHistoryDisplay.c itemconfigure state10Tag -text ""
	.stateHistoryDisplay.c itemconfigure state11Tag -text ""
	.stateHistoryDisplay.c itemconfigure state12Tag -text ""
	.stateHistoryDisplay.c itemconfigure state13Tag -text ""
	.stateHistoryDisplay.c itemconfigure state14Tag -text ""
	.stateHistoryDisplay.c itemconfigure state15Tag -text ""
    }
}


#=============================================================================
# .calDataDisplay
#
# The calibration-data display. Displays input-output graphs during 
# calibration.
#=============================================================================
namespace eval CalDataDisplay {

    # define the containing frame
    frame .calDataDisplay

    # define the canvas
    canvas .calDataDisplay.c -width 400 -height 300 -relief sunken \
            -borderwidth 1 -background gray85
    pack .calDataDisplay.c -side bottom -padx 10 -pady 1

    set calRawAngleMarker \
            [.calDataDisplay.c create line 500 0 500 300 -width 5]
    set calUrawMarker [.calDataDisplay.c create line 0 0 0 200 -width 3]

    proc actPlotCalDataWork { calfilename } {

        global small_font 
        global xscale KNX0 yscale KNY0
        global calUrawMarker calRawAngleMarker

        if [catch {open $calfilename r} fileId ] {
            puts stdout "Cannot open $calfilename: $fileId\n"
        } else  {
            puts stdout "Opened $calfilename: $fileId\n"

            # clear the canvas
            foreach id [.calDataDisplay.c find all] {
                if { $id != $CalDataDisplay::calUrawMarker && \
                        $id != $CalDataDisplay::calRawAngleMarker } {
                    .calDataDisplay.c delete $id
                }
            }

            # plot the new data
            while { [gets $fileId marker] > 0 } {

                if { $marker == "xaxis" } {
                    gets $fileId xaxis
                    eval {.calDataDisplay.c create line} $xaxis \
                            {-stipple gray50}
                } elseif { $marker == "yaxis" } {
                    gets $fileId yaxis
                    eval {.calDataDisplay.c create line} $yaxis \
                            {-stipple gray50}
                } elseif { $marker == "scaling" } {
                    gets $fileId scaling
                    set xscale [lindex $scaling 0]
                    set KNX0 [lindex $scaling 1]
                    set yscale [lindex $scaling 2]
                    set KNY0 [lindex $scaling 3]
                } elseif { $marker == "posPolyOld" } {
                    gets $fileId posPolyOld
                    eval {.calDataDisplay.c create line} \
                            $posPolyOld {-fill magenta -width 2}
                } elseif { $marker == "negPolyOld" } {
                    gets $fileId negPolyOld
                    eval {.calDataDisplay.c create line} \
                            $negPolyOld {-fill magenta -width 2}
                } elseif { $marker == "posPolyNew" } {
                    gets $fileId posPolyNew
                    eval {.calDataDisplay.c create line} \
                            $posPolyNew {-fill blue -width 2}
                } elseif { $marker == "negPolyNew" } {
                    gets $fileId negPolyNew
                    eval {.calDataDisplay.c create line} \
                            $negPolyNew {-fill blue -width 2}
                } elseif { $marker == "oldPosPolyText" } {
                    gets $fileId oldPosPolyText
                    .calDataDisplay.c create text 300 240 \
                            -text $oldPosPolyText -font $small_font
                } elseif { $marker == "newPosPolyText" } {
                    gets $fileId newPosPolyText
                    .calDataDisplay.c create text 300 260 \
                            -text $newPosPolyText -font $small_font
                } elseif { $marker == "oldNegPolyText" } {
                    gets $fileId oldNegPolyText
                    .calDataDisplay.c create text 100 40 \
                            -text $oldNegPolyText -font $small_font
                } elseif { $marker == "newNegPolyText" } {
                    gets $fileId newNegPolyText
                    .calDataDisplay.c create text 100 60 \
                            -text $newNegPolyText -font $small_font
                } elseif { $marker == "newMaxPosRateText" } {
                    gets $fileId newMaxPosRateText
                    .calDataDisplay.c create text 300 280 \
                            -text $newMaxPosRateText -font $small_font
                } elseif { $marker == "newMaxNegRateText" } {
                    gets $fileId newMaxNegRateText
                    .calDataDisplay.c create text 100 80 \
                            -text $newMaxNegRateText -font $small_font
                } elseif { $marker == "datapoints" } {
                    while { [gets $fileId datapoint] > 0 } {
                        eval {.calDataDisplay.c create text} \
                                $datapoint {-text "x" -font $small_font}
                    }   
                }
            }
            
            close $fileId
        }
    }


    proc actPlotCalData { } {

        puts "actPlotCalData"

	actPlotCalDataWork "/tmp/calDataFile.txt" 
	actPlotCalDataWork "/tmp/implCalDataFile.txt" 

    }


    proc actResetUrawMarker { } {
        .calDataDisplay.c coords $CalDataDisplay::calUrawMarker \
            0 0 0 300
        .calDataDisplay.c itemconfigure $CalDataDisplay::calUrawMarker \
            -fill red
    }

    proc actUpdateUrawMarker { color__ uraw__ } {

        global xscale KNX0 yscale KNY0
        global calUrawMarker

        set uRawScaled [expr round($uraw__*$xscale+$KNX0)]
        .calDataDisplay.c coords $CalDataDisplay::calUrawMarker \
            $uRawScaled 0 $uRawScaled 300
        if { $color__ != "!"} {
            .calDataDisplay.c itemconfigure $CalDataDisplay::calUrawMarker \
                -fill $color__
        }
    }

    proc actUpdateRawAngleMarker { color__ rawAngle__ } {

        global xscale KNX0 yscale KNY0
        global calRawAngleMarker

        set rawAngleScaled [expr round($rawAngle__*$xscale+$KNX0)]
        .calDataDisplay.c coords $CalDataDisplay::calRawAngleMarker \
                $rawAngleScaled 0 $rawAngleScaled 300
        .calDataDisplay.c itemconfigure $CalDataDisplay::calRawAngleMarker \
                -fill $color__
    }

    proc actHideRawAngleMarker { } {
        .calDataDisplay.c coords $CalDataDisplay::calRawAngleMarker \
                500 0 500 300
    }
}


#=============================================================================
# .glCurveDisp
#
# The curve-data display. Display canvas for complex curve capabilities. Can
# be packed into any screen.
#=============================================================================
namespace eval GlCurveDisp {

    # define the containing frame
    frame .glCurveDisp -width 400 -height 400
    pack propagate .glCurveDisp 0

    global startx starty xangle0 yangle0 xangle yangle
    set xangle 0.0
    set yangle 0.0

    togl .glCurveDisp.g1 -width 300 -height 300 \
        -ident ".glCurveDisp.g1" -rgba true -double true \
        -depth true -privatecmap false -time 200
    bind .glCurveDisp.g1 <ButtonPress-1> \
        {GlCurveDisp::userRotSetStart %x %y %W}
    bind .glCurveDisp.g1 <B1-Motion> \
        {GlCurveDisp::userRotMove %x %y %W}
    pack .glCurveDisp.g1 -expand true -fill both

    proc userRotSetStart {x y W } {
        $W cuserRotSetStart $x $y
    }

    proc userRotMove {x y W} {
        $W cuserRotMove $x $y
    }
}


#=============================================================================
# .glImplementStatusDisplay
#
# Widget to graphically display the status of implement steering
#=============================================================================
namespace eval GlImplementStatusDisp {

    # define the containing frame
    frame .glImplementStatusDisp -width 284 -height 130 -background "black"
    pack propagate .glImplementStatusDisp 0

    global startx starty xangle0 yangle0 xangle yangle
    set xangle 0.0
    set yangle 0.0

    togl .glImplementStatusDisp.g1 -width 284 -height 130 \
        -ident ".glImplementStatusDisp.g1" -rgba true -double true \
        -depth true -privatecmap false -time 200
    pack .glImplementStatusDisp.g1 
#-expand true -fill both

    proc setNarrow { } {
	.glImplementStatusDisp.g1 configure -width 100 -height 130
	.glImplementStatusDisp configure -width 100 -height 130
    }

    proc setWide { } {
	.glImplementStatusDisp.g1 configure -width 284 -height 130
	.glImplementStatusDisp configure -width 284 -height 130
    }
}


#=============================================================================
# .glContDataDisp
#
# The controller-data display. A general data-plotting display, which can be 
# packed into any screen. 
#=============================================================================
namespace eval GlContDataDisp {

    # define the containing frame
    frame .glContDataDisp

    togl .glContDataDisp.g1 -width 300 -height 130 \
        -ident ".glContDataDisp.g1" -rgba true -double true \
        -depth true -privatecmap false -time 200
    bind .glContDataDisp.g1 <ButtonPress-1> \
        {GlContDataDisp::userZoomPress %x %y %W}
    bind .glContDataDisp.g1 <ButtonRelease-1> \
        {GlContDataDisp::userZoomRelease %x %y %W}
    pack .glContDataDisp.g1 -side top -expand true -fill both

    frame .glContDataDisp.panel

    frame .glContDataDisp.panel.info
    label .glContDataDisp.panel.info.onoff -text "ON" \
        -font $small_font -anchor w
    bind .glContDataDisp.panel.info.onoff <ButtonPress-1> \
            { beepOnce; post $DISP_CANVAS1_SIG 0 }
    label .glContDataDisp.panel.info.ms0 -text "m=0.001, s=0.05" \
        -font $small_font -foreground magenta
    label .glContDataDisp.panel.info.ms1 -text "m=0.002, s=0.06" \
        -font $small_font -foreground blue
    pack .glContDataDisp.panel.info.onoff -side left -fill x -expand yes
    pack .glContDataDisp.panel.info.ms1 -side right
    pack .glContDataDisp.panel.info.ms0 -side right
    pack .glContDataDisp.panel.info -side top -fill x -expand yes

    frame .glContDataDisp.panel.input
    frame .glContDataDisp.panel.input.r1
    label .glContDataDisp.panel.input.r1.l -text "Channel:" \
        -font $small_font -anchor w
    label .glContDataDisp.panel.input.r1.e -text "0" -anchor c \
          -font $small_font -width 10
    button .glContDataDisp.panel.input.r1.m -text "-" \
            -command { beepOnce; post $DISP_BTN7_SIG 1 } \
            -width 2 -height 1 \
            -background $color3 -activebackground $color3
    button .glContDataDisp.panel.input.r1.p -text "+" \
            -command { beepOnce; post $DISP_BTN7_SIG 2 } \
            -width 2 -height 1 \
            -background $color3 -activebackground $color3
    pack .glContDataDisp.panel.input.r1.l -side left -fill x -expand yes
    pack .glContDataDisp.panel.input.r1.p -side right
    pack .glContDataDisp.panel.input.r1.e -side right
    pack .glContDataDisp.panel.input.r1.m -side right
    pack .glContDataDisp.panel.input.r1 -side top -fill x -expand yes

    frame .glContDataDisp.panel.input.r2
    label .glContDataDisp.panel.input.r2.l -text "Time Window:" \
        -font $small_font -anchor w
    label .glContDataDisp.panel.input.r2.e -text "0" -anchor c \
          -font $small_font -width 10
    button .glContDataDisp.panel.input.r2.m -text "-" \
            -command { beepOnce; post $DISP_BTN7_SIG 3 } \
            -width 2 -height 1 \
            -background $color3 -activebackground $color3
    button .glContDataDisp.panel.input.r2.p -text "+" \
            -command { beepOnce; post $DISP_BTN7_SIG 4 } \
            -width 2 -height 1 \
            -background $color3 -activebackground $color3
    pack .glContDataDisp.panel.input.r2.l -side left -fill x -expand yes
    pack .glContDataDisp.panel.input.r2.p -side right
    pack .glContDataDisp.panel.input.r2.e -side right
    pack .glContDataDisp.panel.input.r2.m -side right
    pack .glContDataDisp.panel.input.r2 -side top -fill x -expand yes

    frame .glContDataDisp.panel.input.r3
    label .glContDataDisp.panel.input.r3.l -text "Data Window:" \
        -font $small_font -anchor w
    label .glContDataDisp.panel.input.r3.e -text "0" -anchor c \
          -font $small_font -width 10
    button .glContDataDisp.panel.input.r3.m -text "-" \
            -command { beepOnce; post $DISP_BTN7_SIG 5 } \
            -width 2 -height 1 \
            -background $color3 -activebackground $color3
    button .glContDataDisp.panel.input.r3.p -text "+" \
            -command { beepOnce; post $DISP_BTN7_SIG 6 } \
            -width 2 -height 1 \
            -background $color3 -activebackground $color3
    pack .glContDataDisp.panel.input.r3.l -side left -fill x -expand yes
    pack .glContDataDisp.panel.input.r3.p -side right
    pack .glContDataDisp.panel.input.r3.e -side right
    pack .glContDataDisp.panel.input.r3.m -side right
    pack .glContDataDisp.panel.input.r3 -side top -fill x -expand yes

    frame .glContDataDisp.panel.input.r4
    button .glContDataDisp.panel.input.r4.b1 -text "Freeze" \
            -command { beepOnce; post $DISP_BTN7_SIG 7 } \
            -width 7 -height 1 \
            -background $color3 -activebackground $color3
    button .glContDataDisp.panel.input.r4.b2 -text "Reset" \
            -command { beepOnce; post $DISP_BTN7_SIG 8 } \
            -width 7 -height 1 \
            -background $color3 -activebackground $color3
    button .glContDataDisp.panel.input.r4.b3 -text "Export" \
            -command { beepOnce; post $DISP_BTN7_SIG 9 } \
            -width 7 -height 1 \
            -background $color3 -activebackground $color3
    button .glContDataDisp.panel.input.r4.b4 -text "Hide" \
            -command { beepOnce; post $DISP_BTN7_SIG 10 } \
            -width 5 -height 1 \
            -background $color5 -activebackground $color5
    pack .glContDataDisp.panel.input.r4.b4 -side right
    pack .glContDataDisp.panel.input.r4.b3 -side right
    pack .glContDataDisp.panel.input.r4.b2 -side right
    pack .glContDataDisp.panel.input.r4.b1 -side right
    pack .glContDataDisp.panel.input.r4 -side top -fill x -expand yes

    pack .glContDataDisp.panel -side top -fill x

    proc userZoomPress {x y W } {
        $W cuserZoomPress $x $y
    }

    proc userZoomRelease {x y W} {
        $W cuserZoomRelease $x $y
    }

    proc actConfigureCDPanelInfo { onoff__ ms0__ ms1__ } {
        .glContDataDisp.panel.info.onoff configure -text $onoff__
        .glContDataDisp.panel.info.ms0 configure -text $ms0__
        .glContDataDisp.panel.info.ms1 configure -text $ms1__
    }

    proc actConfigureCDPanelInput { visible__ chan__ twin__ dwin__ \
                                   freeze__  save__ export__ exit__ } {
        .glContDataDisp.panel.input.r1.e configure -text $chan__
        .glContDataDisp.panel.input.r2.e configure -text $twin__
        .glContDataDisp.panel.input.r3.e configure -text $dwin__
        .glContDataDisp.panel.input.r4.b1 configure -text $freeze__
        .glContDataDisp.panel.input.r4.b2 configure -text $save__
        .glContDataDisp.panel.input.r4.b3 configure -text $export__
        .glContDataDisp.panel.input.r4.b4 configure -text $exit__
        if { $visible__ == 1 } {
            pack .glContDataDisp.panel.input -side top -fill x -expand yes
        } else {
            pack forget .glContDataDisp.panel.input
        }
    }
}

#=============================================================================
# .wizardNavigation
#
# The implement steering wizard display.
#
#=============================================================================
namespace eval wizardNavigation {

    frame .wizardNavigation 

    # Input frame 
    frame .wizardNavigation.left 
    frame .wizardNavigation.left.ex
    frame .wizardNavigation.left.sep -width 2 -height 68 -background "black"
    frame .wizardNavigation.left.spacer -width 50 -height 50
    frame .wizardNavigation.left.right -width 60 
    pack .wizardNavigation.left.ex -side left
    pack .wizardNavigation.left.sep -side left
    pack .wizardNavigation.left.spacer -side left
    pack .wizardNavigation.left.right -side left
    frame .wizardNavigation.right
    frame .wizardNavigation.right.nav
    frame .wizardNavigation.right.sep -width 2 -height 68 -background $colorInx
    frame .wizardNavigation.right.spacer -width 20 -height 50
    frame .wizardNavigation.right.left -width 300
    frame .wizardNavigation.right.left.spacer -width 73
    pack .wizardNavigation.right.nav -side right
    pack .wizardNavigation.right.sep -side right
    pack .wizardNavigation.right.spacer -side right
    pack .wizardNavigation.right.left -side right -expand yes -fill both

    # Exit Wizard button
    SnazzyButton::create .wizardNavigation.left.ex.exit btnExitKhakiSquareDown btnExitKhakiSquareDown 0.2 \
                         "beepOnce; post $DISP_WIZARD_EXIT_SIG 47"

    SnazzyButton::create .wizardNavigation.left.ex.exitHelp btnExitKhakiSquareDown btnExitKhakiSquareDown 0.2 \
                         "beepOnce; post $DISP_WIZARD_HELPRET_SIG 47"

    #Continue button
    SnazzyButton::create .wizardNavigation.right.nav.b0 btnExitKhakiSquareDown btnExitKhakiSquareDown 0.2 \
                         "beepOnce; post $DISP_WIZARD_CONTINUE_SIG 44"
    #BACK button
    SnazzyButton::create .wizardNavigation.right.nav.b1 btnExitKhakiSquareDown btnExitKhakiSquareDown 0.25 \
                         "beepOnce; post $DISP_WIZARD_BACK_SIG 46"
    #Action Buttons
    SnazzyButton::create .wizardNavigation.right.left.b2 btnBlueSquareUp btnBlueSqaureDown 0.25 \
                         "beepOnce; post $DISP_WIZARD_SELECT_SIG 45" 

    SnazzyButton::create .wizardNavigation.right.left.b3 btnBlueSquareUp btnBlueSqaureDown 0.25 \
                         "beepOnce; post $DISP_WIZARD_SELECT_SIG 1" 

    SnazzyButton::create .wizardNavigation.right.left.b4 btnBlueSquareUp btnBlueSqaureDown 0.25 \
                         "beepOnce; post $DISP_WIZARD_SELECT_SIG 2" 

    SnazzyButton::create .wizardNavigation.right.left.b5 btnBlueSquareUp btnBlueSqaureDown 0.25 \
                         "beepOnce; post $DISP_WIZARD_SELECT_SIG 3" 

    # top line
    frame .wizardNavigation.sep -height 2 -width $winWidth -borderwidth 0 \
           -background $colorInx
    pack .wizardNavigation.sep -side top 

    pack .wizardNavigation.left.ex.exit -side left -padx 2 -pady 0  -anchor sw
    pack .wizardNavigation.left -side left -anchor sw

    pack .wizardNavigation.right -side bottom -anchor se -expand yes -fill x
    pack .wizardNavigation.right.nav.b0 -side right -padx 2 -pady 0
    pack .wizardNavigation.right.nav.b1 -side right -padx 2 -pady 0

    SnazzyButton::setText .wizardNavigation.left.ex.exit "Return" 0.5
    SnazzyButton::setText .wizardNavigation.right.nav.b0 "Next" 0.5
    SnazzyButton::setText .wizardNavigation.right.nav.b1 "Back" 0.5  

    proc actSetActionButtonImage { btn img } {
	SnazzyButton::setImage .wizardNavigation.right.left.b$btn $img $img 0.5
    }

    proc actSetExitBtn { label } {
	SnazzyButton::setText .wizardNavigation.left.ex.exit $label 0.5
    }

    proc actSetHelp { label } {
	SnazzyButton::setText .wizardNavigation.left.ex.exitHelp $label 0.5
	pack forget .wizardNavigation.left.ex.exit
	pack .wizardNavigation.left.ex.exitHelp
    }

    proc actClearHelp { label } {
	SnazzyButton::setText .wizardNavigation.left.ex.exit $label 0.5
	pack forget .wizardNavigation.left.ex.exitHelp
	pack .wizardNavigation.left.ex.exit
    }

    proc actPackActionButtons {btn0 btn1 btn2 btn3 btn4 btn5 } {
	pack forget .wizardNavigation.right.nav.b0
	pack forget .wizardNavigation.right.nav.b1
	pack forget .wizardNavigation.right.left.b2
	pack forget .wizardNavigation.right.left.b3
	pack forget .wizardNavigation.right.left.b4
	pack forget .wizardNavigation.right.left.b5

	pack .wizardNavigation.right.left.spacer -side left
	if {$btn0 != "!"} {
	    SnazzyButton::setText .wizardNavigation.right.nav.b0 $btn0 0.5
	    pack .wizardNavigation.right.nav.b0 -side right -padx 2 -pady 0
	}
	if {$btn1 != "!"} {
	    SnazzyButton::setText .wizardNavigation.right.nav.b1 $btn1 0.5
	    pack .wizardNavigation.right.nav.b1 -side right -padx 2 -pady 0
	}
	if {$btn2 != "!"} {
	    SnazzyButton::setText .wizardNavigation.right.left.b2 $btn2 0.5
	    pack .wizardNavigation.right.left.b2 -side left -padx 2 -pady 0
	}
	if {$btn3 != "!"} {
	    #pack forget  .wizardNavigation.right.left.spacer
	    SnazzyButton::setText .wizardNavigation.right.left.b3 $btn3 0.5
	    pack .wizardNavigation.right.left.b3 -side left -padx 2 -pady 0
	}
	if {$btn4 != "!"} {
	    pack forget  .wizardNavigation.right.left.spacer
	    SnazzyButton::setText .wizardNavigation.right.left.b4 $btn4 0.5
	    pack .wizardNavigation.right.left.b4 -side left -padx 2 -pady 0
	}
	if {$btn5 != "!"} {
	    pack forget  .wizardNavigation.right.left.spacer
	    SnazzyButton::setText .wizardNavigation.right.left.b5 $btn5 0.5
	    pack .wizardNavigation.right.left.b5 -side left -padx 2 -pady 0
	}
    }

    proc actListBox { visible__  } {

       if { $visible__ == 1 } {
       }  elseif { $visible__ == 0 } {
           pack forget .wizardNavigation.r1.b2
       }

    }

    proc actMsgBox { visible__  } {

       if { $visible__ == 1 } {

           pack forget .wizardNavigation.r1.b1
	   pack .wizardNavigation.right.nav.b0 -side right -padx 2 -pady 0
	   pack .wizardNavigation.right.nav.b1 -side right -padx 2 -pady 0
           SnazzyButton::setText .wizardNavigation.right.nav.b1 "Back" 0.5 

       }  elseif { $visible__ == 0 } {

           pack forget .wizardNavigation.r1.b0
           # rename the BACK button OK and shift to the right
	   pack .wizardNavigation.right.nav.b1 -side right -padx 2 -pady 0
           SnazzyButton::setText .wizardNavigation.right.nav.b1 "OK" 0.5  
       }

    }

   proc actHelpButtonsByState { visibleState__ } {

   }
}

#=============================================================================
# .wizardHelpNavigation
#
# The implement steering wizard help display.
#
#=============================================================================
namespace eval wizardHelpNavigation {

    frame .wizardHelpNavigation 

    # Return from Help to Wizard button
    SnazzyButton::create .wizardHelpNavigation.ret btnExitKhakiSquareDown btnExitKhakiSquareDown 0.2 \
                         "beepOnce; post $DISP_WIZARD_HELPRET_SIG 49"
    # Input frame 
    frame .wizardHelpNavigation.r1

    #More button
    SnazzyButton::create .wizardHelpNavigation.r1.b0 btnExitKhakiSquareDown btnExitKhakiSquareDown 0.2 \
                         "beepOnce; post $DISP_WIZARD_HELPMORE_SIG 50"
    #BACK button
    SnazzyButton::create .wizardHelpNavigation.r1.b1 btnBlueSquareUp btnBlueSquareDown 0.25 \
                         "beepOnce; post $DISP_WIZARD_HELPBACK_SIG 51"

    # top line
    frame .wizardHelpNavigation.sep -height 2 -width $winWidth -borderwidth 0 \
           -background $colorInx
    pack .wizardHelpNavigation.sep -side top 

    pack .wizardHelpNavigation.ret -side left -padx 5 -pady 5  -anchor sw

    pack .wizardHelpNavigation.r1 -side bottom -anchor se
    pack .wizardHelpNavigation.r1.b1 -side left -padx 5 -pady 5
    pack .wizardHelpNavigation.r1.b0 -side right -padx 5 -pady 5

    SnazzyButton::setText .wizardHelpNavigation.ret "Return" 0.5
    SnazzyButton::setText .wizardHelpNavigation.r1.b0 "More" 0.5
    SnazzyButton::setText .wizardHelpNavigation.r1.b1 "Back" 0.5  

    proc actVisibleButtons { visibleMore__ visibleBack__ } {

       if { $visibleBack__ == 1 } {
	    pack .wizardHelpNavigation.r1.b1 -side right -padx 5 -pady 5
            SnazzyButton::setText .wizardHelpNavigation.r1.b1 "Back" 0.5  
       }  elseif { $visibleBack__ == 0 } {
           pack forget .wizardHelpNavigation.r1.b1
       }
      if { $visibleMore__ == 1 } {
	    pack .wizardHelpNavigation.r1.b0 -side right -padx 5 -pady 5
            SnazzyButton::setText .wizardHelpNavigation.r1.b0 "More" 0.5  
       }  elseif { $visibleMore__ == 0 } {
           pack forget .wizardHelpNavigation.r1.b0
       }
    }

    proc actVisibleButtonsByState { visibleState__ } {

       if { $visibleState__ == 1 } {
	   # ONLY 
           pack forget .wizardHelpNavigation.r1.b1
           pack forget .wizardHelpNavigation.r1.b1
	   pack .wizardHelpNavigation.ret -side left -padx 5 -pady 5  -anchor sw
       } 
      if { $visibleState__ == 2 } {
	  # START
            pack forget .wizardHelpNavigation.r1.b1
	    pack .wizardHelpNavigation.r1.b0 -side right -padx 5 -pady 5
	    pack .wizardHelpNavigation.ret -side left -padx 5 -pady 5  -anchor sw
       }
     if { $visibleState__ == 4 } {
          # END
	    pack forget .wizardHelpNavigation.r1.b0
	    pack .wizardHelpNavigation.r1.b1 -side left -padx 5 -pady 5
	    pack .wizardHelpNavigation.ret -side left -padx 5 -pady 5  -anchor sw
     }
   }
}
