##############################################################################
#
# vc/src/tcl/movingMapConfig.tcl
#
# Screen to display moving map configuration options.  These configuration 
# options are divided into two aspects of the moving map screen, coverage and
# guidence.  The options provided for the coverage maps are display and
# record, while the options for guidence are only record.
#
# 20060119 RCM - created
#
# Copyright (c) 2006 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop
     

    # Define and pack the screen layout
    
    # Main title text frame and label
    #-------------------------------------------------------------------------
    frame .ftop.title
    label .ftop.title.text -text "Space for title" -font $huge_font
    pack  .ftop.title.text -side top -fill x -expand yes
    pack  .ftop.title -side top -fill x -expand yes -anchor n -pady 3
    frame .ftop.subTitle
    label .ftop.subTitle.text -text "Space for sub title" -font $big_font
    pack  .ftop.subTitle.text -side top -fill x -expand yes
    pack  .ftop.subTitle -side top -fill x -expand yes -anchor n
    frame .ftop.helpText
    label .ftop.helpText.text -text "Space for help text" -font $small_font
    pack  .ftop.helpText -side top -fill x -expand yes -anchor n -pady 5     
    
    # List box to contain custom text
    frame     .ftop.main
    frame     .ftop.main.text
    frame     .ftop.main.scroll
    scrollbar .ftop.main.scroll.ysbar -command \
                { .ftop.main.text.customText yview } -width 30
    scrollbar .ftop.main.xsbar -command \
                { .ftop.main.text.customText xview } -width 30 \
                -orient horizontal
    listbox   .ftop.main.text.customText \
                -yscrollcommand {.ftop.main.scroll.ysbar set} \
                -xscrollcommand {.ftop.main.xsbar set} \
                -width 100 -height 5 -font $huge_font
    
    pack      .ftop.main.text.customText -side top -expand yes -fill both
    pack      .ftop.main.scroll.ysbar -side right -fill y -expand yes \
                                      -anchor ne
    pack      .ftop.main.xsbar -side bottom -fill x -expand yes -anchor sw
    pack      .ftop.main.scroll -side right -fill y -expand yes -anchor ne
    pack      .ftop.main.text -side left -fill both -expand yes -anchor n
     
    
    pack      .ftop.main -side top -anchor n -expand yes -fill both
    
            
    # Exit, Edit and Delete buttons
    #-------------------------------------------------------------------------
    frame .ftop.buttons
    SnazzyButton::create .ftop.buttons.edit btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN1_SIG 1" "Edit"
    SnazzyButton::create .ftop.buttons.delete btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN2_SIG 2" "Delete"
    SnazzyButton::create .ftop.buttons.exit btnKhakiU btnKhakiD 0.5 \
                 "beepOnce; post $DISP_BTN9_SIG 9" "Exit"
    label .ftop.buttons.space -width 0
    
    pack .ftop.buttons.exit -side left -anchor sw -padx 5
    pack .ftop.buttons.space -side right -anchor se
    pack .ftop.buttons.edit -side right -anchor se
    pack .ftop.buttons.delete -side right -anchor se        
    pack .ftop.buttons -side bottom -anchor s -pady 7 -fill x -expand yes
            
    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry {title__ subTitle__ editBtn__ deleteBtn__ exitBtn__} {
      
      global small_font
      .ftop.title.text configure -text $title__
      .ftop.subTitle.text configure -text $subTitle__ 
      .ftop.buttons.exit configure -text $exitBtn__
      .ftop.buttons.edit configure -text $editBtn__
      .ftop.buttons.delete configure -text $deleteBtn__ 
      
      # Create frames to hold individual custom lines of data
      
        
      pack .ftop.main -side top -fill both -expand yes -padx 10
      pack .ftop -fill both -expand yes    
    }

        
    
       
    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------
  
    proc actListboxInsert { item__ } {
      .ftop.main.text.customText insert end $item__
    }
            
    proc actListboxGetCurindex {} {
      if { [.ftop.main.text.customText index end] != 0} {
        return [lindex [.ftop.main.text.customText curselection] 0]
      }
      else {
        return -1             ;# -1 tells the C side the listbox was empty
      }
    }

    proc actListboxInsertIndex { item__ index__} {
      .ftop.main.text.customText insert $item__ $index__
    }
    
    proc actListboxDeleteAll { } {
        .ftop.main.text.customText delete 0 end
    }

    proc actListboxSelectionSet { index__ } {
        .ftop.main.text.customText selection set $index__ $index__
        .ftop.main.text.customText see $index__ 
    }
    
    proc actListboxGetCurindex {} {
      if { [.ftop.main.text.customText index end] != 0} {
        return [lindex [.ftop.main.text.customText curselection] 0]
      } 
      else {
        return -1             ;# -1 tells the C side the listbox was empty
      }
    }
    
    proc actListboxGetCurselection {} {
        if { [.ftop.main.text.customText index end] != 0} {
            set index [lindex [.ftop.main.text.customText curselection] 0]
            return [.ftop.main.text.customText get $index]
        } else {
            return "!"             ;# ! tells the C side the listbox was empty
        }
    } 

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
           
  }