##############################################################################
#
# vc/src/tcl/examinePos.tcl
#
# Screen to display details about Position examination.
#
# 20030421 M_S - created
#
# Copyright (c) 2002-2003 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # NOTE: using place for positioning imported frames (-in option)
    #       craches Tcl interpreter. Therefore pack must be used.
    pack .configDisplay -in .ftop -side left \
            -anchor n -padx 10 -pady 10
    raise .configDisplay
    pack .navDisplay -in .ftop -side right -anchor n -padx 10 -pady 10
    raise .navDisplay


    # title and error indicator
    label .ftop.title -font $huge_font
    place .ftop.title -relx 0.5 -rely 0.3 -anchor n
    
    # labels
    label .ftop.northLbl -font $big_font
    place .ftop.northLbl -relx 0.25 -x 60 -rely 0.45 -anchor c
    label .ftop.eastLbl -font $big_font
    place .ftop.eastLbl -relx 0.25 -x 180 -rely 0.45 -anchor c
    label .ftop.downLbl -font $big_font
    place .ftop.downLbl -relx 0.25 -x 300 -rely 0.45 -anchor c
    label .ftop.posLbl -font $big_font
    place .ftop.posLbl -relx 0.25 -rely 0.5 -anchor ne
    label .ftop.rawPosLbl -font $big_font
    place .ftop.rawPosLbl -relx 0.25 -rely 0.5 -y 40 -anchor ne
       
    label .ftop.posNN -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.posNN -relx 0.25 -x  10 -rely 0.5 -anchor nw
    label .ftop.posEE -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.posEE -relx 0.25 -x 130 -rely 0.5 -anchor nw
    label .ftop.posDD -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.posDD -relx 0.25 -x 250 -rely 0.5 -anchor nw
    label .ftop.rawNN -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.rawNN -relx 0.25 -x  10 -rely 0.5 -y 40 -anchor nw
    label .ftop.rawEE -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.rawEE -relx 0.25 -x 130 -rely 0.5 -y 40 -anchor nw
    label .ftop.rawDD -font $big_font -width 10 -relief sunken -borderwidth 2 
    place .ftop.rawDD -relx 0.25 -x 250 -rely 0.5 -y 40 -anchor nw
   
    label .ftop.speedLbl -font $big_font
    place .ftop.speedLbl -relx 0.25 -x 110 -rely 0.7 -anchor ne
    label .ftop.speed -font $big_font -width 23 -relief sunken -borderwidth 2 
    place .ftop.speed -relx 0.25 -x 130 -rely 0.7 -anchor nw
    
    # buttons
    SnazzyButton::create .ftop.return btnKhakiU btnKhakiD 0.5 \
            "beepOnce; post $DISP_BTN9_SIG 9" "Return to\nPrevious\nMenu"
    place .ftop.return -x 2 -rely 1.0 -y -2 -anchor sw 
    SnazzyButton::create .ftop.reset btnBlueU btnBlueD 0.5 \
            "beepOnce; post $DISP_BTN1_SIG 1" "Reset"           
    place .ftop.reset -relx 1.0 -x -2 -rely 1.0 -y -2 -anchor se            

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    
    proc actEntry { titleLbl returnBtn resetPosBtn \
                    posLbl rawPosLbl speedLbl \
                    northLbl eastLbl downLbl } {
   
        global huge_font big_font
   
        .ftop.title configure -text $titleLbl
        .ftop.return configure -text $returnBtn
        .ftop.reset configure -text $resetPosBtn
        
        .ftop.northLbl configure -text $northLbl
        .ftop.eastLbl configure -text $eastLbl
        .ftop.downLbl configure -text $downLbl
        .ftop.posLbl configure -text $posLbl
        .ftop.rawPosLbl configure -text $rawPosLbl
        .ftop.speedLbl configure -text $speedLbl
        pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # other action procedures (called by GuiMgr)
    #-------------------------------------------------------------------------

    proc actDisplayPos { posNN posEE posDD rawNN rawEE rawDD speed } {
        .ftop.posNN configure -text $posNN
        .ftop.posEE configure -text $posEE
        .ftop.posDD configure -text $posDD
        .ftop.rawNN configure -text $rawNN
        .ftop.rawEE configure -text $rawEE
        .ftop.rawDD configure -text $rawDD
        .ftop.speed configure -text $speed
    }
}
