##############################################################################
#
# vc/src/tcl/examineSteeringDisplay.tcl
#
# Screen to display details about Steering examination.
#
# 20021112 QC - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout
    # (Screen has custom display and input frames, and reuses the .logo,
    # .configDisplay, and .navDisplay components)
    frame .ftop.title
    frame .ftop.input
    pack .ftop.input -side bottom -fill both -expand yes \
            -padx 5 -pady 2
    pack .ftop.title -side bottom -fill both -expand yes
    pack .configDisplay -in .ftop -side left \
            -anchor nw -fill y -expand yes -padx 10 -pady 2
    raise .configDisplay
    pack .navDisplay -in .ftop -side right -anchor ne -padx 10 -pady 2
    raise .navDisplay

    # Title
    label .ftop.title.prompt -text "Space for title"
    pack .ftop.title.prompt -side top

    # Row 1
    frame .ftop.input.r1
    label .ftop.input.r1.l0 -text "Wheel angle:" \
          -font $big_font -anchor e
    label .ftop.input.r1.e0 -text "3.20 deg" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.e1 -text "1962" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.l1 -text "Pots:" \
          -font $big_font -anchor e
    label .ftop.input.r1.e2 -text "32521" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.e3 -text "32536" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.e4 -text "32313" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1.l2 -text "Disengage switch:" \
          -font $big_font
    label .ftop.input.r1.e5 -text "OFF" -font $big_font \
          -width 8 -relief sunken -borderwidth 2        
    label .ftop.input.r1.l -text "" -font $small_font
       
    # Row 1a
    frame .ftop.input.r1a
    label .ftop.input.r1a.l1 -text "Spans:" \
          -font $big_font -anchor e
    label .ftop.input.r1a.e2 -text "32521" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1a.e3 -text "32536" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r1a.e4 -text "32313" -font $big_font \
          -width 8 -relief sunken -borderwidth 2
       
    # Row 1b Autotrac: Displays killswitch and valve fault
    frame .ftop.input.r1b
    label .ftop.input.r1b.l1 -text "Disengage switch:" \
          -font $big_font
    label .ftop.input.r1b.e1 -text "OFF" -font $big_font \
          -width 8 -relief sunken -borderwidth 2        
    label .ftop.input.r1b.l2 -text "Valve Fault:" \
          -font $big_font
    label .ftop.input.r1b.e2 -text "OFF" -font $big_font \
          -width 8 -relief sunken -borderwidth 2        

    # Row 2
    frame .ftop.input.r2
    label .ftop.input.r2.l1 -text "Left transducer:" -font $big_font -anchor w
    label .ftop.input.r2.l2 -text "Right transducer:" -font $big_font
    label .ftop.input.r2.l3 -text "Joystick:" -font $big_font
    label .ftop.input.r2.e0 -text "11" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r2.e1 -text "12" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r2.e2 -text "13" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    
    # Row 2a
    frame .ftop.input.r2a
    label .ftop.input.r2a.l1 -text "Seat Switch" -font $big_font -anchor w
    label .ftop.input.r2a.l2 -text "Max" -font $big_font
    label .ftop.input.r2a.e0 -text "" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    label .ftop.input.r2a.e1 -text "" -font $big_font \
                  -width 8 -relief sunken -borderwidth 2
    
    # Pulse Config
    frame .ftop.input.pulseconf 
    radiobutton .ftop.input.pulseconf.pulse -height 2 -width 20 -variable pulsesetting \
        -command {beepOnce; post $DISP_BTN7_SIG 0} -activebackground $color6 \
        -text "Pulse" -font $big_font -value 1 -anchor sw
    radiobutton .ftop.input.pulseconf.cont -height 2 -width 20 -variable pulsesetting \
        -command {beepOnce; post $DISP_BTN7_SIG 1} -activebackground $color6 \
        -text "Continuous" -font $big_font -value 0 -anchor sw
    pack .ftop.input.pulseconf.pulse -side left -padx 25
    pack .ftop.input.pulseconf.cont -side left -padx 25

    # Row 3
    frame .ftop.input.r3
    SnazzyButton::create .ftop.input.r3.cancel btnKhakiU btnKhakiD 0.5 \
            "beepOnce; post $DISP_BTN9_SIG 9" "Return to\nPrevious\nMenu"
    SnazzyButton::create .ftop.input.r3.hardleft btnBlue8TU btnBlue8TD 0.5 \
            "beepOnce; post $DISP_BTN1_SIG 1" "100%\nLeft"
    SnazzyButton::create .ftop.input.r3.left btnBlue8TU btnBlue8TD 0.5 \
            "beepOnce; post $DISP_BTN2_SIG 2" "50%\nLeft"
    SnazzyButton::create .ftop.input.r3.zero btnBlue8TU btnBlue8TD 0.5 \
            "beepOnce; post $DISP_BTN3_SIG 3" "Zero"
    SnazzyButton::create .ftop.input.r3.right btnBlue8TU btnBlue8TD 0.5 \
            "beepOnce; post $DISP_BTN4_SIG 4" "50%Right"
    SnazzyButton::create .ftop.input.r3.hardright btnBlue8TU btnBlue8TD 0.5 \
            "beepOnce; post $DISP_BTN5_SIG 5" "100%\nRight"
    SnazzyButton::create .ftop.input.r3.change btnYellowU btnYellowD 0.5 \
            "beepOnce; post $DISP_BTN6_SIG 6" "Change\nPercent"
    SnazzyButton::create .ftop.input.r3.stop btnBigStopU btnBigStopU 0.5 \
            "beepOnce; post $DISP_BTN0_SIG 0" ""

    # Row 3
    frame .ftop.input.r3a
    label .ftop.input.r3a.l1 -text "Your speed is too fast" -font $huge_font -anchor w
    SnazzyButton::create .ftop.input.r3a.cancel btnKhakiU btnKhakiD 0.5 \
            "beepOnce; post $DISP_BTN9_SIG 9" "Return to\nPrevious\nMenu"
    pack .ftop.input.r3a.cancel -side left -padx 3 -pady 1
    pack .ftop.input.r3a.l1 -side left -padx 3 -pady 1

    pack .ftop.input.r3.cancel -side left -padx 3 -pady 1
    pack .ftop.input.r3.hardleft -side left -padx 1
    pack .ftop.input.r3.left -side left -padx 1
    pack .ftop.input.r3.zero -side left -padx 1
    pack .ftop.input.r3.right -side left -padx 1
    pack .ftop.input.r3.hardright -side left -padx 1
    pack .ftop.input.r3.change -side right -padx 3 -pady 1

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { packProc__  } {
	.ftop.input.pulseconf.pulse select
        eval $packProc__
        pack .ftop -fill both -expand yes

    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actSetPulseValue { pulseVal } {
    }

    proc actDisplayDeereTrack {pot1 pot2 pot3 killswitch} {
        .ftop.input.r1.e2 configure -text $pot1
        .ftop.input.r1.e3 configure -text $pot2
        .ftop.input.r1.e4 configure -text $pot3
        .ftop.input.r1.e5 configure -text $killswitch
    }

    proc actDisplayWheeled {wheelangle rawheelangle \
            killswitch leftrans rightrans} {
        .ftop.input.r1.e0 configure -text $wheelangle
        .ftop.input.r1.e1 configure -text $rawheelangle
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r2.e0 configure -text $leftrans
        .ftop.input.r2.e1 configure -text $rightrans
    }

    proc actDisplayAutotrac {wheelangle rawheelangle \
            killswitch valveFault steerMovement signalLoad} {
        .ftop.input.r1.e0 configure -text $wheelangle
        .ftop.input.r1.e1 configure -text $rawheelangle
        .ftop.input.r1b.e1 configure -text $killswitch
        .ftop.input.r1b.e2 configure -text $valveFault
        .ftop.input.r2.e0 configure -text $steerMovement
        .ftop.input.r2.e1 configure -text $signalLoad
    }
    
    proc actDisplayWPE2 {wheelangle rawheelangle \
            killswitch steerMovement } {
        .ftop.input.r1.e0 configure -text $wheelangle
        .ftop.input.r1.e1 configure -text $rawheelangle
        .ftop.input.r1b.e1 configure -text $killswitch
        .ftop.input.r2.e0 configure -text $steerMovement
    }
    
    proc actDisplayXerion {wheelangle rawheelangle killswitch \
                           autosteerActive autosteerEnabled} {
        .ftop.input.r1.e0 configure -text $wheelangle
        .ftop.input.r1.e1 configure -text $rawheelangle
        .ftop.input.r1b.e1 configure -text $killswitch
        .ftop.input.r1b.e2 configure -text $autosteerActive
        .ftop.input.r2.e0 configure -text $autosteerEnabled
    }

    proc actDisplayCatTrack {killswitch leftrans rightrans} {
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r2.e0 configure -text $leftrans
        .ftop.input.r2.e1 configure -text $rightrans
    }

    proc actDisplayMT {killswitch msg} {
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r1.l configure -text $msg
    }

    proc actDisplayGantry {wheelangle rawheelangle \
            killswitch leftpot rightpot joystick seatSw seatSwMax} {
        .ftop.input.r1.e0 configure -text $wheelangle
        .ftop.input.r1.e1 configure -text $rawheelangle
        .ftop.input.r1.e5 configure -text $killswitch
        .ftop.input.r2.e0 configure -text $leftpot
        .ftop.input.r2.e1 configure -text $rightpot
        .ftop.input.r2.e2 configure -text $joystick
        .ftop.input.r2a.e0 configure -text $seatSw
        .ftop.input.r2a.e1 configure -text $seatSwMax
    }

    proc actDisplayCamecoHarv {sig0 sig1 sig2 span0 span1 span2 killswitch} {
        .ftop.input.r1.e2 configure -text $sig0
        .ftop.input.r1.e3 configure -text $sig1
        .ftop.input.r1.e4 configure -text $sig2
        .ftop.input.r1a.e2 configure -text $span0
        .ftop.input.r1a.e3 configure -text $span1
        .ftop.input.r1a.e4 configure -text $span2
        .ftop.input.r1.e5 configure -text $killswitch
    }

    proc actDisplayPercent { leftPercentBtn rightPercentBtn } {
        .ftop.input.r3.left configure -text $leftPercentBtn
        .ftop.input.r3.right configure -text $rightPercentBtn
    }


    proc actConfigureHighSpeed { warning } {
        pack forget .ftop.input.r3
        pack forget .ftop.input.pulseconf
        .ftop.input.r3a.l1 configure -text $warning
        pack .ftop.input.r3a -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc actConfigureLowSpeed { } {
        pack forget .ftop.input.r3a
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc actConfigureAutosteer { } {
        pack forget .ftop.input.r3.change
        pack .ftop.input.r3.stop -after .ftop.input.r3.hardright \
                                 -padx 5 -pady 1 -anchor s -fill both -expand yes
    }

    proc actConfigureStopAutosteer { } {
        global color3

        pack forget .ftop.input.r3.stop
        pack .ftop.input.r3.change -after .ftop.input.r3.hardright \
                                   -padx 5 -pady 1 -anchor s -fill both -expand yes
        
        .ftop.input.r3.hardright configure -image btnBlue8TU
        .ftop.input.r3.left configure      -image btnBlue8TU
        .ftop.input.r3.zero configure      -image btnBlue8TU
        .ftop.input.r3.right configure     -image btnBlue8TU
        .ftop.input.r3.hardleft configure  -image btnBlue8TU
        
    }

    proc actConfigureHardLeft { } {
    
        .ftop.input.r3.hardright configure -image btnBlue8TU
        .ftop.input.r3.left configure      -image btnBlue8TU
        .ftop.input.r3.zero configure      -image btnBlue8TU
        .ftop.input.r3.right configure     -image btnBlue8TU
        .ftop.input.r3.hardleft configure  -image btnGreen8TU
    }
 
    proc actConfigureLeft { } {
    
        .ftop.input.r3.hardright configure -image btnBlue8TU
        .ftop.input.r3.left configure      -image btnGreen8TU
        .ftop.input.r3.zero configure      -image btnBlue8TU
        .ftop.input.r3.right configure     -image btnBlue8TU
        .ftop.input.r3.hardleft configure  -image btnBlue8TU
    }

    proc actConfigureZero { } {
    
        .ftop.input.r3.hardright configure -image btnBlue8TU
        .ftop.input.r3.left configure      -image btnBlue8TU
        .ftop.input.r3.zero configure      -image btnGreen8TU
        .ftop.input.r3.right configure     -image btnBlue8TU
        .ftop.input.r3.hardleft configure  -image btnBlue8TU
    }

    proc actConfigureRight { } {
      
        .ftop.input.r3.hardright configure -image btnBlue8TU
        .ftop.input.r3.left configure      -image btnBlue8TU
        .ftop.input.r3.zero configure      -image btnBlue8TU
        .ftop.input.r3.right configure     -image btnGreen8TU
        .ftop.input.r3.hardleft configure  -image btnBlue8TU
    }

    proc actConfigureHardRight { } {
    
        .ftop.input.r3.hardright configure -image btnGreen8TU
        .ftop.input.r3.left configure      -image btnBlue8TU
        .ftop.input.r3.zero configure      -image btnBlue8TU
        .ftop.input.r3.right configure     -image btnBlue8TU
        .ftop.input.r3.hardleft configure  -image btnBlue8TU
    }

    proc actConfigurePulse { pulseRight pulseLeft } {
	.ftop.input.r3.hardleft configure -state disabled
	.ftop.input.r3.hardright configure -state disabled
	.ftop.input.r3.left configure -text $pulseLeft
	.ftop.input.r3.right configure -text $pulseRight
    }

    proc actConfigureContinuous { right left } {
	.ftop.input.r3.hardleft configure -state normal
	.ftop.input.r3.hardright configure -state normal
	.ftop.input.r3.left configure -text $left
	.ftop.input.r3.right configure -text $right
    }

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
    proc packDeereTrack { title potsLbl killswitchLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l1 configure -text $potsLbl
        pack .ftop.input.r1.l1 -side left
        pack .ftop.input.r1.e2 -side left -padx 10
        pack .ftop.input.r1.e3 -side left
        pack .ftop.input.r1.e4 -side left -padx 10
        pack .ftop.input.r1.l2 -side left -padx 1
        pack .ftop.input.r1.e5 -side left

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc packWheeled { title wheelLbl killswitchLbl leftLbl rightLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left
        pack .ftop.input.r1.e1 -side left
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left

        # Row 2
        .ftop.input.r2.l1 configure -text $leftLbl
        .ftop.input.r2.l2 configure -text $rightLbl
        pack .ftop.input.r2.l1 -side left
        pack .ftop.input.r2.e0 -side left -padx 10
        pack .ftop.input.r2.l2 -side left
        pack .ftop.input.r2.e1 -side left -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc packAutotrac { title wheelLbl killswitchLbl valveFaultLbl \
            steerLbl signalLoadLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {
            
        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left
        pack .ftop.input.r1.e1 -side left

        # Row 1b
        .ftop.input.r1b.l1 configure -text $killswitchLbl
        .ftop.input.r1b.l2 configure -text $valveFaultLbl
        pack .ftop.input.r1b.l1 -side left -padx 10
        pack .ftop.input.r1b.e1 -side left -padx 10
        pack .ftop.input.r1b.l2 -side left -padx 10
        pack .ftop.input.r1b.e2 -side left -padx 10

        # Row 2
        .ftop.input.r2.l1 configure -text $steerLbl
        .ftop.input.r2.l2 configure -text $signalLoadLbl
        pack .ftop.input.r2.l1 -side left -padx 10
        pack .ftop.input.r2.e0 -side left  -padx 10
        pack .ftop.input.r2.l2 -side left  -padx 10
        pack .ftop.input.r2.e1 -side left  -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 1 -anchor w
        pack .ftop.input.r1b -side top -padx 60 -pady 1 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 1 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 0 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 0 -expand yes -fill both
    }

    proc packSDFG { title wheelLbl killswitchLbl valveFaultLbl \
            steerLbl signalLoadLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {
            
        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left
        pack .ftop.input.r1.e1 -side left

        # Row 1b
        .ftop.input.r1b.l1 configure -text $killswitchLbl
        .ftop.input.r1b.l2 configure -text $valveFaultLbl
        pack .ftop.input.r1b.l1 -side left -padx 10
        pack .ftop.input.r1b.e1 -side left -padx 10
        pack .ftop.input.r1b.l2 -side left -padx 10
        pack .ftop.input.r1b.e2 -side left -padx 10

        # Row 2
        .ftop.input.r2.l1 configure -text $steerLbl
        .ftop.input.r2.l2 configure -text $signalLoadLbl
        pack .ftop.input.r2.l1 -side left -padx 10
        pack .ftop.input.r2.e0 -side left  -padx 10
        pack .ftop.input.r2.l2 -side left  -padx 10
        pack .ftop.input.r2.e1 -side left  -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 1 -anchor w
        pack .ftop.input.r1b -side top -padx 60 -pady 1 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 1 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 0 -anchor w
        pack .ftop.input.r3 -side top -padx 5 -pady 0 -expand yes -fill both
    }
    
    proc packWPE2 { title wheelLbl killswitchLbl \
            steerLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {
            
        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left
        pack .ftop.input.r1.e1 -side left

        # Row 1b
        .ftop.input.r1b.l1 configure -text $killswitchLbl
        pack .ftop.input.r1b.l1 -side left -padx 10
        pack .ftop.input.r1b.e1 -side left -padx 10

        # Row 2
        .ftop.input.r2.l1 configure -text $steerLbl
        pack .ftop.input.r2.l1 -side left -padx 10
        pack .ftop.input.r2.e0 -side left  -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.r1b -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }
    
    proc packXerion { title wheelLbl killswitchLbl autosteerActiveLbl \
            autosteerEnabledLbl hardleftBtn leftBtn zeroBtn rightBtn \
            hardrightBtn returnBtn changeBtn } {
            
        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left -padx 5
        pack .ftop.input.r1.e1 -side left -padx 5

        # Row 1b
        .ftop.input.r1b.l1 configure -text $killswitchLbl
        .ftop.input.r1b.l2 configure -text $autosteerActiveLbl
        pack .ftop.input.r1b.l1 -side left 
        pack .ftop.input.r1b.e1 -side left -padx 5 
        pack .ftop.input.r1b.l2 -side left -padx 5 
        pack .ftop.input.r1b.e2 -side left -padx 5
        
        # Row 2
        .ftop.input.r2.l1 configure -text $autosteerEnabledLbl
        pack .ftop.input.r2.l1 -side left
        pack .ftop.input.r2.e0 -side left -padx 7  

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 5 -anchor w -expand no
        pack .ftop.input.r1b -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc packCatTrack { title killswitchLbl leftLbl rightLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left

        # Row 2
        .ftop.input.r2.l1 configure -text $leftLbl
        .ftop.input.r2.l2 configure -text $rightLbl
        pack .ftop.input.r2.l1 -side left
        pack .ftop.input.r2.e0 -side left -padx 10
        pack .ftop.input.r2.l2 -side left
        pack .ftop.input.r2.e1 -side left -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc packMT { title killswitchLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left
        pack .ftop.input.r1.l -side left -padx 30
        
        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc packGantry { title wheelLbl killswitchLbl leftLbl rightLbl \
            joyLbl seatSwLbl seatSwMaxLbl hardleftBtn leftBtn zeroBtn \
            rightBtn hardrightBtn returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l0 configure -text $wheelLbl
        pack .ftop.input.r1.l0 -side left
        pack .ftop.input.r1.e0 -side left
        pack .ftop.input.r1.e1 -side left
        pack .ftop.input.r1.l2 -side left
        pack .ftop.input.r1.e5 -side left

        # Row 2
        .ftop.input.r2.l1 configure -text $leftLbl
        .ftop.input.r2.l2 configure -text $rightLbl
        .ftop.input.r2.l3 configure -text $joyLbl
        pack .ftop.input.r2.l1 -side left
        pack .ftop.input.r2.e0 -side left -padx 10
        pack .ftop.input.r2.l2 -side left
        pack .ftop.input.r2.e1 -side left -padx 10
        pack .ftop.input.r2.l3 -side left
        pack .ftop.input.r2.e2 -side left -padx 10

        # Row 3
        .ftop.input.r2a.l1 configure -text $seatSwLbl 
        .ftop.input.r2a.l2 configure -text $seatSwMaxLbl 
        pack .ftop.input.r2a.l1 -side left
        pack .ftop.input.r2a.e0 -side left -padx 10
        pack .ftop.input.r2a.l2 -side left
        pack .ftop.input.r2a.e1 -side left -padx 10

        # Row 4
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.r2 -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.r2a -side top -padx 60 -pady 5 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }

    proc packCamecoHarv { title signalLbl spanLbl killswitchLbl \
            hardleftBtn leftBtn zeroBtn rightBtn hardrightBtn \
            returnBtn changeBtn } {

        .ftop.title.prompt configure -text $title

        # Row 1
        .ftop.input.r1.l2 configure -text $killswitchLbl
        .ftop.input.r1.l1 configure -text $signalLbl
        pack .ftop.input.r1.l1 -side left
        pack .ftop.input.r1.e2 -side left -padx 10
        pack .ftop.input.r1.e3 -side left
        pack .ftop.input.r1.e4 -side left -padx 10
        pack .ftop.input.r1.l2 -side left -padx 2
        pack .ftop.input.r1.e5 -side left

        # Row 1a
        .ftop.input.r1a.l1 configure -text $spanLbl
        pack .ftop.input.r1a.l1 -side left
        pack .ftop.input.r1a.e2 -side left -padx 10
        pack .ftop.input.r1a.e3 -side left
        pack .ftop.input.r1a.e4 -side left -padx 10

        # Row 3
        .ftop.input.r3.hardleft configure -text $hardleftBtn
        .ftop.input.r3.left configure -text $leftBtn
        .ftop.input.r3.zero configure -text $zeroBtn
        .ftop.input.r3.right configure -text $rightBtn
        .ftop.input.r3.hardright configure -text $hardrightBtn
        .ftop.input.r3.cancel configure -text $returnBtn
        .ftop.input.r3.change configure -text $changeBtn

        # Pack rows
        pack .ftop.input.r1 -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.r1a -side top -padx 60 -pady 10 -anchor w
        pack .ftop.input.pulseconf -side top -padx 5 -pady 3 -expand yes -fill both
        pack .ftop.input.r3 -side top -padx 5 -pady 3 -expand yes -fill both
    }
}

