##############################################################################
#
# vc/src/tcl/keyboard.tcl
#
# A modal-dialog-type keyboard with a user prompt
#
# 20020625 MLE - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Keyboard { 

    #-------------------------------------------------------------------------
    # Define and pack the keyboard
    #-------------------------------------------------------------------------
    # Top level frame
    frame .keyboard

    # Variables
    set keyWidth 3
    set keyHeight 3
    set userText ""

    # keyboard frame
    frame .keyboard.kbd

    # display frame
    frame .keyboard.kbd.display
    entry .keyboard.kbd.display.e -relief sunken -font $big_font -state normal
    
    pack .keyboard.kbd.display.e -side bottom -fill x -padx 20 -pady 1
    pack .keyboard.kbd.display -side top -pady 10 -fill both -expand yes

    # buttons frame
    frame .keyboard.kbd.but
    pack .keyboard.kbd.but -side bottom -fill both -expand yes

    # 4 rows in the keyboard...
    foreach r {4 3 2} {
        frame .keyboard.kbd.but.row$r
        pack .keyboard.kbd.but.row$r -side bottom -padx 1 \
            -fill both -expand yes
    }

    frame .keyboard.kbd.but.row1
    pack .keyboard.kbd.but.row1 -side bottom -padx 5 \
        -fill both -expand yes

    # special keys...
    SnazzyButton::create .keyboard.kbd.but.row4.keycancel btnKhakiM4U \
        btnKhakiM4D 0.5 "beepOnce; post $DISP_BTN0_SIG 0 \"cancel\"" "Cancel"
    pack .keyboard.kbd.but.row4.keycancel -side left -padx 1 -pady 1
    SnazzyButton::create .keyboard.kbd.but.row3.keyenter btnGreenM4U \
        btnGreenM4D 0.5 "beepOnce; post $DISP_BTN1_SIG 0 \"enter\"" "enter"
    pack .keyboard.kbd.but.row3.keyenter -side right -padx 1 -pady 1
    SnazzyButton::create .keyboard.kbd.but.row1.keybksp btnBlueS4U btnBlueS4D \
        0.5 "beepOnce; post $DISP_BTN2_SIG 0 \"backspace\"" "Bksp"
    pack .keyboard.kbd.but.row1.keybksp -side right -padx 1 -pady 1
    SnazzyButton::create .keyboard.kbd.but.row4.keyspace btnBlueM4U btnBlueM4D \
        0.5 "beepOnce; post $DISP_BTN3_SIG 0 \"_\"" "Space"
    pack .keyboard.kbd.but.row4.keyspace -side right -padx 1 -pady 1

    # spacers...
    button .keyboard.kbd.but.row2.spacer \
        -width 0 -height $keyHeight \
        -state disabled -relief flat
    pack .keyboard.kbd.but.row2.spacer -side left -pady 1
    button .keyboard.kbd.but.row3.spacer \
        -width 3 -height $keyHeight \
        -state disabled -relief flat
    pack .keyboard.kbd.but.row3.spacer -side left -pady 1

    # numbers...
    foreach k {1 2 3 4 5 6 7 8 9 0} {
        set cmd "beepOnce; post $DISP_BTN3_SIG $k \"$k\""
        eval { SnazzyButton::create .keyboard.kbd.but.row1.key$k btnBlueS3U \
                   btnBlueS3D 0.5 $cmd $k}
        pack .keyboard.kbd.but.row1.key$k -side left -pady 1
    }

    # letters...
    foreach {r k} {2 Q 2 W 2 E 2 R 2 T 2 Y 2 U 2 I 2 O 2 P \
                       3 A 3 S 3 D 3 F 3 G 3 H 3 J 3 K 3 L \
                       4 Z 4 X 4 C 4 V 4 B 4 N 4 M} {
        set cmd "beepOnce; post $DISP_BTN3_SIG 0 \"$k\""
        eval { SnazzyButton::create .keyboard.kbd.but.row$r.key$k \
                   btnBlueS3U btnBlueS3D 0.5 $cmd $k}
        pack .keyboard.kbd.but.row$r.key$k -side left -pady 1
    }

    # Pack various common elements
    pack .keyboard.kbd -side bottom -fill both -expand yes

    #-------------------------------------------------------------------------
    # Entry and exit procedures
    #-------------------------------------------------------------------------
    proc actEntry {} {
        pack forget .ftop
        pack .keyboard -fill both -expand yes
    }

    proc actExit {} {
        destroy .keyboard
        pack .ftop -fill both -expand yes
    }

    #-------------------------------------------------------------------------
    # Action procedures
    #-------------------------------------------------------------------------
    proc actDisp {text__} {
       .keyboard.kbd.display.e delete 0 end
       .keyboard.kbd.display.e insert end $text__         
    }
    
    proc actDispPassword {text__} {
       .keyboard.kbd.display.e delete 0 end
       .keyboard.kbd.display.e configure -show *
       .keyboard.kbd.display.e insert end $text__         
    }

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
}
