##############################################################################
#
# vc/src/tcl/numpad.tcl
#
# A modal-dialog-type numpad with a user prompt and toggle units
#
# 20020625 MLE - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Modal { 

    #-------------------------------------------------------------------------
    # Define and pack the numpad
    #-------------------------------------------------------------------------
    # Top level frame
    frame .fmodal

    # Variables
    set keyWidth 3
    set keyHeight 3
    set userText ""

    # numpad frame
    frame .fmodal.numpad

    # display frame
    frame .fmodal.numpad.display
    label .fmodal.numpad.display.l
    label .fmodal.numpad.display.e \
            -relief sunken -font $big_font -width 26 -anchor w
    pack .fmodal.numpad.display.e -side bottom -pady 5 -anchor sw -padx 200
    pack .fmodal.numpad.display.l -side top -padx 5 -pady 5
    pack .fmodal.numpad.display -side top -pady 5 -fill both -expand yes

    # buttons frame
    frame .fmodal.numpad.but
    pack .fmodal.numpad.but -side bottom -pady 2 -fill both -expand yes -anchor sw

    # divide remainder into 3 subframes
    frame .fmodal.numpad.but.l
    frame .fmodal.numpad.but.c
    frame .fmodal.numpad.but.r
    pack .fmodal.numpad.but.l -side left -padx 1 -pady 3 -fill both \
            -expand yes
    pack .fmodal.numpad.but.r -side right -padx 1 -pady 3 -fill both \
            -expand yes
    pack .fmodal.numpad.but.c -side bottom -padx 2 -pady 3 

    # 4 rows in the numpad...
    foreach r {4 3 2 1} {
        frame .fmodal.numpad.but.c.row$r
        pack .fmodal.numpad.but.c.row$r -side bottom -padx 10 -fill x
    }

    # special keys...
    SnazzyButton::create .fmodal.numpad.but.l.keycancel btnKhakiU \
       btnKhakiD 0.5 "beepOnce; post $DISP_BTN0_SIG 0 \"cancel\"" "Cancel"
    pack .fmodal.numpad.but.l.keycancel -side left -padx 2 -pady 0 -anchor sw
    SnazzyButton::create .fmodal.numpad.but.r.keyenter btnGreenU \
            btnGreenD 0.5 "beepOnce; post $DISP_BTN1_SIG 0 \"enter\"" "Enter" 
    pack .fmodal.numpad.but.r.keyenter -side bottom -padx 2 -pady 0 -anchor se
    SnazzyButton::create .fmodal.numpad.but.r.toggleUnits btnBlueU btnBlueD \
        0.5 "beepOnce; post $DISP_BTN2_SIG 0 \"toggle units\"" "toggle\nunits" 
    pack .fmodal.numpad.but.r.toggleUnits -side top -padx 5 -pady 5 -anchor nw
    SnazzyButton::create .fmodal.numpad.but.c.row4.keypm btnBlueS2U \
             btnBlueS2D 0.5 "beepOnce; post $DISP_BTN3_SIG 0 \"+/-\"" "+/-"
    pack .fmodal.numpad.but.c.row4.keypm -side right -padx 1 -pady 2
    SnazzyButton::create .fmodal.numpad.but.c.row4.keybksp btnBlueS2U \
         btnBlueS2D 0.5 "beepOnce; post $DISP_BTN4_SIG 0 \"backspace\"" "Bksp" 
    pack .fmodal.numpad.but.c.row4.keybksp -side right -padx 1 -pady 2
    SnazzyButton::create .fmodal.numpad.but.c.row4.keydot btnBlueS2U \
             btnBlueS2D 0.5 "beepOnce; post $DISP_BTN5_SIG 0 \"dot\"" "." 
    pack .fmodal.numpad.but.c.row4.keydot -side right -padx 1 -pady 2
    SnazzyButton::create .fmodal.numpad.but.c.row4.key0 btnBlueS2U \
             btnBlueS2D 0.5 "beepOnce; post $DISP_BTN6_SIG 0 \"0\"" "0" 
    pack .fmodal.numpad.but.c.row4.key0 -side left -padx 1 -pady 2

    # numeric buttons...
    foreach {r k} {1 1 1 2 1 3 \
                   2 4 2 5 2 6 \
                   3 7 3 8 3 9} {
                set cmd "beepOnce; post $DISP_BTN7_SIG $k \"$k\""
#               set cmd "-command {beepOnce; post $DISP_BTN7_SIG $k \"$k\"} "
               eval { \
                      SnazzyButton::create .fmodal.numpad.but.c.row$r.key$k \
                        btnBlueS2U btnBlueS2D 0.5 $cmd
                      SnazzyButton::setText .fmodal.numpad.but.c.row$r.key$k \
                        $k 0.5   
#                       button .fmodal.numpad.but.c.row$r.key$k \
#                       -text $k -font $big_font \
#                       -width $keyWidth -height $keyHeight \
#                       -activebackground $color3 \
#                       -background $color3 } \
#                       $cmd
               pack .fmodal.numpad.but.c.row$r.key$k -side left -padx 1 \
                       -pady 2
    }

    pack .fmodal.numpad -side bottom -fill both -expand yes -padx 0

    #-------------------------------------------------------------------------
    # Entry and exit procedures
    #-------------------------------------------------------------------------
    proc actEntry {cancelBtn enterBtn toggleBtn bkspBtn } {
        SnazzyButton::setText .fmodal.numpad.but.l.keycancel $cancelBtn 0.5
        SnazzyButton::setText .fmodal.numpad.but.r.keyenter $enterBtn 0.5
	SnazzyButton::setText .fmodal.numpad.but.c.row4.keybksp $bkspBtn 0.5
#        .fmodal.numpad.but.l.keycancel configure -text $cancelBtn
#        .fmodal.numpad.but.r.keyenter configure -text $enterBtn
        pack forget .ftop
        if {$toggleBtn != "!"} {       ;# should
         SnazzyButton::setText .fmodal.numpad.but.r.toggleUnits $toggleBtn 0.5
#            .fmodal.numpad.but.r.toggleUnits configure -text $toggleBtn
            pack .fmodal.numpad.but.r.toggleUnits \
                 -after .fmodal.numpad.but.r.keyenter \
                 -side top -padx 5 -pady 5 -anchor nw
        } else {
            pack forget .fmodal.numpad.but.r.toggleUnits
        }
        pack .fmodal -fill both -expand yes
    }

    proc actExit {} {
        destroy .fmodal
        pack .ftop -fill both -expand yes
    }

    #-------------------------------------------------------------------------
    # Action procedures
    #-------------------------------------------------------------------------
    proc actPrompt { {prompt__ "User prompt"} } {
        .fmodal.numpad.display.l configure -text "$prompt__"
    }

    proc actDisp {text__} {
        .fmodal.numpad.display.e configure -text $text__
    }

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    

}
