##############################################################################
#
# vc/src/tcl/numpadpanel.tcl
#
# Defines packable widgets to facilitate entry of various data types
# 
# 20080319 DEH Created
#
# Copyright (c) 2008 Novariant Inc
# All Rights Reserved. 
##############################################################################


#-------------------------------------------------------------------------
# NumPad
#-------------------------------------------------------------------------
namespace eval Numpad { 

    #-------------------------------------------------------------------------
    # Define and pack the numpad
    #-------------------------------------------------------------------------
    # Top level frame
    frame .numpad

    # divide frame into three parts
    frame .numpad.l
    frame .numpad.c
    frame .numpad.r
    label .numpad.prompt -text "numpad"
    pack .numpad.prompt -side top -padx 5 -pady 0

    pack .numpad.l -side left -padx 2 -pady 0 -fill both \
            -expand yes
    pack .numpad.r -side right -padx 1 -pady 0 -fill both \
            -expand yes
    pack .numpad.c -side right -padx 1 -pady 0 

    frame .numpad.c.display
    label .numpad.c.display.e \
            -relief sunken -font $big_font -width 20 -anchor w

    # display frame
    pack .numpad.c.display.e -side bottom -pady 0 -anchor sw -padx 2
    pack .numpad.c.display -side top -pady 0 -fill x -expand yes

    frame .numpad.c.but
    # 4 rows in the numpad...
    foreach r {4 3 2 1} {
        frame .numpad.c.but.row$r
        pack .numpad.c.but.row$r -side bottom -padx 0 -fill x
    }
    pack .numpad.c.but -side top -pady 0 -fill x -expand yes

    label .numpad.l.filler1 -text ""
    label .numpad.l.filler2 -text ""
    label .numpad.l.img -image smallAutoFarmLogo
    SnazzyButton::create .numpad.l.toggleUnits btnBlueSquareUp btnBlueSquareUp \
        0.5 "beepOnce; post $DISP_BTN2_SIG 0 \"toggle units\"" "toggle\nunits" 
    pack  .numpad.l.filler1 -side top
#    pack .numpad.l.filler2 -side top
    
#    pack .numpad.l.toggleUnits -side bottom -padx 5 -pady 1 -anchor ne

    # special keys...
    SnazzyButton::create .numpad.c.but.row4.keypm btnBlueSquare2Up \
             btnBlueSquare2Down 0.5 "beepOnce; post $DISP_BTN3_SIG 0 \"+/-\"" "+/-"
    pack .numpad.c.but.row4.keypm -side right -padx 1 -pady 0
    SnazzyButton::create .numpad.c.but.row4.keybksp btnBlueSquare2Up \
         btnBlueSquare2Down 0.5 "beepOnce; post $DISP_BTN4_SIG 0 \"backspace\"" "Bksp"
    pack .numpad.c.but.row4.keybksp -side right -padx 1 -pady 0
    SnazzyButton::create .numpad.c.but.row4.keydot btnBlueSquare2Up \
             btnBlueSqaure2Down 0.5 "beepOnce; post $DISP_BTN5_SIG 0 \"dot\"" "."
    pack .numpad.c.but.row4.keydot -side right -padx 1 -pady 0
    SnazzyButton::create .numpad.c.but.row4.key0 btnBlueSquare2Up \
             btnBlueSquare2Down 0.5 "beepOnce; post $DISP_BTN6_SIG 0 \"0\"" "0"
    pack .numpad.c.but.row4.key0 -side left -padx 1 -pady 0

    # numeric buttons...
    foreach {r k} {1 1 1 2 1 3 \
                   2 4 2 5 2 6 \
                   3 7 3 8 3 9} {
                set cmd "beepOnce; post $DISP_BTN7_SIG $k \"$k\""
#               set cmd "-command {beepOnce; post $DISP_BTN7_SIG $k \"$k\"} "
               eval { \
                      SnazzyButton::create .numpad.c.but.row$r.key$k \
                        btnBlueSquare2Up btnBlueSquare2Down 0.5 $cmd
                      SnazzyButton::setText .numpad.c.but.row$r.key$k \
                        $k 0.5   
		   pack .numpad.c.but.row$r.key$k -side left -padx 1 \
                       -pady 0
	       }
    }



    #-------------------------------------------------------------------------
    # Entry and exit procedures
    #-------------------------------------------------------------------------
    proc actEntry {prompt toggleBtn} {
        if {$toggleBtn != "!"} {       ;# should
         SnazzyButton::setText .numpad.l.toggleUnits $toggleBtn 0.5
            pack .numpad.l.toggleUnits \
                 -side top  -pady 0 -anchor ne
        } else {
            pack forget .numpad.l.toggleUnits
        }
        pack .numpad -fill both -expand yes
    }

    proc actConfigLanguage { toggleBtn bkspBtn } {
	.numpad.l.toggleUnits configure -text $toggleBtn
	.numpad.c.but.row4.keybksp configure -text $bkspBtn
    } 

    #-------------------------------------------------------------------------
    # Action procedures
    #-------------------------------------------------------------------------
    proc actPrompt { {prompt__ "User prompt"} } {
        .numpad.prompt configure -text "$prompt__"
    }

    proc actDisp {text__} {
        .numpad.c.display.e configure -text $text__
    }

    proc actClearDisp {} {
        .numpad.c.display.e configure -text ""
    }

    proc actCleanupDisp {} {
        .numpad.c.display.e configure -text ""
	pack forget .numpad.l.img
        .numpad.prompt configure -text ""
    }

    proc actNoToggle {} {
	pack forget .numpad.l.toggleUnits
    }

    proc actAddToggle {} {
	pack .numpad.l.toggleUnits -side bottom -padx 5 -anchor ne
    }

    proc actAddImage { imgName } {
	pack forget .numpad.l.img
	.numpad.l.img configure -image $imgName
	pack .numpad.l.img -side top -padx 1 -pady 0 -anchor nw
    }
}

