##############################################################################
#
# vc/src/tcl/databasediff.tcl
#
# Screen to show database diff.
#
# 20050802 QC - created
#
# Copyright (c) 2004 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout

    # Title
    label .ftop.title -text "Autofarm Postmortum"
                                                                                
    frame .ftop.input
    scrollbar .ftop.input.sbar -command { .ftop.input.lbox yview }
    listbox .ftop.input.lbox -yscrollcommand {.ftop.input.sbar set} \
            -width 35 -height 10

    # Cancel/Accept frame
    frame .ftop.input2
    SnazzyButton::create .ftop.input2.cancel \
                 btnKhakiForSatVUU btnKhakiForSatVUD 0.5 \
                 "beepOnce; post $DISP_BTN9_SIG 9;" "Exit"
    SnazzyButton::create .ftop.input2.export \
                 btnBlueForSatVUU btnBlueForSatVUD 0.5 \
         "beepOnce; post $DISP_BTN1_SIG 1;" "Export to\nCompact\nFlash"
    SnazzyButton::create .ftop.input2.view \
                 btnBlueForSatVUU btnBlueForSatVUD 0.5 \
         "beepOnce; post $DISP_BTN2_SIG 2;" "View"
        
    pack .ftop.input2.cancel -side left -padx 1 -anchor se
    pack .ftop.input2.view -side right -padx 1 -anchor sw
    pack .ftop.input2.export -side right -padx 1 -anchor sw

    # Pack .ftop
    pack .ftop.title -side top -pady 5
    pack .ftop.input -side top -padx 10
    pack .ftop.input2 -side bottom -anchor sw -padx 5 -pady 5 -fill both -expand yes
                
    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry {titleLbl returnBtn} {
 
      .ftop.title configure -text $titleLbl
      SnazzyButton::setText .ftop.input2.cancel $returnBtn 0.5
      # Pack the list box
      pack .ftop.input.lbox -side left -fill both
      pack .ftop.input.sbar -side left -fill y

      pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------

    proc actListboxInsert {fname} {
        .ftop.input.lbox insert end $fname
    }

    proc actListboxGetCurindex {} {
        if { [.ftop.input.lbox index end] != 0} {
            return [lindex [.ftop.input.lbox curselection] 0]
        } else {
            return -1             ;# -1 tells the C side the listbox was empty
        }
    }

    proc actHighLightBtn {itemId} {
        .ftop.input.lbox see $itemId
    }

    proc displayOutputFile {} {
        set fname "testfile"
        set fd 0
        if [catch {open $fname r} fd] {
           set fd 0
           .ftop.input.lbox insert end "empty"           
        } else {
           gets $fd str
           while {$str != ""} {
               .ftop.input.lbox insert end $str           
               gets $fd str
           }
           catch {close $fd}
        }
        .ftop.input.lbox see end
        pack .ftop -fill both -expand yes
    }
    
    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
}
