##############################################################################
#
# vc/src/tcl/rmFeatureCodeConfig.tcl
#
# Screen to configure roof module feature code.
#
# 20061019 RCM - created
#
# Copyright (c) 2006 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout

    # Title
    frame .ftop.title
    label .ftop.title.txt -text "Space for title"
    label .ftop.title.subtxt -text "Space for subtitle" -font $big_font 
    pack  .ftop.title.txt -side top -fill x -expand yes -anchor n -pady 10
    
    # Divider between title and content
    label .ftop.title.divider -image imgDivider
    pack  .ftop.title.divider -side top -fill x -expand yes -anchor n
    
    # Pack SubTitle Text
    pack  .ftop.title.subtxt -side top -fill x -expand yes -anchor n
    
    # Content
    frame .ftop.content
    
    # Current Roof Module Serial Number
    frame .ftop.content.serialnumber
    frame .ftop.content.serialnumber.labeltxt
    frame .ftop.content.serialnumber.value
    label .ftop.content.serialnumber.labeltxt.txt -text "Receiver Serial Number:" \
      -font $medium_font
    label .ftop.content.serialnumber.value.txt -text "Loading..." \
      -font $medium_font
    pack .ftop.content.serialnumber.labeltxt.txt -side left
    pack .ftop.content.serialnumber.value.txt -side left
    pack .ftop.content.serialnumber.labeltxt -side top
    pack .ftop.content.serialnumber.value -side top -padx 2
    pack .ftop.content.serialnumber -side top -anchor n -fill x
    
    # Current Roof Module Feature Code
    frame .ftop.content.activationcode
    frame .ftop.content.activationcode.labeltxt
    frame .ftop.content.activationcode.value
    label .ftop.content.activationcode.labeltxt.txt -text "Feature Code:" \
      -font $medium_font
    label .ftop.content.activationcode.value.txt -text "Loading..." \
      -font $medium_font
    pack .ftop.content.activationcode.labeltxt.txt -side left
    pack .ftop.content.activationcode.value.txt -side left
    pack .ftop.content.activationcode.labeltxt -side top
    pack .ftop.content.activationcode.value -side top
    #pack .ftop.content.activationcode -side top -anchor n -fill x -pady 10
    
    # Feature Code Configure Status
    frame .ftop.content.status
    frame .ftop.content.status.labeltxt
    frame .ftop.content.status.value
    label .ftop.content.status.labeltxt.txt -text "Status:" -font $medium_font
    label .ftop.content.status.value.txt -text "Loading..." -font $medium_font
    
    pack .ftop.content.status.labeltxt.txt -side left
    pack .ftop.content.status.value.txt -side bottom
    pack .ftop.content.status.labeltxt -side top
    pack .ftop.content.status.value -side bottom -padx 40
    pack .ftop.content.status -side bottom -anchor sw -fill both \
      -expand yes -padx 10 -pady 10 
    
    # Cancel/Accept button frame
    frame .ftop.input
    SnazzyButton::create .ftop.input.return btnKhakiU btnKhakiD 0.5 \
                 "beepOnce; post $DISP_BTN9_SIG 9;" "Return To Previous Menu"
    SnazzyButton::create .ftop.input.enter btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN0_SIG 0;" "Enter\nFeature\nCode"            
                     
    pack .ftop.input.return -side left -padx 5 -pady 5
    pack .ftop.input.enter -side right -padx 5 -pady 5
    
    # Pack .ftop
    pack .ftop.title -side top -pady 10 -fill x -expand yes -anchor n
    pack .ftop.content -side top -fill x -expand yes
    pack .ftop.input -side bottom -anchor se -fill x -expand yes
                
    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry {titleLbl subtitleLbl featureCodeLbl serialNumberLbl \
                   statusLbl enterBtn returnBtn} {
        .ftop.title.txt configure -text $titleLbl
        .ftop.title.subtxt configure -text $subtitleLbl
        .ftop.content.serialnumber.labeltxt.txt configure -text $serialNumberLbl
        .ftop.content.activationcode.labeltxt.txt configure -text $featureCodeLbl 
        .ftop.content.status.labeltxt.txt configure -text $statusLbl 
        SnazzyButton::setText .ftop.input.enter $enterBtn 0.5
        SnazzyButton::setText .ftop.input.return $returnBtn 0.5
        
        pack .ftop -fill both -expand yes
    }


    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actDisplayRmActiviationCode {rmactivationcode__} {
      .ftop.content.activationcode.value.txt configure -text $rmactivationcode__    
    }
    
    proc actDisplayRmSerialNumber {rmserialnumber__} {
      .ftop.content.serialnumber.value.txt configure -text $rmserialnumber__    
    }
    
    proc actDisplayRmResponse {rmresponse__} {
      .ftop.content.status.value.txt configure -text $rmresponse__    
    }

   
    
    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
}
