##############################################################################
#
# vc/src/tcl/samdiagmenu.tcl
#
# Screen to show SAM diagnostics Menu.
#
# 20040916 QC - created
#
# Copyright (c) 2004 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout

    # Title
    label .ftop.title -text "SAM Diagnostics Menu"
    
    # Sam Information: Version, Id, Type
    frame .ftop.saminfo -bd 1 -relief solid
    frame .ftop.saminfo.version
    frame .ftop.saminfo.version.initial -bd 1 -relief solid
    frame .ftop.saminfo.version.major -bd 1 -relief solid
    frame .ftop.saminfo.version.minor -bd 1 -relief solid
    frame .ftop.saminfo.version.cust -bd 1 -relief solid
    frame .ftop.saminfo.version.major.lbl
    frame .ftop.saminfo.version.minor.lbl
    frame .ftop.saminfo.version.cust.lbl
    frame .ftop.saminfo.version.major.text
    frame .ftop.saminfo.version.minor.text
    frame .ftop.saminfo.version.cust.text
    frame .ftop.saminfo.id -bd 1 -relief solid
    frame .ftop.saminfo.type -bd 1 -relief solid
    frame .ftop.saminfo.id.lbl
    frame .ftop.saminfo.type.lbl
    frame .ftop.saminfo.id.text
    frame .ftop.saminfo.type.text
    
    label .ftop.saminfo.version.initial.txt -text "Request Sent To SA Module" \
     -font $big_font -bg gray
    
    # Version
    label .ftop.saminfo.version.major.lbl.txt -text "Major:" -font $big_font
    label .ftop.saminfo.version.minor.lbl.txt -text "Minor:" -font $big_font
    label .ftop.saminfo.version.cust.lbl.txt  -text "Cust:" -font $big_font
    label .ftop.saminfo.version.major.text.txt -text "No Response From SA" \
     -font $big_font
    label .ftop.saminfo.version.minor.text.txt -text "No Response From SA" \
     -font $big_font
    label .ftop.saminfo.version.cust.text.txt  -text "No Response From SA" \
     -font $big_font
     
    # ID
    label .ftop.saminfo.id.lbl.txt  -text "ID:" -font $big_font
    label .ftop.saminfo.id.text.txt -text "No Response From SA" \
     -font $big_font
    
    # Type
    label .ftop.saminfo.type.lbl.txt  -text "Type:" \
     -font $big_font
    label .ftop.saminfo.type.text.txt -text "No Response From SA" \
     -font $big_font
     
    # Pack SAM Info
    pack .ftop.saminfo.version.initial.txt -side top -fill x -expand yes
    pack .ftop.saminfo.version.initial -side top -fill x -expand yes
    pack .ftop.saminfo.version.major.lbl -side left -fill x -expand yes
    pack .ftop.saminfo.version.major.text -side right -fill x -expand yes
    pack .ftop.saminfo.version.major -side top -fill x -expand yes
    pack .ftop.saminfo.version.minor.lbl -side left -fill x -expand yes
    pack .ftop.saminfo.version.minor.text -side right -fill x -expand yes
    pack .ftop.saminfo.version.minor -side top -fill x -expand yes
    pack .ftop.saminfo.version.cust.lbl -side left -fill x -expand yes
    pack .ftop.saminfo.version.cust.text -side top -fill x -expand yes
    pack .ftop.saminfo.version.cust -side top -fill x -expand yes
    pack .ftop.saminfo.version -side top -fill x -expand yes
    pack .ftop.saminfo.type.lbl -side left -fill x -expand yes
    pack .ftop.saminfo.type.text -side right -fill x -expand yes
    pack .ftop.saminfo.type -side top -fill x -expand yes
    pack .ftop.saminfo.id.lbl -side left -fill x -expand yes
    pack .ftop.saminfo.id.text -side right -fill x -expand yes
    pack .ftop.saminfo.id -side top -fill x -expand yes
    
    # Cancel/Accept frame
    frame .ftop.input
    SnazzyButton::create .ftop.input.cancel \
                 btnKhakiU btnKhakiD 0.5 \
                 "beepOnce; post $DISP_BTN9_SIG 9;" "Return To Previous Menu"
    pack .ftop.input.cancel -side left
    SnazzyButton::create .ftop.input.nvRamPut \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN0_SIG 0;" "NVRAM Set Test"
    pack .ftop.input.nvRamPut -side right
    SnazzyButton::create .ftop.input.nvRamGet \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN1_SIG 1;" "NVRAM Get Test"
    pack .ftop.input.nvRamGet -side right
    SnazzyButton::create .ftop.input.samdiag \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN2_SIG 2;" "SAM Diagnostics"
    pack .ftop.input.samdiag -side right -padx 5
    SnazzyButton::create .ftop.input.codedownload \
                 btnBlueU btnBlueD 0.5 \
                 "beepOnce; post $DISP_BTN3_SIG 3;" "Code Download"
    #pack .ftop.input.codedownload -side right

    # Pack .ftop
    pack .ftop.title -side top -pady 40
    pack .ftop.saminfo -side top
    pack .ftop.input -side bottom -anchor se -padx 5 -pady 5 \
                     -expand yes -fill x
                
    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry { titleLbl initial codeDownloadBtn \
                    samDiagBtn nvRamPutBtn nvRamGetBtn returnBtn } {
        .ftop.title configure -text $titleLbl
        .ftop.saminfo.version.initial.txt configure -text $initial
        SnazzyButton::setText .ftop.input.cancel $returnBtn 0.5
        SnazzyButton::setText .ftop.input.codedownload $codeDownloadBtn 0.5
        SnazzyButton::setText .ftop.input.samdiag $samDiagBtn 0.5
        SnazzyButton::setText .ftop.input.nvRamPut $nvRamPutBtn 0.5
        SnazzyButton::setText .ftop.input.nvRamGet $nvRamGetBtn 0.5
        pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }

    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------
    proc actConfigVersionMsg {title majorlbl majortxt minorlbl minortxt custlbl \
                              custtxt idlbl idtxt} {
      pack .ftop.saminfo.version.major.lbl.txt -side left
      pack .ftop.saminfo.version.major.text.txt -side right
	  pack .ftop.saminfo.version.minor.lbl.txt -side left
	  pack .ftop.saminfo.version.minor.text.txt -side right
	  pack .ftop.saminfo.version.cust.lbl.txt -side left
	  pack .ftop.saminfo.version.cust.text.txt -side left
	  pack .ftop.saminfo.type.lbl.txt -side left
	  pack .ftop.saminfo.type.text.txt -side right
	  pack .ftop.saminfo.id.lbl.txt -side left
	  pack .ftop.saminfo.id.text.txt -side right
	  
	  .ftop.saminfo.version.initial.txt configure -text $title
      .ftop.saminfo.version.major.lbl.txt configure -text $majorlbl
      .ftop.saminfo.version.major.text.txt configure -text $majortxt
      .ftop.saminfo.version.minor.lbl.txt configure -text $minorlbl
      .ftop.saminfo.version.minor.text.txt configure -text $minortxt
      .ftop.saminfo.version.cust.lbl.txt configure -text $custlbl
      .ftop.saminfo.version.cust.text.txt configure -text $custtxt
      .ftop.saminfo.id.lbl.txt configure -text $idlbl
      .ftop.saminfo.id.text.txt configure -text $idtxt
      
    }
    
    proc actConfigVehicleType { typelbl typetxt } {
      .ftop.saminfo.type.lbl.txt configure -text $typelbl
      .ftop.saminfo.type.text.txt configure -text $typetxt
    }

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
}
