##############################################################################
#
# vc/src/tcl/unconfiguredSAMWarning.tcl
#
# Unconfigured SA Module Warning Screen,
#
# 20060407 RCM - created
#
# Copyright (c) 2006 Novariant Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout
    # ..screen has custom display and input frame components 
    frame .ftop.display             ;# Display Frame
    frame .ftop.display.title       ;# Title Display Frame
    frame .ftop.display.info        ;# SAM Info Display Frame
    frame .ftop.display.info.top
    frame .ftop.display.info.bottom
    frame .ftop.input               ;# Input Frame
    
    pack .ftop.input -side bottom -fill both -expand yes -anchor s
    pack .ftop.display.title -side top -fill x -expand yes
    pack .ftop.display.info.top -side top -anchor w
    pack .ftop.display.info.bottom -side bottom -anchor w
    pack .ftop.display.info -side top
    pack .ftop.display -side bottom -fill both -expand yes

    

    # Title Display Frame Text
    label .ftop.display.title.text
    
    # SAM Info Display Frame\
     
    label .ftop.display.info.top.title -text "SA Module Configuration" \
                                       -font $big_font
    label .ftop.display.info.top.divider -text "------------------------------" \
                                       -font $big_font
    label .ftop.display.info.top.serialnum -text "Serial Number:" \
                                           -font $big_font
    label .ftop.display.info.bottom.samvtype -text "Vehicle Type  :" \
                                             -font $big_font
    label .ftop.display.info.top.num -text "UNKNOWN" -font $big_font
    label .ftop.display.info.bottom.vtype -text "UNKNOWN" -font $big_font
    label .ftop.display.info.disabletext -text "Disable SAM" \
                                         -font $big_font
         
    # Input frame 
    frame .ftop.input.r1
    SnazzyButton::create .ftop.input.r1.continue btnAcceptU btnAcceptD 0.2 \
                         "beepOnce; post $DISP_BTN0_SIG 0"

    # Return to Previous Menu button
    SnazzyButton::create .ftop.input.return btnKhakiU btnKhakiD 0.5 \
                         "beepOnce; post $DISP_BTN1_SIG 1"
    
    # Pack the display and input frames
    pack .ftop.display.title.text -side top -pady 5 -fill both -expand yes
    pack .ftop.display.info.top.title -side top -padx 5
    pack .ftop.display.info.top.divider -side top -padx 5
    pack .ftop.display.info.top.serialnum -side left -padx 30 -pady 5
    pack .ftop.display.info.bottom.samvtype -side left -padx 30
    pack .ftop.display.info.top.num -side left -padx 5 -anchor w
    pack .ftop.display.info.bottom.vtype -side left -padx 5 -anchor w
    pack .ftop.input.return -side left -padx 5 -pady 5 -anchor sw
    pack .ftop.input.r1 -side bottom -anchor se
    pack .ftop.input.r1.continue -side right -padx 5 -pady 5

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry {title returnBtn continueBtn} {
        .ftop.display.title.text configure -text $title
        SnazzyButton::setText .ftop.input.return $returnBtn 0.5
        SnazzyButton::setText .ftop.input.r1.continue $continueBtn 0.2
        pack .ftop -fill both -expand yes

    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }


    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------

    proc actDisplaySAMInfo {samConfigTitle samSerialNumLbl samSerialNum \
                            samVTypeLbl samVType} {
      
      .ftop.display.info.top.title configure -text $samConfigTitle 
      .ftop.display.info.top.serialnum configure -text $samSerialNumLbl 
      .ftop.display.info.top.num configure -text $samSerialNum
      .ftop.display.info.bottom.samvtype configure -text $samVTypeLbl
      .ftop.display.info.bottom.vtype configure -text $samVType
    }

    proc actDisplayDisableSAM {text} {
      
      # Remove SAM Configuration Text
      pack forget .ftop.display.info.top
      pack forget .ftop.display.info.bottom
      
      # Display Attempting to Disable SAM Text  
      .ftop.display.info.disabletext configure -text $text
      pack .ftop.display.info.disabletext -side top 
       
    }
    
    proc actEnableContinueBtn {} {
      .ftop.input.r1.continue configure -state normal
    }
    
    proc actDisableContinueBtn {} {
      .ftop.input.r1.continue configure -state disabled
    }
    
    proc actEnableReturnBtn {} {
      .ftop.input.return configure -state normal
    }
    
    proc actDisableReturnBtn {} {
      .ftop.input.return configure -state disabled
    }
    
    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
}
