##############################################################################
#
# vc/src/tcl/warning.tcl
#
# Indemnity warning screen,
#
# 20020724 MLE - created
#
# Copyright (c) 2002 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
namespace eval Screen { 

    #-------------------------------------------------------------------------
    # Define and pack the screen
    #-------------------------------------------------------------------------
    # All screens must create a top level frame .ftop and destroy it on exit
    frame .ftop

    # Define and pack the screen layout
    # (Screen has custom display and input frames
    # components) 
    frame .ftop.display         ;# Display Frame
    frame .ftop.input           ;# Input Frame
    pack .ftop.input -side bottom -fill both -expand yes -anchor s
    pack .ftop.display -side bottom -fill both -expand yes

    # Exit button
    SnazzyButton::create .ftop.input.exit btnExitKhakiU btnExitKhakiD 0.2 \
                         "beepOnce; guiExit"

    # Display Frame - displays up to 3 lines of text
    label .ftop.display.l0

    # Input frame 
    frame .ftop.input.r1
    SnazzyButton::create .ftop.input.r1.b0 btnAcceptU btnAcceptD 0.2 \
                         "beepOnce; post $DISP_BTN0_SIG 0"

    SnazzyButton::create .ftop.input.r1.b1 btnRejectU btnRejectD 0.25 \
                         "beepOnce; post $DISP_BTN1_SIG 1"

    # Pack the display and input frames
    pack .ftop.display.l0 -side top -pady 2 -fill both -expand yes
    pack .ftop.input.exit -side left -padx 0 -pady 2 -anchor sw
    pack .ftop.input.r1 -side bottom -anchor se
    pack .ftop.input.r1.b0 -side right -padx 0 -pady 2
    pack .ftop.input.r1.b1 -side right -padx 2 -pady 2

    #-------------------------------------------------------------------------
    # Standard entry and exit procedures (called by GuiMgr)
    #-------------------------------------------------------------------------
    # Action called by GuiMgr on entry to this screen
    proc actEntry {title exitBtn acceptBtn rejectBtn} {
        .ftop.display.l0 configure -text $title
        SnazzyButton::setText .ftop.input.exit $exitBtn 0.2
        SnazzyButton::setText .ftop.input.r1.b0 $acceptBtn 0.2
        SnazzyButton::setText .ftop.input.r1.b1 $rejectBtn 0.25
        pack .ftop -fill both -expand yes
    }

    # Action called by GuiMgr on exit from this screen
    proc actExit {} {
        destroy .ftop
    }


    #-------------------------------------------------------------------------
    # Screen action procedures (to be called by GuiMgr)
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Delegated procedures
    #-------------------------------------------------------------------------
    proc actChangeTractor { tractorName__ } {
        ConfigDisplay::actChangeTractor $tractorName__
    }

    proc actChangeImplement { implementName__ } {
        ConfigDisplay::actChangeImplement $implementName__
    }



    #-------------------------------------------------------------------------
    # Local procedures
    # {MLE-020418: Tcl/Tk goal should be to get rid of all local procedures, 
    # because they represent intelligence, which the display should not have.}
    #-------------------------------------------------------------------------
    # Local procedures
}
