##############################################################################
#
# syncdisk.tcl utility (synchronize databases 5.x <-> 5.y)
#
# Copyright (c) 2003 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
source iniparse.tcl
source fsm.tcl

set family helvetica
#set family {hp system}
set tiny_font [font create -family $family -weight bold -size -11]
set small_font [font create -family $family -weight bold -size -12]
set medium_font [font create -family $family -weight bold -size -14]
set big_font [font create -family $family -weight bold -size -16]
set btn_font [font create -family $family -weight bold -size -16]
set huge_font [font create -family $family -weight bold -size -24]
option add *Button*font $medium_font
option add *Text*font $medium_font
option add *Label*font $huge_font

set color3         #00BFFF                                       ;# Light Blue
set color5         "dark khaki"
set colorGood      "green"
set colorMedium    "yellow"
set colorBad       "red"


if { [llength $argv] < 6} {
    puts "Usage: wish8.4 syncdisk.tcl -- {fullScreenOption} {vcParentDir}\
            {localDbDir} {remoteDbDir} {toolDir} {target}\n\
            fullScreenOption can be 0 (window) or 1 (full screen)\n\
            e.g. wish8.4 syncdisk.tcl -- 0 /mnt/apps/ \
            /configuration/currdir0/ /extconfiguration/currdir0/ \
            {vcParentDir}/vcutils/bin/build_target GPS5001"
    exit
}
set isUserDisplay [lindex $argv 0]
set vcParentDir [lindex $argv 1]
set localDir [lindex $argv 2]
set remoteDir [lindex $argv 3]
set toolDir [lindex $argv 4]
set target [lindex $argv 5]

puts "full_screen=$isUserDisplay\n  vcParentDir=$vcParentDir\n \
      localDir=$localDir\n  remoteDir=$remoteDir\n toolDir=$toolDir\n \
      target=$target"

# Set the main window geometry
# NOTE: globals tclDir and isUserDisplay are set from the C
#       before executing this script
 if { $isUserDisplay && $target == "GPS5001"} {
   # User display used : fill the entire screen         
   set screenWidth [expr [lindex [wm maxsize .] 0] + 15]
   set screenHeight [expr [lindex [wm maxsize .] 1] + 28]
   set offset "-8-25"
   . configure -cursor tcross
   proc beepOnce {} { bell }
 } else {                    ;# No user display (we're running a simulation)                    
    set screenWidth 650
    set screenHeight 485
    set offset "+0+0"
    . configure -cursor hand2
    proc beepOnce {} {bell }
 }  
wm geometry . "${screenWidth}x${screenHeight}${offset}"
wm resizable . 0 0                              ;# Don't allow window resizing                             

# Button Images  .............................................................
image create photo btnAcceptU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnAcceptU.gif
image create photo btnAcceptD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnAcceptD.gif
image create photo btnRejectU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnRejectU.gif
image create photo btnRejectD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnRejectD.gif
image create photo btnExitRedU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnExitRedU.gif
image create photo btnExitRedD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnExitRedD.gif
image create photo btnKhakiU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnKhakiU.gif
image create photo btnKhakiD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnKhakiD.gif
image create photo btnBlueM2U -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnBlueM2U.gif
image create photo btnBlueM2D -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnBlueM2D.gif
image create photo btnBlueU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnBlueU.gif
image create photo btnBlueD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnBlueD.gif
image create photo btnBlueQuestionU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnBlueQuestionU.gif
image create photo btnBlueQuestionD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnBlueQuestionD.gif
image create photo btnMainSyncBlueU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnMainSyncBlueU.gif
image create photo btnMainSyncBlueD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnMainSyncBlueD.gif
image create photo btnMainSyncRedU -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnMainSyncRedU.gif
image create photo btnMainSyncRedD -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/btnMainSyncRedD.gif
image create photo bar -format gif -file \
    ${vcParentDir}vc/src/tcl/gif/bar.gif

# Sync Images.................................................................
image create photo ExclamationPoint -format gif -file ExclamationPoint.gif

#.............................................................................
namespace eval SnazzyButton {

    proc create { me imgU imgD yFrac command {text ""} } {
        global btn_font

        set xMax [image width  $imgU]
        set yMax [image height $imgU]
        set yTxt [expr $yMax*$yFrac]

        canvas $me -width $xMax -height $yMax
        $me create image    0    0 -anchor nw -image $imgU -tags imageU
        $me create image -200 -200 -anchor nw -image $imgD -tags imageD
        $me create text [expr $xMax/2] $yTxt \
	    -text $text -font $btn_font -justify center -tags text
        # create an off-screen text item to store the command
        $me create text 0 -50 -text $command -tags command            
        
        bind $me <ButtonPress-1> {
	    %W coords imageD    0    0
	    %W coords imageU -200 -200
	    set txtCoords [%W coords text]
	    %W coords text [expr [lindex $txtCoords 0] + 2] \
		[expr [lindex $txtCoords 1] + 2]
        }
        
        bind $me <ButtonRelease-1> {
	    set xMax [%W cget -width]
	    set yMax [%W cget -height]
	    %W coords imageD -200 -200
	    %W coords imageU    0    0
	    set txtCoords [%W coords text]
	    %W coords text [expr [lindex $txtCoords 0] - 2] \
		[expr [lindex $txtCoords 1] - 2]
	    if {0 < %x && %x < $xMax && 
		0 < %y && %y < $yMax} {
		uplevel #0 [%W itemcget command -text]
	    }
        }
    }

    proc setText { me text yFrac } {
        $me itemconfigure text -text $text

        set xMax [$me cget -width]
        set yMax [$me cget -height]
        $me coords text [expr $xMax/2] [expr $yMax*$yFrac]
    }

    proc setImage { me imgU imgD yFrac } {
        set xMax [image width  $imgU]
        set yMax [image height $imgU]
        $me configure -width  $xMax
        $me configure -height $yMax
        $me itemconfigure imageU -image $imgU
        $me itemconfigure imageD -image $imgD
        $me coords text [expr $xMax/2] [expr $yMax*$yFrac]
    }
}


# status bar .................................................................
image create photo smallLOGO -format gif -file smallLOGO.gif
frame .statusbar -height 25 
label .statusbar.logo -image smallLOGO
place .statusbar.logo -relx 1.0 -x -10 -y 2 -anchor ne 
label .statusbar.msg -width 65 -anchor w -font $small_font \
    -borderwidth 1 -relief sunken
place .statusbar.msg -x 5 -rely 0.5 -y 1 -anchor w
frame .statusbar.sep -height 2 -borderwidth 0 -background black
place .statusbar.sep -x 0 -y 0 -anchor nw -relwidth 1.0
pack .statusbar -side bottom -fill x

# title ......................................................................
label .title -text "Synchronizing Databases" -font $huge_font
place .title -relx 0.5 -y 15 -anchor n

# about Sync .................................................................
SnazzyButton::create .aboutsync btnBlueQuestionU btnBlueQuestionD \
    0.5 "beepOnce; FSM:dispatch ABOUT;" ""

# screen1 main menu ..........................................................
frame .s1 -width 600 -height 350

SnazzyButton::create .s1.automatic btnMainSyncBlueU btnMainSyncBlueD \
    0.5 "beepOnce; FSM:dispatch AUTO;" "Automatic\nMerge"
place .s1.automatic -relx 0.5 -y 5 -anchor n

SnazzyButton::create .s1.manual btnMainSyncBlueU btnMainSyncBlueD \
    0.5 "beepOnce; FSM:dispatch MANUAL;" "Manual\nMerge"
place .s1.manual -relx 0.5 -y 110 -anchor n

SnazzyButton::create .s1.clear btnMainSyncRedU btnMainSyncRedD \
    0.5 "beepOnce; FSM:dispatch CLEAR;" "Clear\nSyncDisk\n(DANGEROUS)"
place .s1.clear -relx 0.5 -y 215 -anchor n

SnazzyButton::create .s1.exit btnExitRedU btnExitRedD \
    0.2 "beepOnce; FSM:dispatch SHUTDOWN;" "Exit"
place .s1.exit -relx 1.0 -x -5 -rely 1.0 -anchor se

# screen2 automatic menu .....................................................
frame .s2 -width 600 -height 350

label .s2.lbl
place .s2.lbl -relx 0.5 -y 50 -anchor n

button .s2.back -width 8 -height 4 -text "Back" \
    -background $color5 -activebackground $color5 \
    -command { beepOnce; FSM:dispatch BACK }

# screen3 manual menu ........................................................
frame .s3 -width 600 -height 400

SnazzyButton::create .s3.back btnKhakiU btnKhakiD \
    0.5 "beepOnce; FSM:dispatch BACK;" "Back"

frame .s3.conflict -width 600 -height 400
label .s3.conflict.attention -font $big_font -text "Attention:"
place .s3.conflict.attention -relx .54 -y 20 -anchor n 
label .s3.conflict.title -font $big_font -width 300
place .s3.conflict.title -relx .5 -y 55 -anchor n
label .s3.conflict.exclamationPoint -image ExclamationPoint  \
    -width 45 -height 50
place .s3.conflict.exclamationPoint  -relx 0.42 -y 2 -anchor n
label .s3.conflict.remoteLbl -text "On Your Sync Disk:" -font $medium_font 
place .s3.conflict.remoteLbl -relx 0.20 -y 90 -anchor n

scrollbar .s3.conflict.sbar1 -orient vertical \
    -command [list .s3.conflict.remoteTxt yview] \
    -width 38 

scrollbar .s3.conflict.sbar2 -orient vertical \
    -command [list .s3.conflict.localTxt yview] \
    -width 38

text  .s3.conflict.remoteTxt -width 28 -height 19 -font $small_font \
    -yscrollcommand [list .s3.conflict.sbar1 set] \
    -wrap word -background white
place .s3.conflict.sbar1 -relx 0.0 -y 110 -height 274   
place .s3.conflict.remoteTxt -relx 0.245 -y 110 -anchor n

label .s3.conflict.localLbl -text "On Your GPS System:" -font $medium_font
place .s3.conflict.localLbl -relx 0.80 -y 90 -anchor n

text  .s3.conflict.localTxt -width 28 -height 19 -font $small_font \
    -yscrollcommand [list .s3.conflict.sbar2 set] \
    -wrap word -background white
place .s3.conflict.sbar2 -relx .925 -y 110 -height 274                             
place .s3.conflict.localTxt -relx 0.755 -y 110 -anchor n


SnazzyButton::create .s3.conflict.toLocal btnBlueM2U btnBlueM2D \
    0.5 "beepOnce; FSM:dispatch TO_LOCAL;" "Copy >>"

SnazzyButton::create .s3.conflict.toRemote btnBlueM2U btnBlueM2D \
    0.5 "beepOnce; FSM:dispatch TO_REMOTE;" "<< Copy"

SnazzyButton::create .s3.conflict.delLocal btnBlueM2U btnBlueM2D \
    0.5 "beepOnce; FSM:dispatch DEL_LOCAL;" "Delete ->"

SnazzyButton::create .s3.conflict.delRemote btnBlueM2U btnBlueM2D \
    0.5 "beepOnce; FSM:dispatch DEL_REMOTE;" "<- Delete"

SnazzyButton::create .s3.conflict.skip btnBlueM2U btnBlueM2D \
    0.5 "beepOnce; FSM:dispatch SKIP;" "Skip"
place .s3.conflict.skip -relx 0.501 -y 245 -anchor n

SnazzyButton::create .s3.conflict.skipAll btnBlueM2U btnBlueM2D \
    0.5 "beepOnce; FSM:dispatch SKIP_ALL;" "Skip All"
place .s3.conflict.skipAll -relx 0.501 -y 320 -anchor n

frame .s3.section -width 600 -height 270
label .s3.section.title -font $huge_font
place .s3.section.title -relx 0.5 -y 80 -anchor n

SnazzyButton::create .s3.section.yes btnAcceptU btnAcceptD \
    0.2 "beepOnce; FSM:dispatch YES;" "YES"
place .s3.section.yes -relx 0.75 -rely 1.0 -y -10 -anchor s

SnazzyButton::create .s3.section.no btnRejectU btnRejectD \
    0.2 "beepOnce; FSM:dispatch NO;" "NO"
place .s3.section.no -relx 0.25 -rely 1.0 -y -10 -anchor s

#screen 4 about syncdisk......................................................
frame .s4 -width 600 -height 380

SnazzyButton::create .s4.back btnKhakiU btnKhakiD \
    0.5 "beepOnce; FSM:dispatch BACK;" "Back"

label .s4.intro1 -text \
    "The syncdisk program compares the Base Station, Field, Job, and\
Implement entries on your\nSyncdisk and GPS System. The table below\
shows the action that will be performed for each of\n the four possible\
cases encountered when comparing the data. The only differences between\n\
Automatic and Manual modes are the actions taken in cases 3 and 4." \
    -font $small_font
place .s4.intro1 -relx 0 -y 0 -anchor nw 

label .s4.header1 -text "Case" -font $big_font
place .s4.header1 -relx 0 -y 70 -height 30 -anchor nw 

label .s4.header2 -text "Automatic Mode" -font $big_font
place .s4.header2 -relx 0.55 -y 70 -height 30 -anchor nw

label .s4.header3 -text "Manual Mode" -font $big_font
place .s4.header3 -relx 0.80 -y 70 -height 30 -anchor nw

label .s4.bar -image bar
place .s4.bar -relx 0 -y 93 -height 10 -anchor nw

label .s4.case1 -text "1. Entry exists on Syncdisk and on GPS System:\n \
                        same name, same data." -font $small_font
place .s4.case1 -relx 0 -y 123 -anchor w

label .s4.auto1 -text "Do Nothing." -font $small_font
place .s4.auto1 -relx 0.55 -y 115 -anchor w

label .s4.manual1 -text "Do Nothing." -font $small_font
place .s4.manual1 -relx 0.80 -y 115 -anchor w

label .s4.case2 -text "2. Entry exists on Syncdisk and on GPS System:\n \
                        same name, different data." -font $small_font
place .s4.case2 -relx 0 -y 162 -anchor w

label .s4.auto2 -text "Ask user." -font $small_font
place .s4.auto2 -relx 0.55 -y 156 -anchor w

label .s4.manual2 -text "Ask user." -font $small_font
place .s4.manual2 -relx 0.80 -y 156 -anchor w

label .s4.case3 -text "3. Entry exists only on Syncdisk." -font $small_font
place .s4.case3 -relx 0 -y 203 -anchor w

label .s4.auto3 -text "Copy to GPS System." -font $small_font
place .s4.auto3 -relx 0.55 -y 201 -anchor w

label .s4.manual3 -text "Ask user." -font $small_font
place .s4.manual3 -relx 0.80 -y 201 -anchor w

label .s4.case4 -text "4. Entry exists only on GPS System." -font $small_font
place .s4.case4 -relx 0 -y 248 -anchor w

label .s4.auto4 -text "Copy to Syncdisk." -font $small_font
place .s4.auto4 -relx 0.55 -y 246 -anchor w

label .s4.manual4 -text "Ask user." -font $small_font
place .s4.manual4 -relx 0.80 -y 246 -anchor w

#-----------------------------------------------------------------------------
# helper procedures
#
proc status { msg } {
    .statusbar.msg configure -text $msg
}

#=============================================================================
# main menu state (selection of options)
#
proc main_menu { sig data } {
    global vcParentDir toolDir localDir remoteDir mode locFileList
    global remoteFileList remoteDelFileList


    switch -- $sig {
        ENTRY {
            .title configure -text "Choose Operation"
            place .s1 -relx 0.5 -y 80 -anchor n
            place .aboutsync -relx 0.73 -y 10 -anchor n
            status "Choose operation"
            lappend locFileList
            lappend remoteFileList
            lappend remoteDelFileList
        }
        CLEAR {
	    place forget .aboutsync
	    FSM:tran confirm_clear    
        }
        AUTO {
	    place forget .aboutsync
	    set mode "AUTO"
	    if {[manual_init] == 0} {
		FSM:tran manual_base_sect
	    }    
        }
        MANUAL {
        place forget .aboutsync  
	    set mode "MANUAL"
	    if {[manual_init] == 0} {
                FSM:tran manual_base_sect
	    }
        }
        ABOUT {
	    place forget .aboutsync
	    FSM:tran about_syncdisk
        }
        SHUTDOWN {
            exit
        }
        EXIT {
        }
    }
}

FSM:init main_menu                  ;# NOW we can initialize the state machine

#=============================================================================
# confirm automatic merge
#
proc confirm_auto { sig data } {
    global mode
    switch -- $sig {
	ENTRY {
	    place forget .s3.back
	    place .s3 -relx 0.5 -y 50 -anchor n
	    .title configure -text "Confirm Automatic Merge"
	    .s3.section.title configure -text "Do you wish to continue?"
	    place .s3.section -x 0 -y 0
	    status "Confirm operation"
	}
	YES {
	    place forget .s3
	    set mode "AUTO"
	    if {[manual_init] == 0} {
		FSM:tran manual_base_sect
	    }
	}
	NO {
	    place forget .s3
	    FSM:tran main_menu
	}  
    }
}

#=============================================================================
# confirm clear syncdisk
#
proc confirm_clear { sig data } {
    global vcParentDir toolDir localDir remoteDir mode
    switch -- $sig {
	ENTRY {
	    place forget .s3.back
	    place .s3 -relx 0.5 -y 50 -anchor n
	    .title configure -text "Confirm Clearing SyncDisk"
	    .s3.section.title configure -text "Do you wish to continue?"
	    place .s3.section -x 0 -y 0
	    status "Confirm operation"
	}
	YES {
	    # catch {exec mount -o rw ...}
	    if [catch {exec ${toolDir}ini2db \
			   ${vcParentDir}vc/src/data/template.ini \
			   ${remoteDir}dbmain.db}] {
		  status "Clear SyncDisk failed"
	    } else {
	      status "SyncDisk cleared"
	    }
        
        #Serach the sync disk for any files ending in ".db"
        #If a "vehicle_*.db" file is found, add its file name to the list
        #For all of the file names in the list delete
        
        if [catch {glob ${remoteDir}/vehicle_*.db} fileList] {
          status "Error reading sync disk!"
        } else {
            foreach fileName $fileList {
              if [catch {file delete -force $fileName}] {
                status "Error deleting: $fileName"
              } else {
                puts "Deleting file: [file tail $fileName]"
              }
            }
        }
        
	    place forget .s3
	    FSM:tran main_menu
	    # catch {exec umount -o rw ...}
	}
	NO {
	    place forget .s3
	    FSM:tran main_menu
	}  
    }
}

#=============================================================================
# about SyncDisk
#
proc about_syncdisk { sig data } {

    switch -- $sig {
	ENTRY {
	    place .s4.back -relx 1.0 -x -5 -rely 1.0 -anchor se
	    place .s4 -relx 0.5 -y 50 -anchor n
	    .title configure -text "SyncDisk Help"
	    status "Help"  
	}
	BACK {
	    place forget .s4 
	    FSM:tran main_menu
	}
    }
}
#=============================================================================
# automatic merge state
#
proc auto_menu { sig data } {
    switch -- $sig {
        ENTRY {
            .s2.lbl configure -text "Automatic Merge\nPlease Wait..."
            place .s2 -relx 0.5 -y 80 -anchor n
            status "synchronizing..."
            update
            autoSynch
            .s2.lbl configure -text "Automatic Merge\nDone."
            place .s2.back -relx 1.0 -x -20 -rely 1.0 -y -10 -anchor se
        }
        BACK {
            FSM:tran main_menu
        }
        EXIT {
            place forget .s2
        }
    }
}

#=============================================================================
# manual merge states 
#
proc manual_base_sect { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx mode 
    global section entry objFormat objDifferent mode


    switch -- $sig {
        ENTRY {
            place forget .s3.back
            place .s3 -relx 0.5 -y 50 -anchor n

            # start with synchronizing [base stations]
            set section "base stations"
            .title configure -text "Synchronizing Databases"
            .s3.section.title configure -text "Synchronize $section?"
            place .s3.section -x 0 -y 0
            #Auto Mode
            if { $mode == "AUTO" } {
		      FSM:dispatch YES
            }  
        }
        YES {
            set localList [readini $localId $section]
            set remoteList [readini $remoteId $section]
            set remoteIdx -1
            set localIdx -2
            set objFormat base_station_format
            set objDifferent base_station_different
            syncSize $localId $remoteId $section
            FSM:tran manual_base_conflict 
        }
        NO {
            FSM:tran manual_fields_sect
        }
        EXIT {
        }
    }
}         

# manual base conflict state .................................................
#
proc manual_base_conflict { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry mode getUserConfirmation 
    
    switch -- $sig {
        ENTRY {
            if {[searchForNextConflict] == 1} {
		place forget .s3.section
		place .s3.conflict -x 0 -y 0
            } else {
                FSM:tran manual_fields_sect  
	    }
        }
        TO_LOCAL {                                ;# copy from remote to local
            set remoteData [readini $remoteId $section $entry]
            writeini $localId $section $entry $remoteData
            puts "copied $entry to local"  
            if {$getUserConfirmation == 1} {
		if {[searchForNextConflict] == 0} {
		    place forget .s3.conflict
		    FSM:tran manual_fields_sect
		}
            }
        }
        TO_REMOTE {                               ;# copy from local to remote
            set localData [readini $localId $section $entry]
            writeini $remoteId $section $entry $localData
            puts "copied $entry to remote"
            if {$getUserConfirmation == 1} {  
		if {[searchForNextConflict] == 0} {
		    place forget .s3.conflict
		    FSM:tran manual_fields_sect
		}
            }
        }
        DEL_LOCAL {
            iniparse:removeitem $localId $section $entry
            purge $localId fields $entry        ;# delete all dependent fields
            purge $localId jobs $entry            ;# delete all dependent jobs
            puts "deleted $entry from local"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_fields_sect
            }
        }
        DEL_REMOTE {
            iniparse:removeitem $remoteId $section $entry
            purge $remoteId fields $entry       ;# delete all dependent fields
            purge $remoteId jobs $entry           ;# delete all dependent jobs
            puts "deleted $entry from remote"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_fields_sect
            }
        }
        SKIP {
            puts "skipped $entry"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_fields_sect
            }
        }
        SKIP_ALL {
            place forget .s3.conflict
            FSM:tran manual_fields_sect
        }       
    }
}

# manual fields section state ................................................
#
proc manual_fields_sect { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry objFormat objDifferent mode

    switch -- $sig {
        ENTRY {
            set section "fields"
            .title configure -text "Synchronizing Databases"
            .s3.section.title configure -text "Synchronize $section?"
            place .s3.section -x 0 -y 0
            #Auto Mode
            if { $mode == "AUTO" } {
		FSM:dispatch YES
            }
        }
        YES {
            # CAUTION: field section requires a special treatment, because
            # fields are multi-entry objects (i.e., a field with boundary can
            # occupy many entires, which are the boundary points).
            
            # prune the field boundary points from local/remote entry lists
            set lst [readini $remoteId $section]
            set remoteList [list]                       ;# make the list empty
            foreach e $lst {
                if {[string index $e end] != "."} {         ;# boundary point?
                    lappend remoteList $e
                }
            }
            set lst [readini $localId $section]
            set localList [list]                        ;# make the list empty
            foreach e $lst {
                if {[string index $e end] != "."} {         ;# boundary point?
                    lappend localList $e
                }
            }

            set remoteIdx -1
            set localIdx -2
            set objFormat field_format
            set objDifferent field_different
            syncSize $localId $remoteId $section
            FSM:tran manual_field_conflict
            
        }
        NO {
            FSM:tran manual_jobs_sect
        }
        EXIT {
        }
    }
}

# manual field conflict state ................................................
#
proc manual_field_conflict { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry mode getUserConfirmation
    
    # CAUTION: field section requires a special treatment, because
    # fields are multi-entry objects (i.e., a field with boundary can
    # occupy many entires, which are the boundary points).

    switch -- $sig {
        ENTRY {
            if {[searchForNextConflict] == 1} {
                place forget .s3.section
                place .s3.conflict -x 0 -y 0
	    } else {
		FSM:tran manual_jobs_sect  
	    }  
        }
        TO_LOCAL {                                ;# copy from remote to local
            set remoteData [readini $remoteId $section $entry]
            writeini $localId $section $entry $remoteData
            scan $remoteData "%f,%d" softwareVersion n
            puts "1 softwareVersion=$softwareVersion, n=$n"
            for {set i 0} {$i < $n} {incr i} {
                set point [readini $remoteId $section "${entry}.${i}."]
                writeini $localId $section "${entry}.${i}." $point
                puts "copied ${entry}.${i}. to local"
            }
            puts "copied $entry to local"
            if {$getUserConfirmation == 1} {
		if {[searchForNextConflict] == 0} {
		    place forget .s3.conflict
		    FSM:tran manual_jobs_sect
		}
            }

        }
        TO_REMOTE {                               ;# copy from local to remote
            set localData [readini $localId $section $entry]
            writeini $remoteId $section $entry $localData
            scan $localData "%f,%d" softwareVersion n
            puts "2 softwareVersion=$softwareVersion, n=$n"
            for {set i 0} {$i < $n} {incr i} {
                set point [readini $localId $section "${entry}.${i}."]
                writeini $remoteId $section "${entry}.${i}." $point
                puts "copied ${entry}.${i}. to local"
            }
            puts "copied $entry to remote"
            if {$getUserConfirmation == 1} {
		if {[searchForNextConflict] == 0} {
		    place forget .s3.conflict
		    FSM:tran manual_jobs_sect
		}
            }

        }
        DEL_LOCAL {
            scan [readini $localId $section $entry] "%f,%d" softwareVersion n
            puts "3 softwareVersion=$softwareVersion, n=$n"
            for {set i 0} {$i < $n} {incr i} {
                iniparse:removeitem $localId $section "${entry}.${i}."
                puts "deleted ${entry}.${i}. from local"
            }
            iniparse:removeitem $localId $section $entry

            purge $localId jobs $entry            ;# delete all dependent jobs

            puts "deleted $entry from local"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_jobs_sect
            }
        }
        DEL_REMOTE {
            scan [readini $remoteId $section $entry] "%f,%d" softwareVersion n
            puts "4 softwareVersion=$softwareVersion, n=$n"
            for {set i 0} {$i < $n} {incr i} {
                iniparse:removeitem $remoteId $section "${entry}.${i}."
                puts "deleted ${entry}.${i}. from remote"
            }
            iniparse:removeitem $remoteId $section $entry

            purge $remoteId jobs $entry           ;# delete all dependent jobs

            puts "deleted $entry from remote"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_jobs_sect
            }
        }
        SKIP {
            puts "skipped $entry"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_jobs_sect
            }
        }
        SKIP_ALL {
            place forget .s3.conflict
            FSM:tran manual_jobs_sect
        }
    }
}

# manual jobs section state ..................................................
#
proc manual_jobs_sect { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry objFormat objDifferent mode

    switch -- $sig {
        ENTRY {
            set section "jobs"
            .title configure -text "Synchronizing Databases"
            .s3.section.title configure -text "Synchronize $section?"
            place .s3.section -x 0 -y 0
            #Auto Mode
            if { $mode == "AUTO" } {
		FSM:dispatch YES
            }
        }
        YES {
            set localList [readini $localId $section]
            set remoteList [readini $remoteId $section]
            set remoteIdx -1
            set localIdx -2
            set objFormat job_format
            set objDifferent job_different
            syncSize $localId $remoteId $section
            FSM:tran manual_job_conflict 
        }
        NO {
            FSM:tran manual_implements_sect
        }
        EXIT {
        }
    }
}

# manual job conflict state ..................................................
#
proc manual_job_conflict { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry mode getUserConfirmation
    
    switch -- $sig {
        ENTRY {
            if {[searchForNextConflict] == 1} {
                place forget .s3.section
                place .s3.conflict -x 0 -y 0
            } else {
                FSM:tran manual_implements_sect  
	    }  
        }
        TO_LOCAL {                                ;# copy from remote to local
            set remoteData [readini $remoteId $section $entry]
            writeini $localId $section $entry $remoteData
            puts "copied $entry to local"
            if {$getUserConfirmation == 1} {
		if {[searchForNextConflict] == 0} {
		    place forget .s3.conflict
		    FSM:tran manual_implements_sect
		}
            }        
        }
        TO_REMOTE {                               ;# copy from local to remote
            set localData [readini $localId $section $entry]
            writeini $remoteId $section $entry $localData
            puts "copied $entry to remote"
            if {$getUserConfirmation == 1} {
		if {[searchForNextConflict] == 0} {
		    place forget .s3.conflict
		    FSM:tran manual_implements_sect
		}
            }
        }
        DEL_LOCAL {
            iniparse:removeitem $localId $section $entry
            puts "deleted $entry from local"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_implements_sect
            }
        }
        DEL_REMOTE {
            iniparse:removeitem $remoteId $section $entry
            puts "deleted $entry from remote"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_implements_sect
            }
        }
        SKIP {
            puts "skipped $entry"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_implements_sect
            }
        }
        SKIP_ALL {
            place forget .s3.conflict
            FSM:tran manual_implements_sect
        }
    }
}

# manual implements section state ............................................
#
proc manual_implements_sect { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry objFormat objDifferent mode

    switch -- $sig {
        ENTRY {
            set section "implements"
            .title configure -text "Synchronizing Databases"
            .s3.section.title configure -text "Synchronize $section?"
            place .s3.section -x 0 -y 0
            #Auto Mode
            if { $mode == "AUTO" } {
		      FSM:dispatch YES
            }
        }
        YES {
            set localList [readini $localId $section]
            set remoteList [readini $remoteId $section]
            set remoteIdx -1
            set localIdx -2
            set objFormat implement_format
            set objDifferent implement_different
            syncSize $localId $remoteId $section
            FSM:tran manual_implement_conflict
        }
        NO {
            FSM:tran manual_accept
        }
        EXIT {
        }
    }
}

# manual implement conflict state ............................................
#
proc manual_implement_conflict { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry mode getUserConfirmation
    
    switch -- $sig {
        ENTRY {
            if {[searchForNextConflict] == 1} {
                place forget .s3.section
                place .s3.conflict -x 0 -y 0
            } else {
                FSM:tran manual_accept  
	    } 
        }     
        TO_LOCAL {                                ;# copy from remote to local
            set remoteData [readini $remoteId $section $entry]
            writeini $localId $section $entry $remoteData
            puts "copied $entry to local"
            if {$getUserConfirmation == 1} {
		      if {[searchForNextConflict] == 0} {
		        place forget .s3.conflict
		        FSM:tran manual_accept
		      }
            } 
        }
        TO_REMOTE {                               ;# copy from local to remote
            set localData [readini $localId $section $entry]
            writeini $remoteId $section $entry $localData
            puts "copied $entry to remote"
            if {$getUserConfirmation == 1} {
		      if {[searchForNextConflict] == 0} {
		        place forget .s3.conflict
		        FSM:tran manual_accept
		      }
            }
        }
        DEL_LOCAL {
            iniparse:removeitem $localId $section $entry
            puts "deleted $entry from local"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_accept
            }
        }
        DEL_REMOTE {
            iniparse:removeitem $remoteId $section $entry
            puts "deleted $entry from remote"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_accept
            }
        }
        SKIP {
            puts "skipped $entry"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_accept
            }
        }
        SKIP_ALL {
            place forget .s3.conflict
            FSM:tran manual_accept
        }
    }
}

# manual tractors section state ............................................
#
proc manual_tractors_sect { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry objFormat objDifferent mode

    switch -- $sig {
        ENTRY {
            set section "tractors"
            .title configure -text "Synchronizing Databases"
            .s3.section.title configure -text "Synchronize $section?"
            place .s3.section -x 0 -y 0
            #Auto Mode
            if { $mode == "AUTO" } {
		      FSM:dispatch NO
            }
        }
        YES {
            set localList [readini $localId $section]
            set remoteList [readini $remoteId $section]
            set remoteIdx -1
            set localIdx -2
            set objFormat tractors_format
            set objDifferent tractors_different
            syncSize $localId $remoteId $section
            FSM:tran manual_tractors_conflict
        }
        NO {
            FSM:tran manual_accept
        }
        EXIT {
        }
    }
}

# manual tractors conflict state ............................................
#
proc manual_tractors_conflict { sig data } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry mode getUserConfirmation localDir remoteDir
    global locFileList remoteFileList remoteDelFileList
    
    switch -- $sig {
        ENTRY {
            if {[searchForNextConflict] == 1} {
                place forget .s3.section
                place .s3.conflict -x 0 -y 0
            } else {
                FSM:tran manual_accept  
	    } 
        }     
        TO_LOCAL {                                ;# copy from remote to local
        
            
            set remoteData [readini $remoteId $section $entry]
            scan $remoteData "%f" remoteSWVersion
            
            
            # Get the first or possibly only tractor fromthe local database
            set localEntry [lindex $localList 1]
            
            # ...retrieve the vehicles data 
            set localData [readini $localId $section $localEntry]
            
            # ...set localSWVersion to the software version of the vehicle 
            scan $localData "%f" localSWVersion
            
            puts "On Remote: $remoteSWVersion"
            puts "On Local: $localSWVersion"
            if {$remoteSWVersion == $localSWVersion} {
              set vehicleDB [format "vehicle_%s.db" $entry]
              if [file exists ${remoteDir}${vehicleDB}] {
                writeini $localId $section $entry $remoteData
                lappend remoteFileList $vehicleDB  
                puts "copied $vehicleDB to list for copying"
              } else {
                puts "file $vehicleDB could not be found on remote"
              }
              if {$getUserConfirmation == 1} {
		        if {[searchForNextConflict] == 0} {
		          place forget .s3.conflict
		          FSM:tran manual_accept
		        }
              }
            } else {
                status "Remote Vehicle Software Version Incompatible"
          } 
        }
        TO_REMOTE {                               ;# copy from local to remote
            set localData [readini $localId $section $entry]
            set vehicleDB [format "vehicle_%s.db" $entry]
            if [file exists ${localDir}${vehicleDB}] {
              writeini $remoteId $section $entry $localData
              lappend locFileList $vehicleDB  
              puts "copied $vehicleDB to list for copying"
            } else {
              puts "file $vehicleDB could not be found on local"
            }
            if {$getUserConfirmation == 1} {
		      if {[searchForNextConflict] == 0} {
		        place forget .s3.conflict
		        FSM:tran manual_accept
		      }
            }
        }
        DEL_LOCAL {
            # The user is not allowed to delete tractors from the local
            # database to eliminate the chance of deleting the current
            # active vehicle.
            #
            #iniparse:removeitem $localId $section $entry
            #puts "deleted $entry from local"
            #if {[searchForNextConflict] == 0} {
            #    place forget .s3.conflict
            #    FSM:tran manual_accept
            #}
            status "Cannot delete local files"
        }
        DEL_REMOTE {
            iniparse:removeitem $remoteId $section $entry
            puts "deleted $entry from remote"
            set vehicleDB [format "vehicle_%s.db" $entry] 
            if [file exists ${remoteDir}${vehicleDB}] {
              lappend remoteDelFileList $vehicleDB  
              puts "copied $vehicleDB to list for deleting"
            } else {
              puts "file $vehicleDB could not be found on remote"
            }
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_accept
            }
        }
        SKIP {
            puts "skipped $entry"
            if {[searchForNextConflict] == 0} {
                place forget .s3.conflict
                FSM:tran manual_accept
            }
        }
        SKIP_ALL {
            place forget .s3.conflict
            FSM:tran manual_accept
        }
    }
}

# manual accept state ........................................................
#
proc manual_accept { sig data } {
    global toolDir localDir remoteDir
    global localId remoteId mode locFileList remoteFileList remoteDelFileList

    switch -- $sig {
        ENTRY {
            .s3.section.title configure -text "Accept changes?"
            place .s3.section -x 0 -y 0
        }
        YES {
            # clean-up the [most recent] section in case some most-recently
            # used objects have been deleted
            writeini $localId "most recent" "basestation" ""
            writeini $localId "most recent" "field" ""
            writeini $localId "most recent" "job" ""
            writeini $localId "most recent" "implement" ""
            writeini $remoteId "most recent" "basestation" ""
            writeini $remoteId "most recent" "field" ""
            writeini $remoteId "most recent" "job" ""
            writeini $remoteId "most recent" "implement" ""
            
            iniparse:closefile $localId
            iniparse:closefile $remoteId
            
            # place the merged database into local and remote locations
            if [catch {exec ${toolDir}ini2db \
			   ${remoteDir}local.ini ${localDir}dbmain.db}] {
		       status "converting databases failed 3"
		       return 1
            }
            if [catch {exec ${toolDir}ini2db \
			   ${remoteDir}remote.ini ${remoteDir}dbmain.db}] {
		       status "converting databases failed 4"
		       return 1
            }
            
            # If there are any files listed to be copied to the local database,
            # copy them.
            foreach fileName $locFileList {
              if [catch {file copy -force ${localDir}${fileName} \
		         ${remoteDir}${fileName}}] {
                 puts "failure to copy $fileName to remote"  
              } else {
                  puts "copied $fileName to remote"  
              }
            } 
            
            # If there are any files listed to be copied to the remote
            # database, copy them.
            foreach fileName $remoteFileList {
              if [catch {file copy -force ${remoteDir}${fileName} \
		          ${localDir}${fileName}}] {
                  puts "failure to copy $fileName to local"  
              } else {
                  puts "copied $fileName to local"  
              }
            }
            
            # If there are any files listed to be deleted from the remote
            # database, delete them.
            foreach fileName $remoteDelFileList {
              set delFileName ${remoteDir}${fileName}
              puts "Trying to delete: $delFileName"
              if [catch {file delete -force $delFileName}] {
                  puts "failure to delete $delFileName from remote"  
              } else {
                  puts "deleted $delFileName from remote"  
              }
            }
            
            status "databases synchronized successfully"
            FSM:tran manual_done
        } 
        NO {
            status "manual database synchronization aborted"
            FSM:tran manual_done
        }
        EXIT {
            # perform the cleanup
            if [catch {file delete -force ${remoteDir}local.ini}] {
		      status "cleaning up after merge failed"
		      return 1
            }
            if [catch {file delete -force ${remoteDir}remote.ini}] {
		      status "cleaning up after merge failed"
		      return 1
            }
            
            # unmount the local and remote file systems
            # ...
        }
    }
}

# manual done state ..........................................................
#
proc manual_done { sig data } {
    global mode
    switch -- $sig {
        ENTRY {
            place forget .s3.section
            place .s3.back -relx 1.0 -x -5 -rely .95 -anchor se
            if {$mode == "AUTO"} {
		.title configure -text "Automatic Merge Complete"
            } else {
		.title configure -text "Manual Merge Complete"
            }
        }
        BACK {
            FSM:tran main_menu
        }
        EXIT {
            place forget .s3
        }
    }
}

##############################################################################
# helper procedures

#.............................................................................
# procedures parsing data records for various objects
#
set R2D 57.29577951
set PI      3.1415926535898
set WGS84_A 6378137.0
set WGS84_E2 6.69437999013e-03

proc SI2units { x u } {
    switch -- $u {
        1 { return [expr $x] }                                  ;# METERS_UNIT
        2 { return [expr $x*100] }                         ;# CENTIMETERS_UNIT
        3 { return [expr $x/(12.0*0.0254)] }                      ;# FEET_UNIT
        4 { return [expr $x/0.0254] }                           ;# INCHES_UNIT
        5 { return [expr $x/1000.0] }                       ;# KILOMETERS_UNIT
        6 { return [expr $x/1609.344] }                          ;# MILES_UNIT
    }
    return 0.0
}    

proc unitsStr { u } {
    switch -- $u {
        1 { return "meters" }
        2 { return "centimeters" }
        3 { return "feet" }
        4 { return "in" }
        5 { return "km" }
        6 { return "miles" }
    }
    return ""
}

proc base_station_format { data } {
    global R2D
    
    scan $data "%f,%f,%f,%f" softwareVersion lla_LAT lla_LON lla_ALT
    set signLat "N"
    if {$lla_LAT < 0.0} {
        set signLat "S"
    }
    set signLon "E"
    if {$lla_LON < 0.0} {
        set signLon "W"
    }
    
    set latDeg [expr abs($lla_LAT)*$R2D]   ;# convert to degrees and drop sign
    set latMin [expr 60.0*($latDeg - floor($latDeg))]
    set latSec [expr 60.0*($latMin - floor($latMin))]
    
    set lonDeg [expr abs($lla_LON)*$R2D]   ;# convert to degrees and drop sign
    set lonMin [expr 60.0*($lonDeg - floor($lonDeg))]
    set lonSec [expr 60.0*($lonMin - floor($lonMin))]

    set lat [format "Lattitude: %3.0f %2.0f' %9.7f'' $signLat" \
		 [expr floor($latDeg)] [expr floor($latMin)] $latSec]
    set lon [format "Longitude: %3.0f %2.0f' %9.7f'' $signLon" \
		 [expr floor($lonDeg)] [expr floor($lonMin)] $lonSec]            
    return "\n\nVer: $softwareVersion\n\n$lat\n\n$lon\n\n[format\
           "Altitude: %7.2f meters" $lla_ALT]"
}

proc base_station_different { data1 data2 } {
    puts "data1=$data1\ndata2=$data2"
    global WGS84_A WGS84_E2
    
    scan $data1 "%f,%f,%f,%f" softwareVersion lat lon alt
    set sinlat [expr sin($lat)]
    set rn [expr $WGS84_A/sqrt(1.0 - $WGS84_E2*$sinlat*$sinlat)]
    set ecefZ1 [expr ($rn*(1.0 - $WGS84_E2) + $alt)*$sinlat]
    set rn [expr ($rn + $alt)*cos($lat)]
    set ecefX1 [expr $rn*cos($lon)]
    set ecefY1 [expr $rn*sin($lon)]

    scan $data2 "%f,%f,%f,%f" softwareVersion lat lon alt
    set sinlat [expr sin($lat)]
    set rn [expr $WGS84_A/sqrt(1.0 - $WGS84_E2*$sinlat*$sinlat)]
    set ecefZ2 [expr ($rn*(1.0 - $WGS84_E2) + $alt)*$sinlat]
    set rn [expr ($rn + $alt)*cos($lat)]
    set ecefX2 [expr $rn*cos($lon)]
    set ecefY2 [expr $rn*sin($lon)]

    set dx [expr $ecefX1 - $ecefX2]
    set dy [expr $ecefY1 - $ecefY2]
    set dz [expr $ecefZ1 - $ecefZ2]
    
    if {($dx*$dx + $dy*$dy + $dz*$dz) < 1.0*1.0} {
	return 0                                 ;# base stations are identical
    }   
    return 1                                    ;# base stations are different
}

proc field_format { data } {
    scan $data "%f,%d" softwareVersion nPoints

    if {$nPoints > 0} {
	return [format "\n\nVer: $softwareVersion\n\nNo. of Boundary Points:\
                     %3d " $nPoints]
    }
    return "\n\nVer: $softwareVersion \n\nNo. of Boundary Points: NONE"
}

proc field_different { data1 data2 } {
    puts "data1=$data1\ndata2=$data2"
    scan $data1 "%f,%d" softwareVersion1 nPoints1
    scan $data2 "%f,%d" softwareVersion2 nPoints2
    if {$nPoints1 == $nPoints2} {
        return 0                                           ;# fields identical
    }
    return 1
}

proc job_format { data } {
    puts "data=$data"
    global R2D
    
    scan $data "%f,%d,%lf,%d" softwareVersion t largeNudge u
    set type "UNKNOWN"
    switch -- $t {
	0 {  
	    scan $data "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf,%lf" \
		softwareVersion t largeNudge u \
		r_mark_B__B2_E r_mark_B__B2_N r_mark_B__B2_U \
		flags repeatSourceName \
		r_Arow_B__B2_NN r_Arow_B__B2_EE r_Arow_B__B2_DD rowDir
	    
	    if {$repeatSourceName == "!"} {
		set repeatSourceName "Master Job"
	    } else {
		set repeatSourceName "Repeat of $repeatSourceName"
	    }  
	    
	    return [format "\nVer: $softwareVersion \
                          \n\nType: STRAIGHT ROWS \
                          \n\nLarge Nudge: %8.2f [unitsStr $u] \
                          \n\nCode: %x \
                          \n\n$repeatSourceName \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260" \
			[SI2units $largeNudge $u] \
			$flags \
			$r_Arow_B__B2_NN $r_Arow_B__B2_EE $r_Arow_B__B2_DD \
			[expr $rowDir/$R2D]]
	}
	1 { 
	    scan $data "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%d,%d,%lf,%lf,%lf,%lf \
          ,%lf,%lf,%lf,%lf"   \
		softwareVersion t largeNudge u \
		r_mark_B__B2_E r_mark_B__B2_N r_mark_B__B2_U \
		flags repeatSourceName numRows spacingUnits \
		r_Arow_B__B2_NN r_Arow_B__B2_EE r_Arow_B__B2_DD rowDir \
		r_Arow_B__B2_NN_2 r_Arow_B__B2_EE_2 r_Arow_B__B2_DD_2 rowDir_2
	    
	    if {$repeatSourceName == "!"} {
		set repeatSourceName "Master Job"
	    } else {
		set repeatSourceName "Repeat of: $repeatSourceName"
	    }  
	    
	    return [format "\nVer: $softwareVersion \
                          \n\nType: FAN ROWS \
                          \n\nLarge Nudge: %8.2f [unitsStr $u] \
                          \n\nCode: %x \
                          \n\n$repeatSourceName \
                          \n\nRow 1 ------  \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260 \
                          \n\nRow 2 ------ \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260" \
			[SI2units $largeNudge $u] \
			$flags \
			$r_Arow_B__B2_NN $r_Arow_B__B2_EE $r_Arow_B__B2_DD \
			[expr $rowDir/$R2D] \
			$r_Arow_B__B2_NN_2 $r_Arow_B__B2_EE_2 \
			$r_Arow_B__B2_DD_2 \
			[expr $rowDir_2/$R2D]]
	}
	2 { scan $data "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%d,%lf,%lf,%lf,%lf \
          ,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf"   \
		softwareVersion t largeNudge u \
		r_mark_B__B2_E r_mark_B__B2_N r_mark_B__B2_U \
		flags repeatSourceName numRows \
		r_Arow_B__B2_NN r_Arow_B__B2_EE r_Arow_B__B2_DD rowDir \
		r_Arow_B__B2_NN_2 r_Arow_B__B2_EE_2 r_Arow_B__B2_DD_2 rowDir_2 \
		r_Arow_B__B2_NN_3 r_Arow_B__B2_EE_3 r_Arow_B__B2_DD_3 rowDir_3 \
		r_Arow_B__B2_NN_4 r_Arow_B__B2_EE_4 r_Arow_B__B2_DD_4 rowDir_4
	    
	    if {$repeatSourceName == "!"} {
		set repeatSourceName "Master Job"
	    } else {
		set repeatSourceName "Repeat of: $repeatSourceName"
	    }  
	    
	    return [format "\nVer: $softwareVersion \
                          \n\nType: BOX ROWS \
                          \n\nLarge Nudge: %8.2f [unitsStr $u] \
                          \n\nCode: %x \
                          \n\n$repeatSourceName \
                          \n\nRow 1 ------ \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260 \
                          \n\nRow 2 ------ \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260 \
                          \n\nRow 3 ------ 
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260 \
                          \n\nRow 4 ------
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260" \
			[SI2units $largeNudge $u] \
			$flags \
			$r_Arow_B__B2_NN $r_Arow_B__B2_EE \
			$r_Arow_B__B2_DD \
			[expr $rowDir/$R2D] \
			$r_Arow_B__B2_NN_2 $r_Arow_B__B2_EE_2 \
			$r_Arow_B__B2_DD_2 \
			[expr $rowDir_2/$R2D] \
			$r_Arow_B__B2_NN_3 $r_Arow_B__B2_EE_3 \
			$r_Arow_B__B2_DD_3 \
			[expr $rowDir_3/$R2D] \
			$r_Arow_B__B2_NN_4 $r_Arow_B__B2_EE_4 \
			$r_Arow_B__B2_DD_4 \
			[expr $rowDir_4/$R2D]]
	}
	3 { scan $data "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf" \
		softwareVersion t largeNudge u \
		r_mark_B__B2_E r_mark_B__B2_N r_mark_B__B2_U \
		flags repeatSourceName \
		centerNorth_ centerEast_ definingRadius_
	    
	    if {$repeatSourceName == "!"} {
		set repeatSourceName "Master Job"
	    } else {
		set repeatSourceName "Repeat of: $repeatSourceName"
	    }  
	    
	    return [format "\nVer: $softwareVersion \
                          \n\nType: CIRCLE ROWS \
                          \n\nLarge Nudge: %8.2f [unitsStr $u] \
                          \n\nCode: %x \
                          \n\n$repeatSourceName \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nRadius:  %8.2f m" \
			[SI2units $largeNudge $u] \
			$flags \
			$centerNorth_ $centerEast_ $definingRadius_]
	}
	4 { scan $data "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf,%lf,%lf, \
                     %lf,%d" \
		softwareVersion t largeNudge u \
		r_mark_B__B2_E r_mark_B__B2_N r_mark_B__B2_U \
		flags repeatSourceName \
		r_Arow_B__B2_NN r_Arow_B__B2_EE r_Arow_B__B2_DD rowDir \
		propDir_ propDist_ propDistUnits_
	    
	    if {$repeatSourceName == "!"} {
		set repeatSourceName "Master Job"
	    } else {
		set repeatSourceName "Repeat of: $repeatSourceName"
	    }  
	    
	    return [format "\nVer: $softwareVersion \
                          \n\nType: SIMPLE CURVE ROWS \
                          \n\nLarge Nudge: %8.2f [unitsStr $u] \
                          \n\nCode: %X \
                          \n\n$repeatSourceName \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260" \
			[SI2units $largeNudge $u] \
			$flags \
			$r_Arow_B__B2_NN $r_Arow_B__B2_EE $r_Arow_B__B2_DD \
			[expr $rowDir/$R2D]]

	}
	5 { scan $data "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf,%lf,%lf, \
                      %lf,%d" \
		softwareVersion t largeNudge u \
		r_mark_B__B2_E r_mark_B__B2_N r_mark_B__B2_U \
		flags repeatSourceName \
		r_Arow_B__B2_NN r_Arow_B__B2_EE r_Arow_B__B2_DD rowDir\
		propDir_ propDist_ propDistUnits_
	    
	    if {$repeatSourceName == "!"} {
		set repeatSourceName "Master Job"
	    } else {
		set repeatSourceName "Repeat of: $repeatSourceName"
	    }  
	    
	    return [format "\nVer: $softwareVersion \
                          \n\nType: CURVE ROWS \
                          \n\nLarge Nudge: %8.2f [unitsStr $u] \
                          \n\nCode: %X \
                          \n\n$repeatSourceName \
                          \n\nNorth: %8.2f m\
                          \n\nEast:  %8.2f m\
                          \n\nDown:  %8.2f m\
                          \n\nDir: %8.5f\260" \
			[SI2units $largeNudge $u] \
			$flags \
			$r_Arow_B__B2_NN $r_Arow_B__B2_EE $r_Arow_B__B2_DD \
			[expr $rowDir/$R2D]]
	}
    }
    
}

proc job_different { data1 data2 } {
    puts "data1=$data1\ndata2=$data2"

    if {$data1 == $data2} {
        return 0                                     ;# the jobs are identical 
    }

    scan $data1 "%f,%d" softwareVersion1 t1
    switch -- $t1 {
        0 { 
            scan $data1 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf,%lf" \
                softwareVersion1 t1 largeNudge1 u1 \
                r_mark_B__B2_E1 r_mark_B__B2_N1 r_mark_B__B2_U1 \
                flags1 repeatSourceName1 \
                r_Arow_B__B2_NN1 r_Arow_B__B2_EE1 r_Arow_B__B2_DD1 rowDir1
        }
        1 { scan $data1 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%d,%d,%lf,%lf,%lf,%lf \
                       ,%lf,%lf,%lf,%lf"   \
		softwareVersion1 t1 largeNudge1 u1 \
		r_mark_B__B2_E1 r_mark_B__B2_N1 r_mark_B__B2_U1 \
		flags1 repeatSourceName1 numRows1 spacingUnits1 \
		r_Arow_B__B2_NN1 r_Arow_B__B2_EE1 r_Arow_B__B2_DD1 rowDir1 \
		r_Arow_B__B2_NN_21 r_Arow_B__B2_EE_21 r_Arow_B__B2_DD_21 rowDir_21    
        }
        2 { scan $data1 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%d,%lf,%lf,%lf,%lf \
                       ,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf"   \
		softwareVersion1 t1 largeNudge1 u1 \
		r_mark_B__B2_E1 r_mark_B__B2_N1 r_mark_B__B2_U1 \
		flags1 repeatSourceName1 numRows1 \
		r_Arow_B__B2_NN1 r_Arow_B__B2_EE1 r_Arow_B__B2_DD1 rowDir1 \
		r_Arow_B__B2_NN_21 r_Arow_B__B2_EE_21 r_Arow_B__B2_DD_21 \
		rowDir_21 \
		r_Arow_B__B2_NN_31 r_Arow_B__B2_EE_31 r_Arow_B__B2_DD_31 \
		rowDir_31 \
		r_Arow_B__B2_NN_41 r_Arow_B__B2_EE_41 r_Arow_B__B2_DD_41 \
		rowDir_41 
        }
        3 { scan $data1 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf" \
		softwareVersion1 t1 largeNudge1 u1 \
		r_mark_B__B2_E1 r_mark_B__B2_N1 r_mark_B__B2_U1 \
		flags1 repeatSourceName1 \
		centerNorth_1 centerEast_1 definingRadius_1
        }
         4 { scan $data1 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf,%lf" \
		softwareVersion1 t1 largeNudge1 u1 \
		r_mark_B__B2_E1 r_mark_B__B2_N1 r_mark_B__B2_U1 \
		flags1 repeatSourceName1 \
		r_Arow_B__B2_NN1 r_Arow_B__B2_EE1 r_Arow_B__B2_DD1 rowDir1 \
	    }
        5 { 
            scan $data1 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf,%lf" \
		softwareVersion1 t1 largeNudge1 u1 \
		r_mark_B__B2_E1 r_mark_B__B2_N1 r_mark_B__B2_U1 \
		flags1 repeatSourceName1 \
		r_Arow_B__B2_NN1 r_Arow_B__B2_EE1 r_Arow_B__B2_DD1 rowDir1
        }
    }            

    scan $data2 "%f,%d" softwareVersion2 t2
    if {$t1 == $t2} {
        switch -- $t2 {
            0 {
                scan $data2 \
		    "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf,%lf" \
                    softwareVersion2 t2 largeNudge2 u2 \
                    r_mark_B__B2_E2 r_mark_B__B2_N2 r_mark_B__B2_U2 \
                    flags2 repeatSourceName2 \
                    r_Arow_B__B2_NN2 r_Arow_B__B2_EE2 r_Arow_B__B2_DD2 rowDir2
		
                if {$largeNudge1 == $largeNudge2 && \
			$repeatSourceName1 == $repeatSourceName2 && \
			$r_Arow_B__B2_NN1 == $r_Arow_B__B2_NN2 && \
			$r_Arow_B__B2_EE1 == $r_Arow_B__B2_EE2 && \
			$r_Arow_B__B2_DD1 == $r_Arow_B__B2_DD2 && \
			$rowDir1 == $rowDir2} {
		    return 0
                }
            }
            1 { scan $data2 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%d,%d,%lf,%lf \
                            ,%lf,%lf,%lf,%lf,%lf,%lf"   \
		    softwareVersion2 t2 largeNudge2 u2 \
		    r_mark_B__B2_E2 r_mark_B__B2_N2 r_mark_B__B2_U2 \
		    flags2 repeatSourceName2 numRows2 spacingUnits2 \
		    r_Arow_B__B2_NN2 r_Arow_B__B2_EE2 r_Arow_B__B2_DD2 rowDir2 \
		    r_Arow_B__B2_NN_22 r_Arow_B__B2_EE_22 r_Arow_B__B2_DD_22 \
		    rowDir_22
		
                if {$largeNudge1 == $largeNudge2 && \
			$repeatSourceName1 == $repeatSourceName2 && \
			$numRows1 == $numRows2 && \
			$spacingUnits1 == $spacingUnits2 && \
			$r_Arow_B__B2_NN1 == $r_Arow_B__B2_NN2 && \
			$r_Arow_B__B2_EE1 == $r_Arow_B__B2_EE2 && \
			$r_Arow_B__B2_DD1 == $r_Arow_B__B2_DD2 && \
			$rowDir1 == $rowDir2 && \
			$r_Arow_B__B2_NN_21 == $r_Arow_B__B2_NN_22 && \
			$r_Arow_B__B2_EE_21 == $r_Arow_B__B2_EE_22 && \
			$r_Arow_B__B2_DD_21 == $r_Arow_B__B2_DD_22 && \
			$rowDir_21 == $rowDir_22 } {
		    return 0
                }      
            }
            2 { scan $data2 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%d,%lf,%lf,%lf \
                            ,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf \
                            ,%lf"   \
		    softwareVersion2 t2 largeNudge2 u2 \
		    r_mark_B__B2_E2 r_mark_B__B2_N2 r_mark_B__B2_U2 \
		    flags2 repeatSourceName2 numRows2 \
		    r_Arow_B__B2_NN2 r_Arow_B__B2_EE2 r_Arow_B__B2_DD2 rowDir2 \
		    r_Arow_B__B2_NN_22 r_Arow_B__B2_EE_22 r_Arow_B__B2_DD_22 \
		    rowDir_22 \
		    r_Arow_B__B2_NN_32 r_Arow_B__B2_EE_32 r_Arow_B__B2_DD_32 \
		    rowDir_32 \
		    r_Arow_B__B2_NN_42 r_Arow_B__B2_EE_42 r_Arow_B__B2_DD_42 \
		    rowDir_42
                
                if {$largeNudge1 == $largeNudge2 && \
			$repeatSourceName1 == $repeatSourceName2 && \
			$numRows1 == $numRows2 && \
			$r_Arow_B__B2_NN1 == $r_Arow_B__B2_NN2 && \
			$r_Arow_B__B2_EE1 == $r_Arow_B__B2_EE2 && \
			$r_Arow_B__B2_DD1 == $r_Arow_B__B2_DD2 && \
			$rowDir1 == $rowDir2 && \
			$r_Arow_B__B2_NN_21 == $r_Arow_B__B2_NN_22 && \
			$r_Arow_B__B2_EE_21 == $r_Arow_B__B2_EE_22 && \
			$r_Arow_B__B2_DD_21 == $r_Arow_B__B2_DD_22 && \
			$rowDir_21 == $rowDir_22 && \
			$r_Arow_B__B2_NN_31 == $r_Arow_B__B2_NN_32 && \
			$r_Arow_B__B2_EE_31 == $r_Arow_B__B2_EE_32 && \
			$r_Arow_B__B2_DD_31 == $r_Arow_B__B2_DD_32 && \
			$rowDir_31 == $rowDir_32 && \
			$r_Arow_B__B2_NN_41 == $r_Arow_B__B2_NN_42 && \
			$r_Arow_B__B2_EE_41 == $r_Arow_B__B2_EE_42 && \
			$r_Arow_B__B2_DD_41 == $r_Arow_B__B2_DD_42 && \
			$rowDir_41 == $rowDir_42 } {
		    return 0
                }     
            }
            3 { scan $data2 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf" \
		    softwareVersion2 t2 largeNudge2 u2 \
		    r_mark_B__B2_E2 r_mark_B__B2_N2 r_mark_B__B2_U2 \
		    flags2 repeatSourceName2 \
		    centerNorth_2 centerEast_2 definingRadius_2
                
                if {$largeNudge1 == $largeNudge2 && \
			$repeatSourceName1 == $repeatSourceName2 && \
			$centerNorth_1 == $centerNorth_2 && \
			$centerEast_1 == $centerEast_2 && \
			$definingRadius_1 == $definingRadius_2 } {
		    return 0 
                }
            }
            4 { scan $data2 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf, \
                             %lf" \
		    softwareVersion2 t2 largeNudge2 u2 \
		    r_mark_B__B2_E2 r_mark_B__B2_N2 r_mark_B__B2_U2 \
		    flags2 repeatSourceName2 \
		    r_Arow_B__B2_NN2 r_Arow_B__B2_EE2 r_Arow_B__B2_DD2 rowDir2
                
                if {$largeNudge1 == $largeNudge2 && \
			$repeatSourceName1 == $repeatSourceName2 && \
			$r_Arow_B__B2_NN1 == $r_Arow_B__B2_NN2 && \
			$r_Arow_B__B2_EE1 == $r_Arow_B__B2_EE2 && \
			$r_Arow_B__B2_DD1 == $r_Arow_B__B2_DD2 && \
			$rowDir1 == $rowDir2 } {
		    return 0
                }      
            }
            5 {
                 scan $data2 "%f,%d,%lf,%d,%lf,%lf,%lf,%d, %s ,%lf,%lf,%lf, \
                             %lf" \
		    softwareVersion2 t2 largeNudge2 u2 \
		    r_mark_B__B2_E2 r_mark_B__B2_N2 r_mark_B__B2_U2 \
		    flags2 repeatSourceName2 \
		    r_Arow_B__B2_NN2 r_Arow_B__B2_EE2 r_Arow_B__B2_DD2 rowDir2
                
                if {$largeNudge1 == $largeNudge2 && \
			$repeatSourceName1 == $repeatSourceName2 && \
			$r_Arow_B__B2_NN1 == $r_Arow_B__B2_NN2 && \
			$r_Arow_B__B2_EE1 == $r_Arow_B__B2_EE2 && \
			$r_Arow_B__B2_DD1 == $r_Arow_B__B2_DD2 && \
			$rowDir1 == $rowDir2 } {
		    return 0
                }
            }
        }            
    }

    return 1                                    ;# jobs are probably different
}

proc implement_format { data } {
    puts "implement data: $data"
    scan $data "%f,%d,%lf,%lf,%lf,%lf,%d,%d" \
	softwareVersion t width r_I_C__A_X r_I_C__A_Y r_I_C__A_Z u_w u_o
    set type "UNKNOWN"
    switch -- $t {
        0 { set type "HITCHED"}
        1 { set type "TOWED"}
    }
    if {$r_I_C__A_Y == 0} {
	set r_I_C__A_Y "NONE"
	return [format "\n\nVer: $softwareVersion \
                    \n\nType: $type \
                    \n\nWidth:%8.2f %s \
                    \n\nOffset: NONE" \
                    [SI2units $width $u_w] [unitsStr $u_w]] 
    }
    
    if {$r_I_C__A_Y < 0} {
	set offsetDir "to left"
    } else {
	set offsetDir "to right"
    }           
    set offset 0
    
    return [format "\n\nVer: $softwareVersion \
                    \n\nType: $type \
                    \n\nWidth:%8.2f %s \
                    \n\nOffset:%8.2f %s $offsetDir" \
		[SI2units $width $u_w] [unitsStr $u_w] \
		[SI2units [expr abs($r_I_C__A_Y)] $u_o] [unitsStr $u_o]]             
}

proc implement_different { data1 data2 } {
    puts "data1=$data1\ndata2=$data2"
    scan $data1 "%f,%d,%lf,%lf,%lf,%lf,%d,%d" \
        softwareVersion1 \
        type1 width1 r_I_C__A_X1 r_I_C__A_Y1 r_I_C__A_Z1 u_w1 u_o1

    scan $data2 "%f,%d,%lf,%lf,%lf,%lf,%d,%d" \
        softwareVersion2 \
        type2 width2 r_I_C__A_X2 r_I_C__A_Y2 r_I_C__A_Z2 u_w2 u_o2
    
    if {$type1 == $type2 && \
	    [expr abs($width1 - $width2)] < 0.001 && \
	    [expr abs($r_I_C__A_Y1 - $r_I_C__A_Y2)] < 0.001} {
        
        return 0                                    ;# implements are the same
    }
    return 1                                       ;# implements are different 
}

proc tractors_format { data } {
  puts "tractor data: $data"
  scan $data "%f,%d,%d,%d" softwareVersion type subType color
    
  set typeStr    "None"
  set subTypeStr "None"
  set colorStr   "None"
  
  switch -- $type { 
    0 { set typeStr "Deere Track"}
    1 { set typeStr "Wheeled"
        switch -- $subType {
          0 { set subTypeStr "Standard" }
          1 { set subTypeStr "Pusher" }
          2 { set subTypeStr "Articulated" }
          3 { set subTypeStr "STX Quadtrac" }
          4 { set subTypeStr "Case Floater" }
          5 { set subTypeStr "Deere 4710 Sprayer" }
          6 { set subTypeStr "Agrosky" }
          7 { set subTypeStr "AutoTrac Wheeled" }
          8 { set subTypeStr "AutoTrac Articulated" }
          9 { set subTypeStr "AutoTrac Sprayer" }
         10 { set subTypeStr "Combine" }
          default { set subTypeStr "Unknown" }
        }
      }
    2 { set typeStr "Cat Track" }
    3 { set typeStr "MT" }
    4 { set typeStr "Gantry" }
    5 { set typeStr "Cameco Harvester" }
    6 { set typeStr "Unknown" }
  }
    
  switch -- $color {
    0 { set colorStr "BLUE" }
    1 { set colorStr "GREEN" }
    2 { set colorStr "ORANGE" }
    3 { set colorStr "RED" }
    4 { set colorStr "YELLOW" }
  }
  
  if {$softwareVersion >= 5.003} {
    scan $data "%f,%d,%d,%d" softwareVersion type subType color
    return [format "\n\nVer    : $softwareVersion \
                    \n\nType   : $typeStr \
                    \n\nSubType: $subTypeStr \
                    \n\nColor  : $colorStr"] 
  }
  
  if {$softwareVersion >= 5.005} {
    scan $data "%f,%d,%d,%d,%d,%1d,%1d" \
    softwareVersion type subType color strInterface \
    SAMSerialNum RMSerialNum
    
    return [format "\n\nVer: $softwareVersion \
                   \n\nType: $typeStr \
                   \n\nSubType: $subTypeStr \
                   \n\ncolor: $colorStr \
                   \n\nSteer Interface: $strInterface \
                   \n\nSAM Serial Number: $SAMSerialNum \
                   \n\nRM Serial Number: $RMSerialNum"]
  }   
}

proc tractors_different { data1 data2 } {
  puts "data1=$data1\ndata2=$data2"
  
  if {$data1 == $data2} {
    return 0                                          ;# tractors are the same
  }  
  
  scan $data1 "%f" softwareVersion1
  scan $data2 "%f" softwareVersion2
  
  # Only tractors having the same software version will be considered
  # for syncing.
  if {$softwareVersion1 != $softwareVersion2} {
    return 0
  }
  
  if {$softwareVersion1 >= 5.003 && $softwareVersion1 < 5.005} {
    #Parse data1
    scan $data1 "%f,%d,%d,%d" \
      softwareVersion1 type1 subType1 color1
  
    #Parse data2
    scan $data2 "%f,%d,%d,%d" \
      softwareVersion2 type2 subType2 color2
    
    if {$softwareVersion1 == $softwareVersion2 && \
        $type1 == $type2 && \
        $subType1 == $subType2 && \
        $color1 == $color2} {
        return 0                                      ;# tractors are the same
    }
  }
  
  if {$softwareVersion1 >= 5.005} {
    #Parse data1
    scan $data1 "%f,%d,%d,%d,%d,%1d,%1d" \
      softwareVersion1 type1 subType1 color1 strInterface1 \
      SAMSerialNum1 RMSerialNum1
  
    #Parse data2
    scan $data2 "%f,%d,%d,%d,%d,%1d,%1d" \
      softwareVersion2 type2 subType2 color2 strInterface2 \
      SAMSerialNum2 RMSerialNum2
        
    if {$softwareVersion1 == $softwareVersion2 && \
        $type1 == $type2 && \
        $subType1 == $subType2 && \
        $color1 == $color2 && \
        $strInterface1 == $strInterface2 && \
        $SAMSerialNum1 == $SAMSerialNum2 && \
        $RMSerialNum1 == $RMSerialNum2} {
        return 0                                      ;# tractors are the same
    }
  }
  return 1                                           ;# tractors are different  
}

proc purge { iniId section entry } {
    set lst [readini $iniId $section]
    foreach e $lst {
        if [string match "${entry}*" $e] {                  ;# prefix matched?
            iniparse:removeitem $iniId $section $e
            puts "deleted $e from $section"
        }
    }
}


##############################################################################
# action procedures

#.............................................................................
# set up conflict dialog when both databases have a given entry, but with
# different data
#
proc conflictBoth { section entry remote local } {
    .title configure -text "Synchronizing $section"
    .s3.conflict.title configure -text \
        "Conflicting versions of $entry found:"
    .s3.conflict.localTxt delete 1.0 end
    .s3.conflict.remoteTxt delete 1.0 end
    .s3.conflict.remoteTxt insert 1.0 $remote   
    .s3.conflict.localTxt insert 1.0 $local
    .s3.conflict.toLocal configure -state normal  
    .s3.conflict.toRemote configure -state normal  
    place .s3.conflict.toLocal -relx 0.501 -y 95 -anchor n
    place .s3.conflict.toRemote -relx 0.501 -y 170 -anchor n
    place forget .s3.conflict.delLocal
    place forget .s3.conflict.delRemote
}

proc conflictRemote { section entry remote } {
    .title configure -text "Synchronizing $section"
    .s3.conflict.title configure -text \
        "$entry exists only on your Sync Disk"
    .s3.conflict.localTxt delete 1.0 end
    .s3.conflict.remoteTxt delete 1.0 end
    .s3.conflict.remoteTxt insert 1.0 $remote  
    .s3.conflict.localTxt insert 1.0 "\n\n    (Not Found)"  
    place forget .s3.conflict.toRemote
    place .s3.conflict.toLocal -relx 0.501 -y 95 -anchor n
    place forget .s3.conflict.delLocal
    place .s3.conflict.delRemote -relx 0.501 -y 170 -anchor n
}

proc conflictLocal { section entry local } {
    .title configure -text "Synchronizing $section"
    .s3.conflict.title configure -text \
        "$entry exists only on your GPS System"
    .s3.conflict.localTxt delete 1.0 end
    .s3.conflict.remoteTxt delete 1.0 end    
    .s3.conflict.remoteTxt insert 1.0 "\n\n    (Not Found)"    
    .s3.conflict.localTxt insert 1.0 $local   
    place .s3.conflict.toRemote -relx 0.501 -y 95 -anchor n
    place forget .s3.conflict.toLocal
    place .s3.conflict.delLocal -relx 0.501 -y 170 -anchor n
    place forget .s3.conflict.delRemote
}

proc syncSize { localId remoteId section } {
    set max1 [readini $localId $section "!size"] 
    set max2 [readini $remoteId $section "!size"] 
    set max 0
    if {$max1 != ""} {
        set max $max1
    }
    if {$max2 != ""} {
        if {$max2 > $max} {
            set max $max2
        }
    }
    if {$max > 0} {
        writeini $localId $section "!size" $max
        writeini $remoteId $section "!size" $max
        puts "synchronized size of $section to $max"  
    }
}

#.............................................................................
# manual merge action procedure
#
proc manual_init {} {
    global toolDir localDir remoteDir
    
    # mount the local and remote file systems
    # ...
    
    
    # prepare the local and remote databases in local directory
    if [file exists ${remoteDir}dbmain.db] { 
        if [catch {exec ${toolDir}db2ini \
                       ${localDir}dbmain.db ${remoteDir}local.ini}] {
            status "converting databases failed 1"
            return 1
        }
    } else {;# remote file doesn't exist, just copy local file and we're done!
        if [catch {file copy -force ${localDir}dbmain.db \
		       ${remoteDir}dbmain.db}] {

		    status "merging databases failed ${remoteDir} ${localDir}"
            return 1
        }
        
        # Copy all vehicle_*.db files to the syncdisk
        if [catch {glob ${localDir}vehicle_*.db} fileList] {
          status "Error reading local disk!"
        } else {
            foreach fileName $fileList {
              if [catch {file copy -force ${localDir}[file tail $fileName] \
		         ${remoteDir}[file tail $fileName]}] {
                 puts "failure to copy $fileName to remote"  
              } else {
                  puts "copied $fileName to remote"  
              }
            }
        }
        
        
        # unmount the local and remote file systems
        # ...
 
        
        status "databases synchronized successfully"
	
        return 1
    }    
    
    # convert remote database to remote.ini file
    if [catch {exec ${toolDir}db2ini \
		   ${remoteDir}dbmain.db ${remoteDir}remote.ini}] {
        status "converting databases failed 2"
        return 1
    }

    # open and parse the local file
    global localId
    set localId [iniparse:openfile ${remoteDir}local.ini]
    if {$localId == -1} {
        status "file ${remoteDir}local.ini open failed"
        return 1
    } else {
        status "file ${remoteDir}local.ini parsed successfully"
    }

    # open and parse the remote file
    global remoteId
    set remoteId [iniparse:openfile ${remoteDir}remote.ini]
    if {$remoteId == -1} {
        status "file ${remoteDir}remote.ini open failed"
        return 1
    } else {
        status "file ${remoteDir}remote.ini parsed successfully"
    }
    
    # local and remote database files converted and parsed 
    return 0
}

#.............................................................................
# scan entries in a given section in search for conflicts  
#    returns 1 if next conflict found
#    returns 0 if no more conflicts found
#
proc searchForNextConflict { } {
    global localId remoteId localList remoteList localIdx remoteIdx 
    global section entry objFormat objDifferent mode getUserConfirmation

    set getUserConfirmation 0

    if {$localIdx == -2} {            ;# have we started processing local yet?
        incr remoteIdx   ;# advance remote index to the next entry in the list
        for {} {$remoteIdx < [llength $remoteList]} {incr remoteIdx} {
            set entry [lindex $remoteList $remoteIdx]
            if {$entry == "!size"} continue      ;# !size already synchronized
            set matchIndex [lsearch -exact $localList $entry]
            set remoteData [readini $remoteId $section $entry]
            if {$matchIndex == -1} {        ;# remote entry not found in Local
                if {$mode == "AUTO"} {
		    set getUserConfirmation 0
		    FSM:dispatch TO_LOCAL
                } else {  
		    set getUserConfirmation 1
                    conflictRemote $section $entry \
			[eval {$objFormat $remoteData}]
                    return 1
                }                   
            } else {                       ;# remote entry exists in the local
                set localData [readini $localId $section $entry]
		        set getUserConfirmation 1
                if [eval {$objDifferent $remoteData $localData}] {
                    conflictBoth $section $entry \
			[eval {$objFormat $remoteData}] \
			[eval {$objFormat $localData}]
                    return 1
                }
            }
        }
        set localIdx -1         ;# indicate start processing of the local list
    } 
    
    if {$localIdx >= -1} {
        incr localIdx    ;# advance remote index to the next entry in the list
        for {} {$localIdx < [llength $localList]} {incr localIdx} {
            set entry [lindex $localList $localIdx]
            if {$entry == "!size"} continue      ;# !size already synchronized
            set matchIndex [lsearch -exact $remoteList $entry]
            if {$matchIndex == -1} {        ;# local entry not found in remote
                set localData [readini $localId $section $entry]
                if {$mode == "AUTO"} {
		    set getUserConfirmation 0
		    FSM:dispatch TO_REMOTE
                } else {
		    set getUserConfirmation 1
            conflictLocal $section $entry \
			[eval {$objFormat $localData}]
                    return 1
                }                  
            }
        }       
    }
    return 0
}

#=============================================================================
# automatic merge procedure
#
proc autoSynch {} {
    global toolDir localDir remoteDir
    global objFormat objDifferent
    # mount the local and remote file systems
    # ...
    
    
    # prepare the local and remote databases in local directory
    if [file exists ${remoteDir}dbmain.db] { 
	if [catch {exec ${toolDir}db2ini \
                       ${localDir}dbmain.db ${remoteDir}local.ini}] {
	    status "converting databases failed 1"
	    return 1
        }
    } else {;# remote file doesn't exist, just copy local file and we're done!
        if [catch {file copy -force ${localDir}dbmain.db \
		       ${remoteDir}dbmain.db}] {
	    status "merging databases failed"
	    return 1
        }
        # unmount the local and remote file systems
        # ...

        status "databases synchronized successfully"
	
        return 0
    }    
    
    if [catch {exec ${toolDir}db2ini \
		   ${remoteDir}dbmain.db ${remoteDir}remote.ini}] {
        status "converting databases failed 2"
        return 1
    }

    # open and parse the local file
    set localId [iniparse:openfile ${remoteDir}local.ini]
    if {$localId == -1} {
        status "file ${remoteDir}local.ini open failed"
        return 1
    } else {
        status "file ${remoteDir}local.ini parsed successfully"
    }

    # open and parse the remote file
    set remoteId [iniparse:openfile ${remoteDir}remote.ini]
    if {$remoteId == -1} {
        status "file ${remoteDir}remote.ini open failed"
        return 1
    } else {
        status "file ${remoteDir}remote.ini parsed successfully"
    }
    
    set objFormat base_station_format
    set objDifferent base_station_different   
    autoSyncSection $localId $remoteId "base stations"
    #autoSyncSection $localId $remoteId "fields"
    #autoSyncSection $localId $remoteId "jobs"
    #autoSyncSection $localId $remoteId "implements"

    iniparse:closefile $localId
    iniparse:closefile $remoteId

    # place the merged database into local and remote locations
    if [catch {exec ${toolDir}ini2db \
		   ${remoteDir}local.ini ${localDir}dbmain.db}] {
        status "converting databases failed 3"
        return 1
    }
    if [catch {exec ${toolDir}ini2db \
		   ${remoteDir}remote.ini ${remoteDir}dbmain.db}] {
        status "converting databases failed 4"
        return 1
    }
    
    # perform the cleanup
    if [catch {file delete -force ${remoteDir}local.ini}] {
        status "cleaning up after merge failed"
        return 1
    }
    if [catch {file delete -force ${remoteDir}remote.ini}] {
        status "cleaning up after merge failed"
        return 1
    }
    
    # unmount the local and remote file systems
    # ...

    status "databases synchronized successfully"
    
    return 0
}

#=============================================================================
# Automatically syncs the local and remote INI files
#
proc autoSyncSection { localId remoteId section } {
    global objFormat objDifferent entry
    
    set localList [readini $localId $section]
    set remoteList [readini $remoteId $section]

    syncSize $localId $remoteId $section         ;# synchronize the size first

    # special treatment of [field] section
    
    # scan thru each entry in Remote List
    foreach entry $remoteList {
        if {$entry == "!size"} continue          ;# !size already synchronized
        set matchIndex [lsearch -exact $localList $entry]
        if {$matchIndex == -1} {                   ;# entry not found in Local
            set remoteData [readini $remoteId $section $entry]
            writeini $localId $section $entry $remoteData
        } else {                                  ;# entry exists in the local
	    puts $entry
	    set localData [readini $localId $section $entry]
	    set remoteData [readini $remoteId $section $entry]

	    puts $localData
	    puts $remoteData
	    if [eval {$objDifferent $remoteData $localData}] {
		conflictBoth $section $entry \
		    [eval {$objFormat $remoteData}] \
		    [eval {$objFormat $localData}]
		place forget .s3.section
		place .s3.conflict -x 0 -y 0
		FSM:tran manual_base_conflict        
	    }
	    #writeini $remoteId $section $entry $localData
	    
        }
    }
    
    # scan thru each entry in Local List and look for entries that are 
    # present only in the local list (the ones present in both have been
    # handled already).
    foreach entry $localList {
        if {$entry == "!size"} continue          ;# !size already synchronized
        set matchIndex [lsearch -exact $remoteList $entry]
        if {$matchIndex == -1} {                  ;# entry not found in remote
            set localData [readini $localId $section $entry]
            writeini $remoteId $section $entry $localData
        }
    }       
}

